@@ -1,6 +1,4 @@
 #' @title Comprehensive Luminescence Dating Data Analysis\cr
-#'
-#'
 #' \if{html}{
 #' \figure{Luminescence_logo.png}{options: width="75" alt="r-luminescence.org"}
 #' }
@@ -14,8 +12,6 @@
 #'
 #' @aliases Luminescence-package Luminescence
 #'
-#' @docType package
-#'
 #' @details
 #'
 #' **Supervisor of the initial version in 2012**
@@ -24,53 +20,54 @@
 #'
 #' **Support contact**
 #'
-#' - \email{developers@@r-luminescence.org}
-#' - [https://github.com/R-Lum/Luminescence/discussions]()
+#' * \email{developers@@r-luminescence.org}
+#' * [https://github.com/R-Lum/Luminescence/discussions]()
 #'
 #' **Bug reporting**
 #'
-#' - \email{developers@@r-luminescence.org} or
-#' - [https://github.com/R-Lum/Luminescence/issues]()
+#' * \email{developers@@r-luminescence.org} or
+#' * [https://github.com/R-Lum/Luminescence/issues]()
 #'
 #' **Project website**
 #'
-#' - [https://r-luminescence.org]()
+#' * [https://r-luminescence.org]()
 #'
 #' **Project source code repository**
 #'
-#' - [https://github.com/R-Lum/Luminescence]()
+#' * [https://github.com/R-Lum/Luminescence]()
 #'
 #' **Related package projects**
 #'
-#' - [https://cran.r-project.org/package=RLumShiny]()
-#' - [https://cran.r-project.org/package=RLumModel]()
-#' - [https://cran.r-project.org/package=RLumCarlo]()
-#'
-#' **Package maintainer**
-#'
-#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany),\cr
-#' \email{sebastian.kreutzer@@aber.ac.uk}
+#' * [https://cran.r-project.org/package=RLumShiny]()
+#' * [https://cran.r-project.org/package=RLumModel]()
+#' * [https://cran.r-project.org/package=RLumCarlo]()
+#' * [https://cran.r-project.org/package=RCarb]()
 #'
 #' **Funding**
 #'
-#' 2011-2013:  The initial version of the package was developed, while Sebastian Kreutzer
+#' * 2011-2013:  The initial version of the package was developed, while Sebastian Kreutzer
 #' was funded through the DFG programme "Rekonstruktion der Umweltbedingungen
 #' des Spätpleistozäns in Mittelsachsen anhand von Löss-Paläobodensequenzen"
 #' (DFG id: 46526743)
 #'
-#' 2014-2018: Cooperation and personal exchange between the developers is gratefully
+#' * 2014-2018: Cooperation and personal exchange between the developers is gratefully
 #' funded by the DFG (SCHM 3051/3-1) in the framework of the program
 #' "Scientific Networks". Project title: "RLum.Network: Ein
 #' Wissenschaftsnetzwerk zur Analyse von Lumineszenzdaten mit R" (2014-2018)
 #'
-#' 05/2014-12/2019: The work of Sebastian Kreutzer as maintainer of the package was supported
+#' * 05/2014-12/2019: The work of Sebastian Kreutzer as maintainer of the package was supported
 #' by LabEx LaScArBx (ANR - n. ANR-10-LABX-52).
 #'
-#' 01/2020-04/2022: Sebastian Kreutzer as maintainer of the package has received funding
+#' * 01/2020-04/2022: Sebastian Kreutzer as maintainer of the package has received funding
 #' from the European Union’s Horizon 2020 research and innovation programme under
 #' the Marie Skłodowska-Curie grant agreement No 844457 (CREDit), and could continue
 #' maintaining the package.
 #'
+#' * since 03/2023: Sebastian Kreutzer as maintainer of the package receives funding from the
+#' DFG Heisenberg programme No 505822867.
+#'
+#' * All other authors gratefully received additional funding from various public funding bodies.
+#'
 #' @references
 #' Dietze, M., Kreutzer, S., Fuchs, M.C., Burow, C., Fischer, M.,
 #' Schmidt, C., 2013. A practical guide to the R package Luminescence.
@@ -110,17 +107,14 @@
 #' @import utils methods data.table
 #'
 #' @importFrom  Rcpp evalCpp
-#'
-#' @importFrom graphics plot plot.default frame abline mtext text lines par layout lines arrows axTicks axis barplot box boxplot contour curve grconvertX grconvertY hist legend persp points polygon rug segments title grid close.screen screen split.screen
-#' @importFrom grDevices adjustcolor axisTicks colorRampPalette gray.colors rgb topo.colors xy.coords dev.off
-#' @importFrom stats formula approx as.formula complete.cases density dnorm glm lm median na.exclude na.omit nls nls.control pchisq pnorm quantile rnorm runif sd smooth smooth.spline spline t.test uniroot var weighted.mean setNames coef confint predict update residuals fitted qf
-#' @importFrom parallel parLapply makeCluster stopCluster
-#' @importFrom httr GET accept_json status_code content
+#' @importFrom graphics plot abline mtext text lines par layout lines axTicks axis curve hist legend points polygon segments title
+#' @importFrom grDevices adjustcolor rgb
+#' @importFrom stats approx density dnorm median na.exclude nls quantile rnorm runif sd uniroot setNames coef predict residuals
 #'
 #' @useDynLib Luminescence, .registration = TRUE
 #'
 #' @md
-NULL
+"_PACKAGE"
 
 #' Base data set of dose-rate conversion factors
 #'
@@ -401,20 +395,20 @@ NULL
 #'
 #' \deqn{y = 0.0002*x + 4.0914}
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #'
 #' ##load data
 #' data(BaseDataSet.CosmicDoseRate)
 #'
+#' @docType data
 #' @name BaseDataSet.CosmicDoseRate
 #' @aliases values.cosmic.Softcomp values.factor.Altitude values.par.FJH
 #'
 #' @md
 NULL
 
-
 #' @title Example data from a SAR OSL and SAR TL measurement for the package
 #' Luminescence
 #'
@@ -468,7 +462,7 @@ NULL
 #'
 #' @docType data
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #'
@@ -493,10 +487,10 @@ NULL
 #'
 #' @format Data frame with 1000 observations on the following 2 variables:
 #'
-#' \describe{
-#' \item{list("x")}{a numeric vector, time}
-#' \item{list("y")}{a numeric vector, counts}
-#' }
+#'
+#' - `list("x")`: a numeric vector, time
+#' - `list("y")`: a numeric vector, counts
+#'
 #'
 #' @references
 #' Baartman, J.E.M., Veldkamp, A., Schoorl, J.M., Wallinga, J.,
@@ -526,7 +520,7 @@ NULL
 #' Reference: \tab Bos & Wallinga (2012) and Baartman et al. (2011)
 #' }
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #'
@@ -595,6 +589,9 @@ NULL
 #' Material: \tab Beach deposit, coarse grain quartz measured on aluminium discs on a Risø TL/OSL DA-15 reader\cr
 #' }
 #'
+#'
+#' @keywords datasets internal
+#'
 #' @examples
 #'
 #' ##show LM data
@@ -624,6 +621,8 @@ NULL
 #' Material: \tab Middle grain (38-63 \eqn{\mu}m) quartz measured on a Risoe TL/OSL DA-15 reader.
 #' }
 #'
+#' @keywords datasets internal
+#'
 #' @examples
 #'
 #' ## plot Lx/Tx data vs dose [s]
@@ -647,6 +646,8 @@ NULL
 #' @source
 #' Arbitrary OSL measurement.
 #'
+#' @keywords datasets internal
+#'
 #' @examples
 #'
 #' ##load data
@@ -661,6 +662,32 @@ NULL
 #' @md
 NULL
 
+#' Example equivalent dose data from mortar samples
+#'
+#' Arbitrary data to test the function `calc_EED_Model`
+#'
+#' @format Two [`data.frame`]s containing De and De error
+#'
+#' @references unpublished data
+#'
+#' @source
+#' Arbitrary measurements.
+#'
+#' @keywords datasets internal
+#'
+#' @examples
+#'
+#' ##load data
+#' data(ExampleData.MortarData, envir = environment())
+#'
+#' ##plot data
+#' plot(MortarData)
+#'
+#' @name ExampleData.MortarData
+#' @aliases MortarData
+#' @md
+NULL
+
 #' Example data as [RLum.Analysis-class] objects
 #'
 #' Collection of different [RLum.Analysis-class] objects for
@@ -694,7 +721,7 @@ NULL
 #' Reference: \tab Kreutzer et al. (2014)
 #' }
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #'
@@ -813,7 +840,7 @@ NULL
 #' Heating: \tab 1 K/s, up to 500 deg. C
 #' }
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #' ##show data
@@ -918,7 +945,7 @@ NULL
 #' ##(2) plot values as histogram (with second to gray conversion)
 #' data(ExampleData.DeValues, envir = environment())
 #'
-#' De.values <- Second2Gray(ExampleData.DeValues$BT998,
+#' De.values <- convert_Second2Gray(ExampleData.DeValues$BT998,
 #'                          dose.rate = c(0.0438, 0.0019))
 #'
 #'
@@ -1007,7 +1034,7 @@ NULL
 #' IR50_De.LxTx <- ExampleData.Fading$equivalentDose.data$IR50
 #'
 #' ## Calculate the De of the IR50 signal
-#' IR50_De <- plot_GrowthCurve(IR50_De.LxTx,
+#' IR50_De <- fit_DoseResponseCurve(IR50_De.LxTx,
 #'                                 mode = "interpolation",
 #'                                 fit.method = "EXP")
 #'
@@ -1207,7 +1234,7 @@ NULL
 #' Environmental Dose Rate Determination Using a Passive Dosimeter: Techniques and Workflow for alpha-Al2O3:C Chips.
 #' Geochronometria 45, 56–67.
 #'
-#' @keywords datasets
+#' @keywords datasets internal
 #'
 #' @examples
 #'
@@ -1314,6 +1341,3 @@ NULL
 #' @name extdata
 #' @md
 NULL
-
-
-
