@@ -1,6 +1,3 @@
-#' @include get_RLum.R set_RLum.R length_RLum.R structure_RLum.R names_RLum.R smooth_RLum.R
-NULL
-
 #' Class `"RLum.Analysis"`
 #'
 #' Object class to represent analysis data for protocol analysis, i.e. all curves,
@@ -25,7 +22,7 @@ NULL
 #' @section Objects from the Class:
 #' Objects can be created by calls of the form `set_RLum("RLum.Analysis", ...)`.
 #'
-#' @section Class version: 0.4.16
+#' @section Class version: 0.4.18
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -96,10 +93,7 @@ setAs("list", "RLum.Analysis",
 
 setAs("RLum.Analysis", "list",
       function(from){
-        lapply(1:length(from@records), function(x){
-          from@records[[x]]
-
-        })
+        lapply(from@records, function(x) x)
       })
 
 
@@ -116,8 +110,7 @@ setMethod("show",
             ##print
             cat("\n [RLum.Analysis-class]")
 
-            ##show slot originator, for compatibly reasons with old example data, here
-            ##a check
+            ## show originator, for compatibility reasons with old example data
             if(.hasSlot(object, "originator")){cat("\n\t originator:", paste0(object@originator,"()"))}
 
             cat("\n\t protocol:", object@protocol)
@@ -173,15 +166,12 @@ setMethod("show",
                         } else if (linebreak) {
                           last <- "\n\t .. .. : "
                           assign(x = "linebreak", value = FALSE, envir = env)
-
                         }
-
                       }
                       return(paste0(first,last))
 
                     }else{
                       return("")
-
                     }
 
               }, FUN.VALUE = vector(mode = "character", length = 1))
@@ -195,17 +185,13 @@ setMethod("show",
 
                  } else{
                    cat("\n\t .. .. : ", terminal_output, sep = "")
-
                  }
-
               })
 
             }else{
               cat("\n\t >> This is an empty object, which cannot be used for further analysis! <<")
-
             }
-
-
+            cat("\n")
           }
 )##end show method
 
@@ -259,12 +245,12 @@ setMethod(
     ##allow self set to reset an RLum.Analysis object
     if(inherits(records, "RLum.Analysis")){
       #fill slots (this is much faster than the old code!)
-      newRLumAnalysis@protocol <- if(missing(protocol)){records@protocol}else{protocol}
+      newRLumAnalysis@protocol <- if(missing(protocol)) records@protocol else protocol
       newRLumAnalysis@originator <- originator
       newRLumAnalysis@records <- records@records
-      newRLumAnalysis@info <- if(missing(info)){records@info}else{c(records@info, info)}
+      newRLumAnalysis@info <- if(missing(info)) records@info else c(records@info, info)
       newRLumAnalysis@.uid <- .uid
-      newRLumAnalysis@.pid <- if(missing(.pid)){records@.uid}else{.pid}
+      newRLumAnalysis@.pid <- if(missing(.pid)) records@.uid else .pid
 
     }else{
       #fill slots (this is much faster than the old code!)
@@ -274,11 +260,9 @@ setMethod(
       newRLumAnalysis@info <- info
       newRLumAnalysis@.uid <- .uid
       newRLumAnalysis@.pid <- .pid
-
     }
 
     return(newRLumAnalysis)
-
   }
 )
 
@@ -295,7 +279,7 @@ setMethod(
 #' The selection of a specific RLum.type object superimposes the default selection.
 #' Currently supported objects are: RLum.Data.Curve and RLum.Data.Spectrum
 #'
-#' @param object [`get_RLum`]: [`names_RLum`], [`length_RLum`], [`structure_RLum`] (**required**):
+#' @param object (**required**):
 #' an object of class [RLum.Analysis-class]
 #'
 #' @param record.id [`get_RLum`]: [numeric] or [logical] (*optional*):
@@ -316,10 +300,10 @@ setMethod(
 #' return a numeric vector with the index of each element in the RLum.Analysis object.
 #'
 #' @param recursive [`get_RLum`]: [logical] (*with default*):
-#' if `TRUE` (the default) and the result of the `get_RLum()` request is a single
-#' object this object will be unlisted, means only the object itself and no
-#' list containing exactly one object is returned. Mostly this makes things
-#' easier, however, if this method is used within a loop this might be undesired.
+#' if `TRUE` (default) when the result of the `get_RLum()` request is a single
+#' object, the object itself will be returned directly, rather than being
+#' wrapped in a list. Mostly this makes things easier, but this might be
+#' undesired if this method is used within a loop.
 #'
 #' @param drop [`get_RLum`]: [logical] (*with default*):
 #' coerce to the next possible layer (which are `RLum.Data`-objects),
@@ -351,36 +335,32 @@ setMethod(
 #' @export
 setMethod("get_RLum",
           signature = ("RLum.Analysis"),
-
           function(object, record.id = NULL, recordType = NULL, curveType = NULL, RLum.type = NULL,
-                   protocol = "UNKNOWN", get.index = NULL, drop = TRUE, recursive = TRUE, info.object = NULL, subset = NULL, env = parent.frame(2)) {
+                   protocol = "UNKNOWN", get.index = NULL, drop = TRUE, recursive = TRUE,
+                   info.object = NULL, subset = NULL, env = parent.frame(2)) {
+            .set_function_name("get_RLum")
+            on.exit(.unset_function_name(), add = TRUE)
 
             if (!is.null(substitute(subset))) {
-
               # To account for different lengths and elements in the @info slot we first
               # check all unique elements (in all records)
               info_el <- unique(unlist(lapply(object@records, function(el) names(el@info))))
 
               envir <- as.data.frame(do.call(rbind,
-                                             lapply(object@records, function(el) {
-                                               val <- c(curveType = el@curveType, recordType = el@recordType, unlist(el@info))
+                lapply(object@records, function(el) {
+                  val <- c(curveType = el@curveType, recordType = el@recordType, unlist(el@info))
 
-                                               # add missing info elements and set NA
-                                               if (any(!info_el %in% names(val))) {
-                                                 val_new <- setNames(rep(NA, length(info_el[!info_el %in% names(val)])), info_el[!info_el %in% names(val)])
-                                                 val <- c(val, val_new)
+                  # add missing info elements and set NA
+                  if (any(!info_el %in% names(val))) {
+                    val_new <- setNames(
+                      rep("",length(info_el[!info_el %in% names(val)])), info_el[!info_el %in% names(val)])
+                    val <- c(val, val_new)
+                  }
 
-                                               }
-
-                                               # order the named char vector by its names so we don't mix up the columns
-                                               val <- val[order(names(val))]
-                                               return(val)
-                                             })
-              ), stringsAsFactors = FALSE)
-
-              ##check for a logical expression, to avoid problems afterwards
-              # if(class(eval(substitute(subset), envir = envir, enclos = env))[1] != "logical")
-              #   stop("[get_RLum] The argument 'subset' does not contain a logical expression!", call. = FALSE)
+                 # order the named char vector by its names so we don't mix up the columns
+                 val <- val[order(names(val))]
+                 return(val)
+               })), stringAsFactors = FALSE)
 
               ##select relevant rows
               sel <- tryCatch(eval(
@@ -389,10 +369,14 @@ setMethod("get_RLum",
                 enclos = env
               ),
               error = function(e) {
-                stop("[get_RLum()] Invalid subset options. \nValid terms are: ", paste(names(envir), collapse = ", "),
-                     call. = FALSE)
+                .throw_error("Invalid subset expression, valid terms are: ",
+                             .collapse(names(envir)))
               })
 
+              if (!is.logical(sel)) {
+                .throw_error("'subset' must contain a logical expression")
+              }
+
               if (all(is.na(sel)))
                 sel <- FALSE
 
@@ -400,12 +384,13 @@ setMethod("get_RLum",
                 object@records <- object@records[sel]
                 return(object)
               } else {
-                tmp <- mapply(function(name, op) { message("  ",name, ": ", paste(unique(op),  collapse = ", ")) },
-                              names(envir), envir)
-                message("\n [get_RLum()] Invalid value, please refer to unique options given above.")
+                tmp <- mapply(function(name, op) {
+                  message("  ", name, ": ", .collapse(unique(op), quote = FALSE))
+                }, names(envir), envir)
+                .throw_message("'subset' expression produced an ",
+                               "empty selection, NULL returned")
                 return(NULL)
               }
-
             }
 
             ##if info.object is set, only the info objects are returned
@@ -416,17 +401,11 @@ setMethod("get_RLum",
               }else{
                 ##check for entries
                 if(length(object@info) == 0){
-                  warning(
-                    "[get_RLum()] This RLum.Analysis object has no info objects! NULL returned!)",
-                          call. = FALSE)
-
+                  .throw_warning("This 'RLum.Analysis' object has no info ",
+                                 "objects, NULL returned")
                 }else{
-                  warning(paste0(
-                    "[get_RLum()] Invalid info.object name. Valid names are: ",
-                    paste(names(object@info), collapse = ", ")
-                  ),
-                  call. = FALSE)
-
+                  .throw_warning("Invalid 'info.object' name, valid names are: ",
+                                 .collapse(names(object@info)))
                 }
                 return(NULL)
               }
@@ -435,34 +414,30 @@ setMethod("get_RLum",
 
               ##check for records
               if (length(object@records) == 0) {
-
-                warning("[get_RLum()] This RLum.Analysis object has no records! NULL returned!)")
+                .throw_warning("This 'RLum.Analysis' object has no records, ",
+                               "NULL returned")
                 return(NULL)
-
               }
 
               ##record.id
               if (is.null(record.id)) {
                 record.id <- c(1:length(object@records))
 
-              } else if (!is.numeric(record.id) &
-                         !is.logical(record.id)) {
-                stop("[get_RLum()] 'record.id' has to be of type 'numeric' or 'logical'!", call. = FALSE)
-
+              } else {
+                .validate_class(record.id, c("integer", "numeric", "logical"))
               }
               ##logical needs a slightly different treatment
               ##Why do we need this? Because a lot of standard R functions work with logical
-              ##values instead of numerical indicies
+              ##values instead of numerical indices
               if (is.logical(record.id)) {
                 record.id <- c(1:length(object@records))[record.id]
-
               }
 
               ##check if record.id exists
-              if (FALSE %in% (abs(record.id) %in% (1:length(object@records)))) {
-                try(stop("[get_RLum()] At least one 'record.id' is invalid!", call. = FALSE))
+              if (!all(abs(record.id) %in% (1:length(object@records)))) {
+                .throw_message("At least one 'record.id' is invalid, ",
+                               "NULL returned")
                 return(NULL)
-
               }
 
               ##recordType
@@ -471,9 +446,8 @@ setMethod("get_RLum",
                   unique(vapply(object@records, function(x)
                     x@recordType, character(1)))
 
-              } else if (!inherits(recordType, "character")){
-                  stop("[get_RLum()] 'recordType' has to be of type 'character'!", call. = FALSE)
-
+              } else {
+                .validate_class(recordType, "character")
               }
 
               ##curveType
@@ -483,36 +457,29 @@ setMethod("get_RLum",
                                                     object@records[[x]]@curveType
                                                   })))
 
-              } else if (!is(curveType, "character")) {
-                stop("[get_RLum()] 'curveType' has to be of type 'character'!", call. = FALSE)
-
+              } else {
+                .validate_class(curveType, "character")
               }
 
               ##RLum.type
               if (is.null(RLum.type)) {
                 RLum.type <- c("RLum.Data.Curve", "RLum.Data.Spectrum", "RLum.Data.Image")
-
-              } else if (!is(RLum.type, "character")) {
-                stop("[get_RLum()] 'RLum.type' has to be of type 'character'!", call. = FALSE)
-
+              } else {
+                .validate_class(RLum.type, "character")
               }
 
               ##get.index
               if (is.null(get.index)) {
                 get.index <- FALSE
 
-              } else if (!is(get.index, "logical")) {
-                stop("[get_RLum()] 'get.index' has to be of type 'logical'!", call. = FALSE)
-
+              } else {
+                .validate_logical_scalar(get.index)
               }
 
               ##get originator
+              originator <- NA_character_
               if (.hasSlot(object, "originator")) {
                 originator <- object@originator
-
-              } else{
-                originator <- NA_character_
-
               }
 
               ##-----------------------------------------------------------------##
@@ -521,30 +488,30 @@ setMethod("get_RLum",
               record.id <- 1:length(object@records)
 
               ##select curves according to the chosen parameter
-              if (length(record.id) > 1) {
+              if (length(record.id) >= 1) {
                 temp <- lapply(record.id, function(x) {
-                  if (is(object@records[[x]])[1] %in% RLum.type == TRUE) {
-                    ##as input a vector is allowed
+                  if (is(object@records[[x]])[1] %in% RLum.type) {
+                      ##as input a vector is allowed
                     temp <- lapply(1:length(recordType), function(k) {
                       ##translate input to regular expression
                       recordType[k] <- glob2rx(recordType[k])
                       recordType[k] <- substr(recordType[k], start = 2, stop = nchar(recordType[k]) - 1)
 
-                      if (grepl(recordType[k], object@records[[x]]@recordType) == TRUE &
+                      ##handle NA
+                      if(is.na(object@records[[x]]@recordType))
+                        recordType_comp <- "NA"
+                      else
+                        recordType_comp <- object@records[[x]]@recordType
+
+                      ## get the results object and if requested, get the index
+                      if (grepl(recordType[k], recordType_comp) &
                           object@records[[x]]@curveType %in% curveType) {
-                        if (!get.index) {
-                          object@records[[x]]
-
-                        } else{
-                          x
-                        }
-
+                        if (!get.index) object@records[[x]] else x
                       }
-
                     })
 
                     ##remove empty entries and select just one to unlist
-                    temp <- temp[!sapply(temp, is.null)]
+                    temp <- temp[!vapply(temp, is.null,logical(1))]
 
                     ##if list has length 0 skip entry
                     if (length(temp) != 0) {
@@ -552,18 +519,15 @@ setMethod("get_RLum",
                     } else{
                       temp <- NULL
                     }
-
                   }
-
                 })
 
-
                 ##remove empty list element
-                temp <- temp[!sapply(temp, is.null)]
+                temp <- temp[!vapply(temp, is.null, logical(1))]
 
                 ##check if the produced object is empty and show warning message
                 if (length(temp) == 0)
-                  warning("[get_RLum()] This request produced an empty list of records!", call. = FALSE)
+                  .throw_warning("This request produced an empty list of records")
 
                 ##remove list for get.index
                 if (get.index) {
@@ -586,15 +550,16 @@ setMethod("get_RLum",
 
                     } else{
                       return(temp)
-
                     }
-
                   }
-
                 }
 
               } else{
-                if (get.index == FALSE) {
+                ## FIXME(mcol): this block seems unreachable : as before the
+                ## `if` block we set `record.id <- 1:length(object@records)`,
+                ## it can never happen that `length(record.id) < 1`
+                # nocov start
+                if (!get.index[1]) {
                   if (drop == FALSE) {
                     ##needed to keep the argument drop == TRUE
                     temp <- set_RLum(
@@ -608,18 +573,14 @@ setMethod("get_RLum",
 
                   } else{
                     return(object@records[[record.id]])
-
                   }
 
-
                 } else{
                   return(record.id)
-
                 }
+                # nocov end
               }
-
             }
-
           })
 
 # structure_RLum() ----------------------------------------------------------------------------
@@ -628,7 +589,7 @@ setMethod("get_RLum",
 #' Method to show the structure of an [RLum.Analysis-class] object.
 #'
 #' @param fullExtent [structure_RLum]; [logical] (*with default*):
-#' extents the returned `data.frame` to its full extent, i.e. all info elements
+#' extends the returned `data.frame` to its full extent, i.e. all info elements
 #' are part of the return as well. The default value is `FALSE` as the data
 #' frame might become rather big.
 #'
@@ -643,17 +604,18 @@ setMethod("get_RLum",
 setMethod("structure_RLum",
           signature= "RLum.Analysis",
           definition = function(object, fullExtent = FALSE) {
+            .set_function_name("structure_RLum")
+            on.exit(.unset_function_name(), add = TRUE)
 
             ##check if the object containing other elements than allowed
             if(!all(vapply(object@records, FUN = class, character(1)) == "RLum.Data.Curve"))
-              stop("[structure_RLum()]  Only 'RLum.Data.Curve' objects are allowed!",
-                   call. = FALSE)
+              .throw_error("Only 'RLum.Data.Curve' objects are allowed")
 
             ##get length object
             temp.object.length <- length(object@records)
 
             ##ID
-            temp.id <- 1:temp.object.length
+            temp.id <- seq_along(object@records)
 
             ##recordType
             temp.recordType <-
@@ -665,27 +627,24 @@ setMethod("structure_RLum",
             temp.protocol.step <- c(NA)
             length(temp.protocol.step) <- temp.object.length
 
-            ##n.channels
-            temp.n.channels <- vapply(object@records, function(x){length(x@data[,1])}, numeric(1))
+            ## GET LIMITS
+            temp.limits <- t(vapply(object@records, function(x) {
+              c(nrow(x@data), min(x@data[, 1]), max(x@data[, 1]), min(x@data[, 2]), max(x@data[, 2]))
+            }, numeric(5)))
 
-            ##X.MIN
-            temp.x.min <- vapply(object@records, function(x){min(x@data[,1])}, numeric(1))
-
-            ##X.MAX
-            temp.x.max <- vapply(object@records, function(x){max(x@data[,1])}, numeric(1))
-
-            ##y.MIN
-            temp.y.min <- vapply(object@records, function(x){min(x@data[,2])}, numeric(1))
-
-            ##X.MAX
-            temp.y.max <- vapply(object@records, function(x){max(x@data[,2])}, numeric(1))
+            temp.n.channels <- temp.limits[, 1]
+            temp.x.min <- temp.limits[, 2]
+            temp.x.max <- temp.limits[, 3]
+            temp.y.min <- temp.limits[, 4]
+            temp.y.max <- temp.limits[, 5]
 
             ##.uid
             temp.uid <- unlist(lapply(object@records, function(x){x@.uid}))
 
             ##.pid
-            temp.pid <- paste(
-              unlist(lapply(object@records, function(x){x@.pid})), collapse = ", ")
+            temp.pid <- unlist(lapply(object@records, function(x){x@.pid}))
+            if (length(temp.pid) > 1)
+              temp.pid <- paste(temp.pid, collapse = ", ")
 
             ##originator
             temp.originator <- unlist(lapply(object@records, function(x){x@originator}))
@@ -699,17 +658,21 @@ setMethod("structure_RLum",
                 x@info
               }), fill = TRUE))
 
+              if (nrow(temp.info.elements) == 0) {
+                ## if we are here temp.info.elements 0 rows and 0 columns:
+                ## to avoid crashing further down in the data.frame() call,
+                ## we create a data frame with the expected number of rows
+                temp.info.elements <- data.frame(info = rep(NA, temp.object.length))
+              }
             } else{
               temp.info.elements <-
-                unlist(sapply(1:temp.object.length, function(x) {
-                  if (length(object@records[[x]]@info) != 0) {
-                    do.call(paste, as.list(names(object@records[[x]]@info)))
+                unlist(lapply(object@records, function(x) {
+                  if (length(x@info) != 0) {
+                    paste(names(x@info), collapse = " ")
                   } else{
                     NA
                   }
-
                 }))
-
             }
 
             ##combine output to a data.frame
@@ -731,7 +694,6 @@ setMethod("structure_RLum",
                 stringsAsFactors = FALSE
               )
             )
-
           })
 
 
@@ -751,12 +713,12 @@ setMethod("length_RLum",
           "RLum.Analysis",
           function(object){
             length(object@records)
-
           })
 
 # names_RLum() --------------------------------------------------------------------------------
 #' @describeIn RLum.Analysis
-#' Returns the names of the [RLum.Data-class] objects objects (same as shown with the show method)
+#' Returns the names of the [RLum.Data-class] objects (same as shown with the
+#' `show` method)
 #'
 #' @return
 #'
@@ -769,9 +731,105 @@ setMethod("length_RLum",
 setMethod("names_RLum",
           "RLum.Analysis",
           function(object){
-            sapply(1:length(object@records), function(x){
-              object@records[[x]]@recordType})
+            sapply(object@records, function(x) x@recordType)
+          })
 
+
+## add_metadata() -----------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Adds metadata to [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to add
+#'
+#' @param value (**required**) The value assigned to the selected element
+#' of the metadata field.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("add_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, value) {
+            .set_function_name("add_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## add the metadata to all records
+            records <- lapply(object@records, function(x) {
+              do.call(`add_metadata<-`,
+                      list(x, info_element = info_element, value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
+          })
+
+## rename_metadata() --------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Renames a metadata entry of [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to rename
+#'
+#' @param value (**required**) The value assigned to the selected element
+#' of the `info` slot.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("rename_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, value) {
+            .set_function_name("rename_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## rename the metadata field in all records
+            records <- lapply(object@records, function(x) {
+              do.call(`rename_metadata<-`,
+                      list(x, info_element = info_element, value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
+          })
+
+## replace_metadata() -------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Replaces or removes metadata of [RLum.Analysis-class] objects
+#'
+#' @param info_element [character] (**required**) name of the metadata field
+#' to replace or remove
+#'
+#' @param value (**required**) The value assigned to the selected elements
+#' of the metadata field. If `NULL` the elements named in `info_element`
+#' will be removed.
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("replace_metadata<-",
+          signature= "RLum.Analysis",
+          definition = function(object, info_element, subset = NULL, value) {
+            .set_function_name("replace_metadata")
+            on.exit(.unset_function_name(), add = TRUE)
+
+            ## this must be evaluated now, as inside the lapply() inner
+            ## function it would be evaluated differently
+            subset.expr <- substitute(subset)
+
+            ## replace the metadata in all records
+            records <- lapply(object@records, function(x) {
+              do.call(`replace_metadata<-`,
+                      list(x, info_element = info_element,
+                           subset = subset.expr,  verbose = FALSE,
+                           value = value))
+            })
+
+            object@records <- records
+            assign(x = deparse(substitute(object))[1], object)
           })
 
 
@@ -779,8 +837,7 @@ setMethod("names_RLum",
 #' @describeIn RLum.Analysis
 #'
 #' Smoothing of `RLum.Data` objects contained in this `RLum.Analysis` object
-#' [zoo::rollmean] or [zoo::rollmedian][zoo::rollmean]. In particular the internal
-#' function `.smoothing` is used.
+#' using the internal function `.smoothing`.
 #'
 #' @param ... further arguments passed to underlying methods
 #'
@@ -798,10 +855,150 @@ setMethod(
   function(object, ...) {
         object@records <- lapply(object@records, function(x){
           smooth_RLum(x, ...)
-
         })
 
     return(object)
-
   }
 )
+
+
+## sort_RLum() --------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#'
+#' Sorting of `RLum.Data` objects contained in this `RLum.Analysis` object.
+#' At least one of `slot` and `info_element` must be provided. If both are
+#' given, ordering by `slot` always supersedes ordering by `info_element`.
+#'
+#' @param slot [character] (*optional*): slot name to use in sorting.
+#'
+#' @param info_element [character] (*optional*): name of the `info` field
+#' to use in sorting.
+#'
+#' @param decreasing [logical] (*with default*): whether the sort order should
+#' be decreasing (`FALSE` by default).
+#'
+#' @param ... further arguments passed to underlying methods
+#'
+#' @return
+#'
+#' **`sort_RLum`**
+#'
+#' Same object as input, but sorted according to the specified parameters.
+#'
+#' @md
+#' @export
+setMethod(
+  f = "sort_RLum",
+  signature = "RLum.Analysis",
+  function(object, slot = NULL, info_element = NULL,
+           decreasing = FALSE, ...) {
+    .set_function_name("sort_RLum")
+    on.exit(.unset_function_name(), add = TRUE)
+
+    ## an empty object has nothing to sort
+    if (length(object) == 0)
+      return(object)
+
+    ## input validation
+    sort.by.slot <- FALSE
+    if (!is.null(slot)) {
+      .validate_class(slot, "character", extra = "NULL")
+      valid.names <- slotNames(object@records[[1]])
+      if (any(!slot %in% valid.names)) {
+        .throw_error("Invalid 'slot' name, valid names are: ",
+                     .collapse(valid.names))
+      }
+      if (length(slot) > 1) {
+        message("[sort_RLum()]: Only the first field will be used in sorting")
+        slot <- slot[1]
+      }
+      sort.by.slot <- TRUE
+    }
+    if (!is.null(info_element)) {
+      .validate_class(info_element, "character", extra = "NULL")
+      valid.names <- names(object@records[[1]]@info)
+      if (any(!info_element %in% valid.names)) {
+        .throw_error("Invalid 'info_element' name, valid names are: ",
+                     .collapse(valid.names))
+      }
+      if (length(info_element) > 1) {
+        message("[sort_RLum()]: Only the first field will be used in sorting")
+        info_element <- info_element[1]
+      }
+    }
+    if (is.null(slot) && is.null(info_element)) {
+      .throw_error("At least one of 'slot' and 'info_element' should not be NULL")
+    }
+    .validate_logical_scalar(decreasing)
+
+    ## extract the values from the records according to which the sorting is
+    ## done: ordering by slot always supersedes ordering by info_element
+    vals <- if (sort.by.slot) {
+              sapply(object@records, function(x) slot(x, slot))
+            } else {
+              sapply(object@records, function(x) slot(x, "info")[[info_element]])
+            }
+
+    ## determine the new ordering if possible
+    tryCatch(ord <- order(vals, decreasing = decreasing),
+             error = function(e) {
+               .throw_error("Records could not be sorted according to ",
+                            ifelse(sort.by.slot,
+                                   paste0("slot = '", slot, "'"),
+                                   paste0("info_element = '", info_element, "'")))
+             })
+
+    ## return reordered object
+    return(object[ord])
+  }
+)
+
+# melt_RLum() -------------------------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#' Melts [RLum.Analysis-class] objects into a flat data.frame to be used
+#' in combination with other packages such as `ggplot2`.
+#'
+#' @return
+#'
+#' **`melt_RLum`**
+#'
+#' Flat [data.frame] with `X`, `Y`, `TYPE`, `UID`
+#'
+#' @md
+#' @export
+setMethod(
+  f = "melt_RLum",
+  signature = "RLum.Analysis",
+  function(object) {
+    melt_RLum(object@records)
+  }
+)
+
+## view() -------------------------------------------------------------------
+#' @describeIn RLum.Analysis
+#'
+#' View method for [RLum.Analysis-class] objects
+#'
+#' @param ... other arguments that might be passed
+#'
+#' @keywords internal
+#'
+#' @md
+#' @export
+setMethod("view",
+          signature = "RLum.Analysis",
+          definition = function(object, ...) {
+
+    ## set title
+    name <- list(...)$title
+    if (is.null(name))
+      name <- deparse(substitute(object))
+
+    ## collect info lists from all records
+    info <- lapply(seq_along(object@records),
+                   function(x) c(aliquot = x, object@records[[x]]@info))
+    info <- rbindlist(info, fill = TRUE)
+
+    ## run view
+    .view(x = info, title = name)
+})
