@@ -20,11 +20,11 @@
 #'
 #' \tabular{lll}{
 #' **ARGUMENT** \tab **FUNCTION** \tab **DESCRIPTION**\cr
-#' `mode` \tab `plot_GrowthCurve` \tab as in [plot_GrowthCurve]; sets the mode used for fitting\cr
-#' `fit.method` \tab `plot_GrowthCurve` \tab as in [plot_GrowthCurve]; sets the function applied for fitting\cr
+#' `mode` \tab `fit_DoseResponseCurve` \tab as in [fit_DoseResponseCurve]; sets the mode used for fitting\cr
+#' `fit.method` \tab `fit_DoseResponseCurve` \tab as in [fit_DoseResponseCurve]; sets the function applied for fitting\cr
 #' }
 #'
-#' @param object [RLum.Analysis-class] or [list] **(required)**:
+#' @param object [RLum.Analysis-class] or [list] (**required**):
 #' results obtained from the measurement.
 #' Alternatively a list of [RLum.Analysis-class] objects can be provided to allow an automatic analysis
 #'
@@ -45,10 +45,10 @@
 #' See details for further explanations
 #'
 #' @param verbose [logical] (*with default*):
-#' enable/disable verbose mode
+#' enable/disable output to the terminal.
 #'
 #' @param plot [logical] (*with default*):
-#' enable/disable plot output
+#' enable/disable the plot output.
 #'
 #' @param ... further arguments that can be passed to the plot output
 #'
@@ -68,7 +68,7 @@
 #'  `$data` \tab `data.frame` \tab correction value and error \cr
 #'  `$table` \tab `data.frame` \tab table used for plotting  \cr
 #'  `$table_mean` \tab `data.frame` \tab table used for fitting \cr
-#'  `$fit` \tab `lm` or `nls` \tab the fitting as returned by the function [plot_GrowthCurve]
+#'  `$fit` \tab `lm` or `nls` \tab the fitting as returned by the function [fit_DoseResponseCurve]
 #' }
 #'
 #'**slot:** **`@info`**
@@ -85,7 +85,7 @@
 #'
 #' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
-#' @seealso [plot_GrowthCurve]
+#' @seealso [fit_DoseResponseCurve]
 #'
 #' @references
 #'
@@ -114,32 +114,21 @@ analyse_Al2O3C_ITC <- function(
   verbose = TRUE,
   plot = TRUE,
   ...
-){
-
+) {
+  .set_function_name("analyse_Al2O3C_ITC")
+  on.exit(.unset_function_name(), add = TRUE)
 
   # SELF CALL -----------------------------------------------------------------------------------
   if(is.list(object)){
     ##check whether the list contains only RLum.Analysis objects
-    if(!all(unique(sapply(object, class)) == "RLum.Analysis")){
-      stop("[analyse_Al2O3C()] All objects in the list provided by 'objects' need to be of type 'RLum.Analysis'",
-           call. = FALSE)
+    lapply(object,
+           function(x) .validate_class(x, "RLum.Analysis",
+                                       name = "All elements of 'object'"))
 
-    }
-
-    ##expand input arguments
-    if(!is.null(signal_integral)){
-      signal_integral <- rep(list(signal_integral, length = length(object)))
-
-    }
-
-    ##dose points
-    if(is(dose_points, "list")){
-      dose.points <- rep(dose_points, length = length(object))
-
-    }else{
-      dose_points <- rep(list(dose_points), length = length(object))
-
-    }
+    ## expand input arguments
+    rep.length <- length(object)
+    signal_integral <- .listify(signal_integral, rep.length)
+    dose_points <- .listify(dose_points, rep.length)
 
     ##method_control
     ##verbose
@@ -157,7 +146,7 @@ analyse_Al2O3C_ITC <- function(
         plot = plot,
         main = ifelse("main"%in% names(list(...)), list(...)$main, paste0("ALQ #",x)),
         ...
-      ))
+      ), outFile = stdout()) # redirect error messages so they can be silenced
 
       ##catch error
       if(inherits(results, "try-error")){
@@ -165,22 +154,16 @@ analyse_Al2O3C_ITC <- function(
 
       }else{
         return(results)
-
       }
-
     })
 
     ##return
     return(merge_RLum(results_full))
-
   }
 
-  # Integrity check  ---------------------------------------------------------------------------
-  ##check input object
-  if(!inherits(object, "RLum.Analysis")){
-    stop("[analyse_Al2O3C_ITC()] 'object' needs to be of type 'RLum.Analysis'", call. = FALSE)
+  ## Integrity tests  -------------------------------------------------------
 
-  }
+  .validate_class(object, "RLum.Analysis")
 
   ##TODO
   ##implement more checks ... if you find some time, somehow, somewhere
@@ -194,15 +177,14 @@ analyse_Al2O3C_ITC <- function(
   method_control_settings <- list(
     mode = "extrapolation",
     fit.method = "EXP"
-
   )
 
-    ##modify on request
-    if(!is.null(method_control)){
-      method_control_settings <- modifyList(x = method_control_settings, val = method_control)
-
-    }
-
+  ## modify on request
+  if (!is.null(method_control)) {
+    .validate_class(method_control, "list")
+    method_control_settings <- modifyList(x = method_control_settings,
+                                          val = method_control)
+  }
 
   ##dose points enhancement
   ##make sure that the dose_point is enhanced
@@ -217,14 +199,9 @@ analyse_Al2O3C_ITC <- function(
     ##check whether the input is valid, otherwise make it valid
     if(min(signal_integral) < 1 | max(signal_integral) > nrow(object[[1]][])){
       signal_integral <- c(1:nrow(object[[1]][]))
-      warning(
-        paste0(
-          "[analyse_Al2O3C_ITC()] Input for 'signal_integral' corrected to 1:", nrow(object[[1]][])
-        ),
-        call. = FALSE
-      )
-    }
-
+      .throw_warning("Input for 'signal_integral' corrected to 1:",
+                     max(signal_integral))
+   }
   }
 
   ##calculate curve sums, assuming the background
@@ -259,23 +236,19 @@ analyse_Al2O3C_ITC <- function(
 
 
   ##calculate GC
-  GC <- plot_GrowthCurve(
-    sample = df_mean,
+  GC <- fit_DoseResponseCurve(
+    object = df_mean,
     mode = method_control_settings$mode,
-    output.plotExtended = FALSE,
-    output.plot = FALSE,
     fit.method = method_control_settings$fit.method,
     verbose = FALSE
   )
 
-
   ##output
   if(verbose){
     cat("\n[analyse_Al2O3C_ITC()]\n")
     cat(paste0("\n Used fit:\t\t",method_control_settings$fit.method))
     cat(paste0("\n Time correction value:\t", round(GC$De$De,3), " \u00B1 ", round(GC$De$De.Error, 3)))
     cat("\n\n")
-
   }
 
 
@@ -291,7 +264,6 @@ analyse_Al2O3C_ITC <- function(
       legend.pos = "right",
       legend.text = "dose points",
       mtext = ""
-
     )
 
     ##modify list on request
@@ -363,7 +335,6 @@ analyse_Al2O3C_ITC <- function(
 
     ##add mtext
     mtext(side = 3, text = plot_settings$mtext)
-
   }
 
   # Output --------------------------------------------------------------------------------------
@@ -380,6 +351,4 @@ analyse_Al2O3C_ITC <- function(
     ),
     info = list(call = sys.call())
   ))
-
 }
-
