@@ -4,13 +4,13 @@
 #' including curve
 #' fitting on [RLum.Analysis-class] objects.
 #'
-#' @details To allow post-IR IRSL protocol (Thomsen et al., 2008) measurement analyses
-#' this function has been written as extended wrapper function for the function
-#' [analyse_SAR.CWOSL], facilitating an entire sequence analysis in
+#' @details To allow post-IR IRSL protocol (Thomsen et al., 2008) measurement
+#' analyses, this function has been written as extended wrapper for function
+#' [analyse_SAR.CWOSL], thus facilitating an entire sequence analysis in
 #' one run. With this, its functionality is strictly limited by the
-#' functionality of the function [analyse_SAR.CWOSL].
+#' functionality provided by  [analyse_SAR.CWOSL].
 #'
-#' **Defining the sequence structure **
+#' **Defining the sequence structure**
 #'
 #' The argument `sequence.structure` expects a shortened pattern of your sequence structure and was
 #' mainly introduced to ease the use of the function. For example: If your measurement data contains
@@ -21,13 +21,15 @@
 #'
 #' **If the input is a `list`**
 #'
-#' If the input is a list of RLum.Analysis-objects, every argument can be provided as list to allow
+#' If the input is a list of [RLum.Analysis-class] objects, every argument
+#' can be provided as list to allow
 #' for different sets of parameters for every single input element.
 #' For further information see [analyse_SAR.CWOSL].
 #'
 #' @param object [RLum.Analysis-class] or [list] of [RLum.Analysis-class] objects (**required**):
 #' input object containing data for analysis.
-#' If a [list] is provided the functions tries to iterate over the list.
+#' If a [list] is provided the functions tries to iterate over each element
+#' in the list.
 #'
 #' @param signal.integral.min [integer] (**required**):
 #' lower bound of the signal integral. Provide this value as vector for different
@@ -57,16 +59,17 @@
 #' preheat as none TL, remove the TL step.)
 #'
 #' @param plot [logical] (*with default*):
-#' enables or disables plot output.
+#' enable/disable the plot output.
 #'
-#' @param plot.single [logical] (*with default*):
-#' single plot output (`TRUE/FALSE`) to allow for plotting the results in single plot
-#' windows. Requires `plot = TRUE`.
+#' @param plot_singlePanels [logical] (*with default*):
+#' enable/disable plotting of the results in a single windows for each plot.
+#' Ignored if `plot = FALSE`.
 #'
-#' @param ... further arguments that will be passed to the function
-#' [analyse_SAR.CWOSL] and [plot_GrowthCurve]. Furthermore, the arguments `main` (headers), `log` (IRSL curves), `cex` (control
+#' @param ... further arguments that will be passed to
+#' [analyse_SAR.CWOSL] and [plot_GrowthCurve]. Furthermore, the
+#' arguments `main` (headers), `log` (IRSL curves), `cex` (control
 #' the size) and `mtext.outer` (additional text on the plot area) can be passed to influence the plotting. If the input
-#' is list, `main` can be passed as [vector] or [list].
+#' is a list, `main` can be passed as [vector] or [list].
 #'
 #' @return
 #' Plots (*optional*) and an [RLum.Results-class] object is
@@ -87,11 +90,11 @@
 #' Best graphical output can be achieved by using the function `pdf`
 #' with the following options:
 #'
-#' `pdf(file = "<YOUR FILENAME>", height = 15, width = 15)`
+#' `pdf(file = "<YOUR FILENAME>", height = 18, width = 18)`
 #'
-#' @section Function version: 0.2.4
+#' @section Function version: 0.2.5
 #'
-#' @author Sebastian Kreutzer, Geography & Earth Sciences, Aberystwyth University (United Kingdom)
+#' @author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @seealso [analyse_SAR.CWOSL], [calc_OSLLxTxRatio], [plot_GrowthCurve],
 #' [RLum.Analysis-class], [RLum.Results-class] [get_RLum]
@@ -129,9 +132,7 @@
 #'                                        function(x){sequence.structure + x}))
 #'
 #' object <-  sapply(1:length(sequence.structure), function(x){
-#'
 #'   object[[sequence.structure[x]]]
-#'
 #' })
 #'
 #' object <- set_RLum(class = "RLum.Analysis", records = object, protocol = "pIRIR")
@@ -146,14 +147,14 @@
 #'      fit.method = "EXP",
 #'      sequence.structure = c("TL", "pseudoIRSL1", "pseudoIRSL2"),
 #'      main = "Pseudo pIRIR data set based on quartz OSL",
-#'      plot.single = TRUE)
+#'      plot_singlePanels = TRUE)
 #'
 #'
 #' ##(3) Perform pIRIR analysis (for this example with quartz OSL data!)
 #' ## Alternative for PDF output, uncomment and complete for usage
 #' \dontrun{
 #' tempfile <- tempfile(fileext = ".pdf")
-#' pdf(file = tempfile, height = 15, width = 15)
+#' pdf(file = tempfile, height = 18, width = 18)
 #'   results <- analyse_pIRIRSequence(object,
 #'          signal.integral.min = 1,
 #'          signal.integral.max = 2,
@@ -176,61 +177,46 @@ analyse_pIRIRSequence <- function(
   dose.points = NULL,
   sequence.structure = c("TL", "IR50", "pIRIR225"),
   plot = TRUE,
-  plot.single = FALSE,
+  plot_singlePanels = FALSE,
   ...
-){
+) {
+  .set_function_name("analyse_pIRIRSequence")
+  on.exit(.unset_function_name(), add = TRUE)
 
 # SELF CALL -----------------------------------------------------------------------------------
  if(is.list(object)){
 
-    ##make live easy
+   lapply(object, function(x) {
+     .validate_class(x, "RLum.Analysis", name = "All elements of 'object'")
+   })
+
+    ## make life easy
     if(missing("signal.integral.min")){
       signal.integral.min <- 1
-      warning("[analyse_pIRIRSequence()] 'signal.integral.min' missing, set to 1", call. = FALSE)
+      .throw_warning("'signal.integral.min' missing, set to 1")
     }
 
     if(missing("signal.integral.max")){
       signal.integral.max <- 2
-      warning("[analyse_pIRIRSequence()] 'signal.integral.max' missing, set to 2", call. = FALSE)
+      .throw_warning("'signal.integral.max' missing, set to 2")
     }
 
+   ## expand input arguments
+   rep.length <- length(object)
 
-    ##now we have to extend everything to allow list of arguments ... this is just consequent
-    signal.integral.min <- rep(list(signal.integral.min), length = length(object))
-    signal.integral.max <- rep(list(signal.integral.max), length = length(object))
-    background.integral.min <- rep(list(background.integral.min), length = length(object))
-    background.integral.max <- rep(list(background.integral.max), length = length(object))
-    sequence.structure <- rep(list(sequence.structure), length = length(object))
+   signal.integral.min <- .listify(signal.integral.min, rep.length)
+   signal.integral.max <- .listify(signal.integral.max, rep.length)
+   background.integral.min <- .listify(background.integral.min, rep.length)
+   background.integral.max <- .listify(background.integral.max, rep.length)
+   sequence.structure <- .listify(sequence.structure, rep.length)
+   dose.points <- .listify(dose.points, rep.length)
 
-    if(!is.null(dose.points)){
-
-      if(is(dose.points, "list")){
-        dose.points <- rep(dose.points, length = length(object))
-
-      }else{
-        dose.points <- rep(list(dose.points), length = length(object))
-
-      }
-
-    }else{
-      dose.points <- rep(list(NULL), length(object))
-
-    }
-
-    ##main
-    if("main" %in% names(list(...))){
-      main_list <- rep(list(...)$main, length.out = length(object))
-
-      if(class(main_list) != "list"){
-        main_list <- as.list(main_list)
-
-      }
-
-    }
-
-    ##run analysis
-    temp <- lapply(1:length(object), function(x){
+   if ("main" %in% names(list(...))) {
+     main_list <- .listify(list(...)$main, rep.length)
+   }
 
+   ## run analysis
+   temp <- .warningCatcher(lapply(1:length(object), function(x) {
       analyse_pIRIRSequence(object[[x]],
                         signal.integral.min = signal.integral.min[[x]],
                         signal.integral.max = signal.integral.max[[x]],
@@ -239,11 +225,10 @@ analyse_pIRIRSequence <- function(
                         dose.points = dose.points[[x]],
                         sequence.structure = sequence.structure[[x]],
                         plot = plot,
-                        plot.single = plot.single,
+                        plot_singlePanels = plot_singlePanels,
                         main = ifelse("main"%in% names(list(...)), main_list[[x]], paste0("ALQ #",x)),
                         ...)
-
-    })
+    }))
 
     ##combine everything to one RLum.Results object as this as what was written ... only
     ##one object
@@ -252,65 +237,74 @@ analyse_pIRIRSequence <- function(
     results <- merge_RLum(temp)
 
     ##DO NOT use invisible here, this will stop the function from stopping
-    if(length(results) == 0){
+    if(length(results) == 0)
       return(NULL)
-
-    }else{
+    else
       return(results)
-
-    }
-
   }
 
+  ## Integrity checks -------------------------------------------------------
 
-# General Integrity Checks ---------------------------------------------------
+  .validate_class(object, "RLum.Analysis", extra = "'list'")
+  .validate_logical_scalar(plot)
+  .validate_logical_scalar(plot_singlePanels)
 
-  ##GENERAL
+  ## there must be at least an IR step
+  if (!any(grepl("IR", sequence.structure))) {
+    .throw_error("'sequence.structure' should contain at least one IR step")
+  }
 
-    ##MISSING INPUT
-    if(missing("object")==TRUE){
-      stop("[analyse_pIRIRSequence()] No value set for 'object'!")
-    }
+  ## check allowed values in sequence structure
+  temp.collect.invalid.terms <- .collapse(
+      sequence.structure[!grepl("TL",  sequence.structure) &
+                         !grepl("IR",  sequence.structure) &
+                         !grepl("OSL", sequence.structure) &
+                         !grepl("EXCLUDE", sequence.structure)])
 
-    ##INPUT OBJECTS
-    if(is(object, "RLum.Analysis")==FALSE){
-      stop("[analyse_pIRIRSequence()] Input object is not of type 'RLum.Analyis'!")
-    }
+  if (temp.collect.invalid.terms != "") {
+    .throw_error(temp.collect.invalid.terms,
+                 " not allowed in 'sequence.structure'")
+  }
 
-    ##CHECK ALLOWED VALUES IN SEQUENCE STRUCTURE
-    temp.collect.invalid.terms <- paste(sequence.structure[
-      (!grepl("TL",sequence.structure)) &
-      (!grepl("IR",sequence.structure)) &
-      (!grepl("OSL",sequence.structure)) &
-      (!grepl("EXCLUDE",sequence.structure))],
-      collapse = ", ")
-
-    if(temp.collect.invalid.terms != ""){
-      stop("[analyse_pIRIRSequence()] ",
-        temp.collect.invalid.terms, " not allowed in 'sequence.structure'!")
-    }
+  ## deprecated argument
+  if ("plot.single" %in% names(list(...))) {
+    plot_singlePanels <- list(...)$plot.single
+    .throw_warning("'plot.single' is deprecated, use 'plot_singlePanels' ",
+                   "instead")
+  }
 
+  ## Enforce a minimum plot device size: this is necessary as otherwise users
+  ## may experience "figure margins too large" errors when trying to draw all
+  ## plots on a single page. We need to round the device size values because
+  ## often they are values such as 15.99999999999 which would incorrectly
+  ## trigger our check
+  min.size <- 16
+  dev.size <- round(grDevices::dev.size("in"), 5)
+  if (plot && !plot_singlePanels && any(dev.size < min.size)) {
+    plot <- FALSE
+    msg <- paste0("Argument 'plot' reset to 'FALSE'. The smallest plot ",
+                  "size required is IN x IN in.\nConsider plotting via ",
+                  "`pdf(..., width = IN, height = IN)` ",
+                  "or setting `plot_singlePanels = TRUE`")
+    .throw_warning(gsub(x = msg, "IN", min.size))
+  }
 
 # Deal with extra arguments -------------------------------------------------------------------
+  ## default values
+  mtext.outer <- "MEASUREMENT INFO"
+  main <- ""
+  log <- ""
+  cex <- 0.7
 
   ##deal with addition arguments
   extraArgs <- list(...)
-
-  mtext.outer <- if("mtext.outer" %in% names(extraArgs)) {extraArgs$mtext.outer} else
-  {"MEASUREMENT INFO"}
-
-  main <- if("main" %in% names(extraArgs)) {extraArgs$main} else
-  {""}
-
-  log <- if("log" %in% names(extraArgs)) {extraArgs$log} else
-  {""}
-
-  cex <- if("cex" %in% names(extraArgs)) {extraArgs$cex} else
-  {.7}
+  mtext.outer <- if ("mtext.outer" %in% names(extraArgs)) extraArgs$mtext.outer
+  main <- if ("main" %in% names(extraArgs)) extraArgs$main
+  log <- if ("log" %in% names(extraArgs)) extraArgs$log
+  cex <- if ("cex" %in% names(extraArgs)) extraArgs$cex
 
 
 # Protocol Integrity Checks --------------------------------------------------
-
   ##(1) Check structure and remove curves that fit not the recordType criteria
 
   ##get sequence structure
@@ -330,14 +324,9 @@ analyse_pIRIRSequence <- function(
         drop = FALSE
       )
 
-  ##compile warning message
-  temp.sequence.rm.warning <- paste(
-    temp.sequence.structure[temp.sequence.rm.id, "recordType"], collapse = ", ")
-
-  temp.sequence.rm.warning <- paste(
-    "Record types are unrecognised and have been removed:", temp.sequence.rm.warning)
-
-  warning(temp.sequence.rm.warning, call. = FALSE)
+  .throw_warning("The following unrecognised record types have been removed: ",
+                 .collapse(temp.sequence.structure[temp.sequence.rm.id,
+                                                   "recordType"]))
   }
 
   ##(2) Apply user sequence structure
@@ -347,8 +336,8 @@ analyse_pIRIRSequence <- function(
 
     ##try to account for a very common mistake
     if(any(grepl(sequence.structure, pattern = "TL", fixed = TRUE)) && !any(grepl(temp.sequence.structure[["recordType"]], pattern = "TL", fixed = TRUE))){
-      warning("[analyse_pIRIRSequence()] Your sequence does not contain 'TL' curves, trying to adapt 'sequence.structure' for you ...",
-              call. = FALSE, immediate. = TRUE)
+      .throw_warning("Your sequence does not contain 'TL' curves, trying ",
+                     "to adapt 'sequence.structure' for you ...")
       sequence.structure <- sequence.structure[!grepl(sequence.structure, pattern = "TL", fixed = TRUE)]
     }
 
@@ -360,9 +349,9 @@ analyse_pIRIRSequence <- function(
       sequence.structure, nrow(temp.sequence.structure)/2/length(sequence.structure))
 
   }else{
-    try(stop("[analyse_pIRIRSequence()] Number of records is not a multiple of the defined sequence structure! NULL returned!", call. = FALSE))
+    .throw_message("The number of records is not a multiple of the defined ",
+                   "sequence structure, NULL returned")
     return(NULL)
-
   }
 
   ##remove values that have been excluded
@@ -385,8 +374,8 @@ analyse_pIRIRSequence <- function(
       sequence.structure, nrow(temp.sequence.structure)/2/length(temp.sequence.structure))
 
     ##print warning message
-    warning("[analyse_pIRIRSequence()] ", length(temp.sequence.rm.id), " records have been removed due to EXCLUDE!", call. = FALSE)
-
+    .throw_warning(length(temp.sequence.rm.id),
+                   " records have been removed due to EXCLUDE")
   }
 
 ##============================================================================##
@@ -395,9 +384,8 @@ analyse_pIRIRSequence <- function(
 
   ##(1) find out how many runs are needed for the analysis by checking for "IR"
   ##    now should by every signal except the TL curves
-  n.TL<- table(grepl("TL", sequence.structure))["TRUE"]
-  if(is.na(n.TL)) {n.TL<- 0}
-  n.loops <- as.numeric(length(grepl("TL", sequence.structure)) - n.TL)
+  n.TL <- sum(grepl("TL", sequence.structure))
+  n.loops <- as.numeric(length(sequence.structure) - n.TL)
 
   ##grep ids of TL curves (we need them later on)
   TL.curves.id <- temp.sequence.structure[
@@ -416,60 +404,30 @@ analyse_pIRIRSequence <- function(
   ## unfortunately a little bit more complicated then expected previously due
   ## the order of the produced plots by the previous functions
 
-  if(plot.single == FALSE & plot == TRUE){
+  if (plot && !plot_singlePanels) {
 
     ##first (Tx,Tn, Lx,Ln)
     temp.IRSL.layout.vector.first <- c(3,5,6,7,3,5,6,8)
 
-  ##middle (any other Lx,Ln)
-  if(n.loops > 2){
+    ## middle (any other Lx,Ln)
+    temp.IRSL.layout.vector.middle <- NULL
+    if (n.loops > 2) {
     temp.IRSL.layout.vector.middle <-
-      vapply(
-        2:(n.loops - 1),
-        FUN = function(x) {
-
-          offset <- 5 * x - 1
-          c((offset):(offset + 3),
-            (offset):(offset + 2), offset + 4)
-
-        },
+      vapply(2:(n.loops - 1),
+        FUN = function(x) 5 * x - 1 + c(0:3, 0:2, 4),
         FUN.VALUE = vector(mode = "numeric", length = 8)
       )
-  }
+    }
 
-  ##last (Lx,Ln and legend)
-  temp.IRSL.layout.vector.last <- c(
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 1,
-           max(temp.IRSL.layout.vector.first) + 1),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 2,
-           max(temp.IRSL.layout.vector.first) + 2),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 4,
-           max(temp.IRSL.layout.vector.first) + 4),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 5,
-           max(temp.IRSL.layout.vector.first) + 5),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 1,
-           max(temp.IRSL.layout.vector.first) + 1),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 2,
-           max(temp.IRSL.layout.vector.first) + 2),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 4,
-           max(temp.IRSL.layout.vector.first) + 4),
-    ifelse(n.loops > 2,max(temp.IRSL.layout.vector.middle) + 6,
-           max(temp.IRSL.layout.vector.first) + 6))
-
-  ##options for different sets of curves
-  if(n.loops > 2){
+    ## last (Lx, Ln and legend)
+    temp.IRSL.layout.vector.last <- c(1, 2, 4, 5, 1, 2, 4, 6) +
+      (if (n.loops > 2) max(temp.IRSL.layout.vector.middle)
+       else max(temp.IRSL.layout.vector.first))
 
     temp.IRSL.layout.vector <- c(temp.IRSL.layout.vector.first,
                                  temp.IRSL.layout.vector.middle,
                                  temp.IRSL.layout.vector.last)
 
-  }else{
-
-    temp.IRSL.layout.vector <- c(temp.IRSL.layout.vector.first,
-                                 temp.IRSL.layout.vector.last)
-
-  }
-
   ##get layout information
   def.par <- par(no.readonly = TRUE)
 
@@ -495,7 +453,6 @@ analyse_pIRIRSequence <- function(
 
   ## show the regions that have been allocated to each plot for debug
   #layout.show(nf)
-
   }
 
   ##(1) INFO PLOT
@@ -526,7 +483,7 @@ analyse_pIRIRSequence <- function(
       temp.signal.integral.min <- signal.integral.min[i]
       temp.signal.integral.max <- signal.integral.max[i]
       temp.background.integral.min <- background.integral.min[i]
-      temp.backbround.integral.max <- background.integral.max[i]
+      temp.background.integral.max <- background.integral.max[i]
 
     }else{
 
@@ -534,7 +491,6 @@ analyse_pIRIRSequence <- function(
       temp.signal.integral.max <- signal.integral.max
       temp.background.integral.min <- background.integral.min
       temp.background.integral.max <- background.integral.max
-
     }
 
     ##(c) call analysis sequence and plot
@@ -548,7 +504,6 @@ analyse_pIRIRSequence <- function(
 
    }else{
       temp.plot.single  <- c(2,4,6)
-
    }
 
     ##start analysis
@@ -560,15 +515,14 @@ analyse_pIRIRSequence <- function(
       background.integral.max = temp.background.integral.max,
       plot = plot,
       dose.points = dose.points,
-      plot.single = temp.plot.single,
-      output.plotExtended.single = TRUE,
+      plot_singlePanels = temp.plot.single,
       cex.global = cex,
       ...
     ) ##TODO should be replaced be useful explicit arguments
 
       ##check whether NULL was return
       if (is.null(temp.results)) {
-        try(stop("[plot_pIRIRSequence()] An error occurred, analysis skipped. Check your sequence!", call. = FALSE))
+        .throw_message("Analysis skipped: check your sequence, NULL returned")
         return(NULL)
       }
 
@@ -606,58 +560,41 @@ analyse_pIRIRSequence <- function(
         )
       )
 
-
       ##merge results
       if (exists("temp.results.final")) {
         temp.results.final <- merge_RLum(list(temp.results.final, temp.results))
 
       } else{
         temp.results.final <- temp.results
-
       }
-
-
   }
 
-
 ##============================================================================##
-# Plotting additionals--------------------------------------------------------
+# Plotting additional --------------------------------------------------------
 ##============================================================================##
-
 if(plot){
-
   ##extract LnLnxTnTx.table
   LnLxTnTx.table <- get_RLum(temp.results.final, "LnLxTnTx.table")
 
-    ##remove Inf
-    if(any(is.infinite(LnLxTnTx.table[["LxTx"]])))
-      LnLxTnTx.table[["LxTx"]][is.infinite(LnLxTnTx.table[["LxTx"]])] <- NA
-
-    if(any(is.infinite(LnLxTnTx.table[["LxTx.Error"]])))
-      LnLxTnTx.table[["LxTx.Error"]][is.infinite(LnLxTnTx.table[["LxTx.Error"]])] <- NA
-
+  ## remove Inf
+  LnLxTnTx.table$LxTx[is.infinite(LnLxTnTx.table$LxTx)] <- NA
+  LnLxTnTx.table$LxTx.Error[is.infinite(LnLxTnTx.table$LxTx.Error)] <- NA
 
   ##plot growth curves
+  min.LxTx <- min(LnLxTnTx.table$LxTx, na.rm = TRUE)
+  max.LxTx <- max(LnLxTnTx.table$LxTx, na.rm = TRUE)
+  max.LxTx.Error <- max(LnLxTnTx.table$LxTx.Error, na.rm = TRUE)
   plot(NA, NA,
        xlim = range(get_RLum(temp.results.final, "LnLxTnTx.table")$Dose),
-       ylim = c(
-         if(min(LnLxTnTx.table$LxTx, na.rm = TRUE) -
-            max(LnLxTnTx.table$LxTx.Error, na.rm = TRUE) < 0){
-            min(LnLxTnTx.table$LxTx, na.rm = TRUE)-
-            max(LnLxTnTx.table$LxTx.Error, na.rm = TRUE)
-         }else{0},
-           max(LnLxTnTx.table$LxTx, na.rm = TRUE)+
-           max(LnLxTnTx.table$LxTx.Error, na.rm = TRUE)),
+       ylim = c(min(min.LxTx - max.LxTx.Error, 0), max.LxTx + max.LxTx.Error),
        xlab = "Dose [s]",
        ylab = expression(L[x]/T[x]),
        main = "Summarised Dose Response Curves")
 
-
     ##set x for expression evaluation
-    x <- seq(0,max(LnLxTnTx.table$Dose)*1.05,length = 100)
+    x <- seq(0,max(LnLxTnTx.table$Dose)*1.05, length.out = 100)
 
     for(j in 1:length(pIRIR.curve.names)){
-
      ##dose points
      temp.curve.points <-  LnLxTnTx.table[,c("Dose", "LxTx", "LxTx.Error", "Signal")]
 
@@ -690,12 +627,10 @@ if(plot){
         temp.results.final, "Formula")[[pIRIR.curve.names[j]]]
 
      try(lines(x, eval(temp.curve.formula), col = j), silent = TRUE)
-
     }
 
     rm(x)
 
-
     ##plot legend
     legend("bottomright", legend = pIRIR.curve.names,
            lty = 1, col = c(1:length(pIRIR.curve.names)),
@@ -720,13 +655,12 @@ if(plot){
         , "TnTx"]
 
       temp.curve.TnTx.matrix[,j] <- temp.curve.TnTx.sel/temp.curve.TnTx.sel[1]
-
     }
 
     plot(NA, NA,
        xlim = c(0,nrow(LnLxTnTx.table)/
                      n.loops),
-       ylim = range(temp.curve.TnTx.matrix),
+       ylim = if (anyNA(range(temp.curve.TnTx.matrix))) c(0,1) else range(temp.curve.TnTx.matrix),
        xlab = "# Cycle",
        ylab = expression(T[x]/T[n]),
        main = "Sensitivity change")
@@ -735,7 +669,6 @@ if(plot){
     abline(h = 1:nrow(temp.curve.TnTx.matrix), col = "gray")
 
     for(j in 1:length(pIRIR.curve.names)){
-
      lines(1:nrow(temp.curve.TnTx.matrix),
            temp.curve.TnTx.matrix[,j],
            type = "b",
@@ -750,7 +683,6 @@ if(plot){
          pch = c(1:length(pIRIR.curve.names))
          )
 
-
    ##Rejection criteria
    temp.rejection.criteria <- get_RLum(temp.results.final,
                                                data.object = "rejection.criteria")
@@ -783,7 +715,6 @@ if(plot){
           y = c(21,29,29,21), col = "gray", border = NA)
     polygon(x = c(-0.3,-0.3,0.3,0.3) , y = c(21,29,29,21))
 
-
    ##consider possibility of multiple pIRIR signals and multiple recycling ratios
    col.id  <- 1
 
@@ -793,7 +724,6 @@ if(plot){
    for(i in seq(1,nrow(temp.rc.recuperation.rate),
                   length(unique(temp.rc.recuperation.rate[,"Criteria"])))){
 
-
         for(j in 0:length(unique(temp.rc.recuperation.rate[,"Criteria"]))){
          points(temp.rc.reycling.ratio[i+j, "Value"]-1,
                y = 25,
@@ -807,8 +737,6 @@ if(plot){
 
     rm(col.id)
 
-
-
    ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
    ##polygon for recuperation rate
    text(x = -.4, y = 20, "Recuperation rate", pos = 1, srt = 0)
@@ -830,7 +758,6 @@ if(plot){
            y = 15,
            pch = i,
            col = i)
-
   }
   }#endif
 
@@ -845,14 +772,12 @@ if(plot){
    polygon(x = c(-0.3,-0.3,0.3,0.3) , y = c(1,9,9,1))
    polygon(x = c(-0.3,-0.3,0,0) , y = c(1,9,9,1), border = NA, density = 10, angle = 45)
 
-
    for(i in 1:nrow(temp.rc.palaedose.error)){
-
-     points(temp.rc.palaedose.error[i, "Value"],
-            y = 5,
-            pch = i,
-            col = i)
-
+     if(length(temp.rc.palaedose.error[i, "Value"]) > 0 && !is.na(temp.rc.palaedose.error[i, "Value"]))
+       points(temp.rc.palaedose.error[i, "Value"],
+              y = 5,
+              pch = i,
+              col = i)
    }
 
    ##add 0 value
@@ -866,8 +791,10 @@ if(plot){
          pch = c(1:length(pIRIR.curve.names)))
 
 
-   ##reset graphic settings
-   if(plot.single == FALSE){par(def.par)}
+  ##reset graphic settings
+  if (!plot_singlePanels) {
+    par(def.par)
+  }
 
 }##end plot == TRUE
 
@@ -877,6 +804,4 @@ if(plot){
 ##============================================================================##
 
   return(temp.results.final)
-
-
 }
