@@ -53,7 +53,7 @@
 #' Expected are x and y coordinates, e.g.,
 #' `coord = list(samp1 = c(0.1, 0.2)`. If you have not measured x coordinates, please x should be 0.
 #'
-#' @param plot [logical] (*with default*): enable/disable plot output
+#' @param plot [logical] (*with default*): enable/disable the plot output.
 #'
 #' @param ... other parameters to be passed to modify the plot output.
 #' Supported are `run` to provide the run name ,
@@ -77,7 +77,7 @@
 #' @author Christoph Burow, University of Cologne (Germany), Sebastian Kreutzer,
 #' Institute of Geography, Ruprecht-Karl University of Heidelberg, Germany
 #'
-#' @section Function version: 0.1.0
+#' @section Function version: 0.1.1
 #'
 #' @keywords datagen plot
 #'
@@ -114,8 +114,10 @@ analyse_portableOSL <- function(
   mode = "profile",
   coord = NULL,
   plot = TRUE,
-  ...)
-  {
+  ...
+) {
+  .set_function_name("analyse_portableOSL")
+  on.exit(.unset_function_name(), add = TRUE)
 
   ## TODO
   ## - add tests for background image option
@@ -123,7 +125,7 @@ analyse_portableOSL <- function(
 
 # Self-call ---------------------------------------------------------------
   if (inherits(object, "list")) {
-      temp <- .warningCatcher(lapply(1:length(object), function(x) {
+      temp <- .warningCatcher(lapply(seq_along(object), function(x) {
         analyse_portableOSL(
           object = object[[x]],
           signal.integral = signal.integral,
@@ -134,34 +136,31 @@ analyse_portableOSL <- function(
       }))
 
       return(merge_RLum(temp))
-
   }
 
-# Start function ----------------------------------------------------------
-  ## INPUT VERIFICATION ----
+  ## Start function ---------------------------------------------------------
+
   ## only RLum.Analysis objects
-  if (!inherits(object, "RLum.Analysis"))
-    stop("[analyse_portableOSL()] Only objects of class 'RLum.Analysis' are allowed.",
-         call. = FALSE)
+  .validate_class(object, "RLum.Analysis")
+  .validate_not_empty(object)
+
   ## only curve objects
   if (!all(sapply(object, class) == "RLum.Data.Curve"))
-    stop("[analyse_portableOSL()] The 'RLum.Analysis' object must contain only objects of class 'RLum.Data.Curve'.",
-         call. = FALSE)
+    .throw_error("The 'RLum.Analysis' object must contain only objects ",
+                 "of class 'RLum.Data.Curve'")
 
   ## check originator
   if (!all(sapply(object, function(x) x@originator) == "read_PSL2R"))
-    stop("[analyse_portableOSL()] Only objects originating from 'read_PSL2R()' are allowed.",
-         call. = FALSE)
+    .throw_error("Only objects originating from 'read_PSL2R()' are allowed")
 
   ## check sequence pattern
   if(!all(names(object)[1:5] == c("USER", "IRSL", "USER", "OSL", "USER")))
-    stop("[analyse_portableOSL()] Sequence pattern not supported, please read manual for details!",
-         call. = FALSE)
+    .throw_error("Sequence pattern not supported, please read manual for details")
 
   if (is.null(signal.integral)) {
     signal.integral <- c(1, 1)
-    warning("No value for 'signal.integral' provided. Only the first data point of each curve was used!",
-            call. = FALSE)
+    .throw_warning("No value for 'signal.integral' provided. Only the ",
+                   "first data point of each curve was used")
   }
 
 
@@ -212,18 +211,13 @@ analyse_portableOSL <- function(
     coord <- .extract_PSL_coord(object)
 
   } else {
-    if(!inherits(coord, "matrix") && !inherits(coord, "list"))
-      stop("[analyse_portableOSL()] Argument 'coord' needs to be a matrix or list!",
-           call. = FALSE)
-
+    .validate_class(coord, c("matrix", "list"))
     if(inherits(coord, "list"))
       coord <- do.call(rbind, coord)
 
     ## check length
     if(nrow(coord) != length(OSL$sum_signal))
-      stop("[analyse_portableOSL()] Number of coordinates differ from the number of samples!",
-           call. = FALSE)
-
+      .throw_error("Number of coordinates differ from the number of samples")
   }
 
   ### GENERATE SUMMARY data.frame -----
@@ -244,7 +238,7 @@ analyse_portableOSL <- function(
   )
 
     ## if coordinates exist, sort by depth
-    if(!any(is.na(coord[,2])))
+    if (!anyNA(coord[, 2]))
       summary <- summary[order(coord[,2]),]
 
    ### INVERT ----------
@@ -291,7 +285,7 @@ analyse_portableOSL <- function(
        xlim = attr(m_list, "xlim"),
        ylim = attr(m_list, "ylim"),
        zlim = if(mode == "surface") NA else attr(m_list, "zlim"),
-       ylab = if(!any(is.na(summary$COORD_Y))) "Depth [m]" else "Index",
+       ylab = if (!anyNA(summary$COORD_Y)) "Depth [m]" else "Index",
        xlab = "x [m]",
        grid = TRUE,
        contour = FALSE,
@@ -304,15 +298,13 @@ analyse_portableOSL <- function(
    if(mode[1] == "surface") {
      ### check for validity of surface value -------
      if(!all(plot_settings$surface_value %in% names(m_list)))
-       stop(paste0("[analyse_portableOSL()] Unknown value to plot: Valid are: ",
-            paste(names(m_list), collapse = ", ")),
-            call. = FALSE)
+       .throw_error("Unknown value to plot, valid values are: ",
+                    .collapse(names(m_list)))
 
      ## set par -------
      if(length(plot_settings$surface_value) > 1) {
        par.default <- par(mfrow = c(2,2))
-       on.exit(par(par.default))
-
+       on.exit(par(par.default), add = TRUE)
      }
 
      ## loop over surface values -------
@@ -340,14 +332,16 @@ analyse_portableOSL <- function(
 
        ## show only warning
        if(inherits(s, "try-error"))
-         warning("[analyse_portableOSL()] Surface interpolation failed, this happens when all points are arranged in one line. Nothing plotted!", call. = FALSE)
+         .throw_warning("Surface interpolation failed: this happens when ",
+                        "all points are arranged in one line or xlim/ylim/zlim ",
+                        "are too tight. Nothing plotted!")
 
        ## show error
        if(!inherits(s, "try-error")) {
          par.default <- c(
            if(exists("par.default")) par.default else NULL,
            par(mar = c(4.5,4.5,4,2), xpd = FALSE))
-         on.exit(par(par.default))
+         on.exit(par(par.default), add = TRUE)
 
          ## open empty plot
          plot(
@@ -362,9 +356,8 @@ analyse_portableOSL <- function(
          ## add background image if available -------
          if (!is.null(plot_settings$bg_img)) {
            ## get corner positions
-           if(!is.null(plot_settings$bg_img_positions))
-             positions <- plot_settings$bg_img_positions[1:4]
-           else
+           positions <- plot_settings$bg_img_positions[1:4]
+           if (is.null(positions))
              positions <- par()$usr
 
            graphics::rasterImage(
@@ -396,7 +389,7 @@ analyse_portableOSL <- function(
          ## add legend
          if(plot_settings$legend) {
            par.default <- c(par.default, par(xpd = TRUE))
-           on.exit(par(par.default))
+           on.exit(par(par.default), add = TRUE)
 
            col_grad <- plot_settings$col_ramp[
              seq(1, length(plot_settings$col_ramp), length.out = 14)]
@@ -454,24 +447,23 @@ analyse_portableOSL <- function(
    ## mode == "profile" ---------
    if (!is.null(mode[1]) && mode == "profile") {
     par.old.full <- par(no.readonly = TRUE)
-    on.exit(par(par.old.full))
+    on.exit(par(par.old.full), add = TRUE)
 
     # default: par(mar = c(5, 4, 4, 2) + 0.1) // bottom, left, top, right
     par(mfrow = c(1, 7))
 
     par(mar = c(5, 4, 4, 1) + 0.1)
 
-    frame()
+    graphics::frame()
 
-    mtext(side= 3, run, cex = 0.7, line = 2)
+    mtext(side= 3, plot_settings$main, cex = 0.7, line = 2)
 
     par(mar = c(5, 0, 4, 1) + 0.1)
 
     ## make sure that wrong zlim settings do not screw up the function
     if(!inherits(plot_settings$zlim, "list")) {
-      warning("[analyse_portableOSL()] In profile mode, zlim needs to be provided
-               as named list. Example: list(BSL = c(0,1)). Reset to default.",
-              call. = FALSE)
+      .throw_warning("In profile mode, zlim needs to be provided as a named ",
+                     "list, example: list(BSL = c(0,1)). Reset to default")
       plot_settings$zlim <- attr(m_list, "zlim")
     }
 
@@ -485,7 +477,7 @@ analyse_portableOSL <- function(
       bty = "n",
       yaxt = "n"
     )
-      if(plot_settings$grid) grid()
+      if (plot_settings$grid) graphics::grid()
       lines(
         x = m_list[["BSL"]][,"value"],
         y = m_list[["BSL"]][,"y"],
@@ -517,7 +509,7 @@ analyse_portableOSL <- function(
         bty = "n",
         yaxt = "n"
     )
-      if(plot_settings$grid) grid()
+      if (plot_settings$grid) graphics::grid()
 
       lines(
         x = m_list[["IRSL"]][,"value"],
@@ -546,7 +538,7 @@ analyse_portableOSL <- function(
       yaxt = "n"
     )
 
-      if(plot_settings$grid) grid()
+      if (plot_settings$grid) graphics::grid()
       lines(
         x = m_list[["BSL_depletion"]][,"value"],
         y = m_list[["BSL_depletion"]][,"y"],
@@ -568,7 +560,7 @@ analyse_portableOSL <- function(
       yaxt = "n"
     )
 
-      if(plot_settings$grid) grid()
+      if (plot_settings$grid) graphics::grid()
 
       lines(
         x = m_list[["IRSL_depletion"]][,"value"],
@@ -591,7 +583,7 @@ analyse_portableOSL <- function(
       yaxt = "n"
     )
 
-      if(plot_settings$grid) grid()
+      if (plot_settings$grid) graphics::grid()
 
       lines(
         x = m_list[["IRSL_BSL_RATIO"]][,"value"],
@@ -656,9 +648,11 @@ analyse_portableOSL <- function(
     sigint <- range(signal.integral)
     if (sigint[2] > length(raw_signal)) {
       sigint[2] <- length(raw_signal)
-      warning("'signal.integral' (", paste(range(signal.integral), collapse = ", "),") ",
-              "exceeded the number of available data points (n = ", length(raw_signal),") and ",
-              "has been automatically reduced to the maximum number.", call. = FALSE)
+      .throw_warning("'signal.integral' (",
+                     .collapse(range(signal.integral), quote = FALSE), ") ",
+                     "exceeded the number of available data points (n = ",
+                     length(raw_signal),") and has been automatically ",
+                     "reduced to the maximum number.")
     }
     sum_signal <- sum(raw_signal[sigint[1]:sigint[2]])
     sum_signal_err <- sqrt(sum(x@info$raw_data$counts_per_cycle_error[sigint[1]:sigint[2]]^2))
@@ -719,8 +713,8 @@ analyse_portableOSL <- function(
     numeric(2)) |> t()
 
   ## if NA, assign index
-  if(any(is.na(coord[,1]))) coord[,1] <- 0
-  if(any(is.na(coord[,2]))) coord[,2] <- 1:nrow(coord)
+  if (anyNA(coord[, 1])) coord[, 1] <- 0
+  if (anyNA(coord[, 2])) coord[, 2] <- 1:nrow(coord)
 
   return(coord)
 }
