@@ -1,9 +1,10 @@
 #' Apply the central age model (CAM) after Galbraith et al. (1999) to a given
 #' De distribution
 #'
+#' @description
 #' This function calculates the central dose and dispersion of the De
 #' distribution, their standard errors and the profile log likelihood function
-#' for sigma.
+#' for `sigma`.
 #'
 #' This function uses the equations of Galbraith & Roberts (2012). The
 #' parameters `delta` and `sigma` are estimated by numerically solving
@@ -16,23 +17,23 @@
 #' Galbraith & Roberts (2012, 15)
 #'
 #' @param data [RLum.Results-class] or [data.frame] (**required**):
-#' for [data.frame]: two columns with De `(data[,1])` and De error `(data[,2])`
+#' for [data.frame]: two columns with De `(data[,1])` and De error `(data[,2])`.
+#' Records containing missing values will be removed.
 #'
 #' @param sigmab [numeric] (*with default*):
 #' additional spread in De values.
 #' This value represents the expected overdispersion in the data should the sample be
 #' well-bleached (Cunningham & Walling 2012, p. 100).
 #' **NOTE**: For the logged model (`log = TRUE`) this value must be
-#' a fraction, e.g. 0.2 (= 20 \%). If the un-logged model is used (`log = FALSE`),
+#' a fraction, e.g. 0.2 (= 20 %). If the un-logged model is used (`log = FALSE`),
 #' sigmab must be provided in the same absolute units of the De values (seconds or Gray).
 #'
 #' @param log [logical] (*with default*):
-#' fit the (un-)logged central age model to De data
-#'
-#' @param na.rm [logical] (*with default*): strip `NA` values before the computation proceeds
+#' fit the (un-)logged central age model to De data. Log transformation is
+#' allowed only if the De values are positive.
 #'
 #' @param plot [logical] (*with default*):
-#' plot output
+#' enable/disable the plot output.
 #'
 #' @param ... further arguments (`trace`, `verbose`).
 #'
@@ -45,9 +46,9 @@
 #' \item{.$call}{[call] the function call}
 #' \item{.$profile}{[data.frame] the log likelihood profile for sigma}
 #'
-#' The output should be accessed using the function [get_RLum]
+#' The output should be accessed using the function [get_RLum].
 #'
-#' @section Function version: 1.4.0
+#' @section Function version: 1.5
 #'
 #' @author
 #' Christoph Burow, University of Cologne (Germany) \cr
@@ -98,41 +99,49 @@
 #'
 #' @md
 #' @export
-calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRUE, ...) {
+calc_CentralDose <- function(data, sigmab, log = TRUE, plot = TRUE, ...) {
+  .set_function_name("calc_CentralDose")
+  on.exit(.unset_function_name(), add = TRUE)
+
   ## ============================================================================##
   ## CONSISTENCY CHECK OF INPUT DATA
   ## ============================================================================##
-  if (!missing(data)) {
-    if (!is(data, "data.frame") & !is(data, "RLum.Results")) {
-      stop("[calc_CentralDose()] 'data' has to be of type 'data.frame' or 'RLum.Results'!", call. = FALSE)
-    } else {
-      if (is(data, "RLum.Results")) {
-        data <- get_RLum(data, "data")
-      }
-    }
+
+  .validate_class(data, c("data.frame", "RLum.Results"))
+  if (inherits(data, "RLum.Results")) {
+    data <- get_RLum(data, "data")
   }
 
   ##remove NA values
-  if(na.rm == TRUE && any(is.na(data))){
-    warning("[calc_CentralDose()] ", length(which(is.na(data))), " NA value(s) removed from dataset!", call. = FALSE)
+  if (anyNA(data)) {
+    message("[calc_CentralDose()] ", length(which(is.na(data))),
+            " NA values removed from dataset")
     data <- na.exclude(data)
   }
 
   ##make sure we consider onlyt take the first two columns
   if(ncol(data) < 2 || nrow(data) < 2)
-    stop("[calc_CentralDose()] 'data' should have at least two columns and two rows!", call. = FALSE)
+    .throw_error("'data' should have at least two columns and two rows")
 
   ##extract only the first two columns and set column names
   data <- data[,1:2]
   colnames(data) <- c("ED", "ED_Error")
 
-  if (!missing(sigmab)) {
-    if (sigmab < 0 | sigmab > 1 & log)
-      stop("[calc_CentralDose()] sigmab needs to be given as a fraction between 0 and 1 (e.g., 0.2)!", call. = FALSE)
-
+  ## don't allow log transformation if there are non-positive values
+  if (any(data[, 1] <= 0) && log == TRUE) {
+    log <- FALSE
+    .throw_warning("'data' contains non-positive De values, 'log' set to FALSE")
   }
 
+  ## don't allow negative errors, silently make them positive
+  if (any(data[, 2] < 0)) {
+    data[, 2] <- abs(data[, 2])
+  }
 
+  if (!missing(sigmab)) {
+    if (sigmab < 0 | sigmab > 1 & log)
+      .throw_error("'sigmab' should be a fraction between 0 and 1 (e.g., 0.2)")
+  }
 
 
   ## ============================================================================##
@@ -144,6 +153,12 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
 
   options <- modifyList(options, list(...))
 
+  ## deprecated argument
+  if ("na.rm" %in% names(list(...))) {
+    .throw_warning("'na.rm' is deprecated, missing values are always removed ",
+                   "by default")
+  }
+
 
   ## ============================================================================##
   ## CALCULATIONS
@@ -188,6 +203,10 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
     # print iterations
     if (options$trace)
       print(round(c(delta, sigma), 4))
+
+    ## don't let sigma become zero
+    if (sigma < 1e-16)
+      break()
   }
 
   # save parameters for terminal output
@@ -213,7 +232,6 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
     out.sedelta <- sedelta / out.delta * 100
     out.sesigma <- sqrt((sedelta / delta)^2 +
                           (sesigma / out.delta * 100 / out.sigma)^2) * out.sigma / 100
-
   }
 
   # profile log likelihood
@@ -245,15 +263,15 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
     cat(paste("\n n:                      ", n))
     cat(paste("\n log:                    ", log))
     cat(paste("\n----------- dose estimate ------------"))
-    cat(paste("\n central dose [Gy]:      ", format(out.delta, digits = 2, nsmall = 2)))
-    cat(paste("\n SE [Gy]:                ", format(out.delta * out.sedelta/100,
+    cat(paste("\n abs. central dose:      ", format(out.delta, digits = 2, nsmall = 2)))
+    cat(paste("\n abs. SE:                ", format(out.delta * out.sedelta/100,
                                                    digits = 2, nsmall = 2)))
     cat(paste("\n rel. SE [%]:            ", format(out.sedelta, digits = 2, nsmall = 2)))
     cat(paste("\n----------- overdispersion -----------"))
-    cat(paste("\n OD [Gy]:                ", format(ifelse(log, sigma * out.delta, sigma), digits = 2, nsmall = 2)))
-    cat(paste("\n SE [Gy]:                ", format(ifelse(log, sesigma * out.delta, sesigma), digits = 2, nsmall = 2)))
+    cat(paste("\n abs. OD:                ", format(ifelse(log, sigma * out.delta, sigma), digits = 2, nsmall = 2)))
+    cat(paste("\n abs. SE:                ", format(ifelse(log, sesigma * out.delta, sesigma), digits = 2, nsmall = 2)))
     cat(paste("\n OD [%]:                 ", format(out.sigma, digits = 2, nsmall = 2)))
-    cat(paste("\n SE [%]:                 ", if (class(sig) != "try-error") {
+    cat(paste("\n SE [%]:                 ", if (!inherits(sig, "try-error")) {
       format(out.sesigma * 100, digits = 2, nsmall = 2)
     } else {
       "-"
@@ -265,16 +283,21 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
   ## RETURN VALUES
   ## ============================================================================##
 
-  if (class(sig) == "try-error") {
+  if (inherits(sig, "try-error")) {
     out.sigma <- 0
     out.sesigma <- NA
   }
 
   if(!log)  sig <- sig / delta
 
-
-  summary <- data.frame(de = out.delta, de_err = out.delta * out.sedelta / 100,
-                        OD = out.sigma, OD_err = out.sesigma * 100, Lmax = Lmax)
+  summary <- data.frame(
+    de = out.delta,
+    de_err = out.delta * out.sedelta / 100,
+    OD = ifelse(log, sigma * out.delta, sigma),
+    OD_err = ifelse(log, sesigma * out.delta, sesigma),
+    rel_OD = out.sigma,
+    rel_OD_err = out.sesigma * 100,
+    Lmax = Lmax)
 
   args <- list(log = log, sigmab = sigmab)
 
@@ -294,7 +317,7 @@ calc_CentralDose <- function(data, sigmab, log = TRUE, na.rm = FALSE, plot = TRU
   )
 
   ## =========## PLOTTING
-  if (plot && class(sig) != "try-error")
+  if (plot && !inherits(sig, "try-error"))
     try(plot_RLum.Results(newRLumResults.calc_CentralDose, ...))
 
   invisible(newRLumResults.calc_CentralDose)
