@@ -1,12 +1,12 @@
-#' Apply the finite mixture model (FMM) after Galbraith (2005) to a given De
+#' @title Apply the finite mixture model (FMM) after Galbraith (2005) to a given De
 #' distribution
 #'
-#' This function fits a k-component mixture to a De distribution with differing
+#' @description  This function fits a k-component mixture to a De distribution with differing
 #' known standard errors. Parameters (doses and mixing proportions) are
 #' estimated by maximum likelihood assuming that the log dose estimates are
 #' from a mixture of normal distributions.
 #'
-#' This model uses the maximum likelihood and Bayesian Information Criterion
+#' @details This model uses the maximum likelihood and Bayesian Information Criterion
 #' (BIC) approaches.
 #'
 #' Indications of overfitting are:
@@ -14,13 +14,13 @@
 #' - increasing BIC
 #' - repeated dose estimates
 #' - covariance matrix not positive definite
-#' - covariance matrix produces NaNs
+#' - covariance matrix produces `NaN`
 #' - convergence problems
 #'
 #' **Plot**
 #'
 #' If a vector (`c(k.min:k.max)`) is provided
-#' for `n.components` a plot is generated showing the the k components
+#' for `n.components` a plot is generated showing the k components
 #' equivalent doses as normal distributions. By default `pdf.weight` is
 #' set to `FALSE`, so that the area under each normal distribution is
 #' always 1. If `TRUE`, the probability density functions are weighted by
@@ -32,8 +32,8 @@
 #' determined by a common `sigmab` (see `pdf.sigma`). For
 #' `pdf.sigma = "se"` the standard error of each component is taken
 #' instead.\cr
-#' The stacked barplot shows the proportion of each component (in
-#' per cent) calculated by the FFM. The last plot shows the achieved BIC scores
+#' The stacked [graphics::barplot] shows the proportion of each component (in
+#' per cent) calculated by the FMM. The last plot shows the achieved BIC scores
 #' and maximum log-likelihood estimates for each iteration of k.
 #'
 #' @param data [RLum.Results-class] or [data.frame] (**required**):
@@ -53,10 +53,6 @@
 #' @param grain.probability [logical] (*with default*):
 #' prints the estimated probabilities of which component each grain is in
 #'
-#' @param dose.scale [numeric]:
-#' manually set the scaling of the y-axis of the first plot with a vector
-#' in the form of `c(min, max)`
-#'
 #' @param pdf.weight [logical] (*with default*):
 #' weight the probability density functions by the components proportion (applies only
 #' when a vector is provided for `n.components`)
@@ -68,17 +64,14 @@
 #' parameter of the normal distribution
 #'
 #' @param pdf.colors [character] (*with default*):
-#' color coding of the components in the the plot.
+#' colour coding of the components in the plot.
 #' Possible options are `"gray"`, `"colors"` and `"none"`
 #'
-#' @param pdf.scale [numeric]:
-#' manually set the max density value for proper scaling of the x-axis of the first plot
-#'
 #' @param plot.proportions [logical] (*with default*):
-#' plot barplot showing the proportions of components
+#' plot [graphics::barplot] showing the proportions of components if
+#' `n.components` a vector with a length > 1 (e.g., `n.components = c(2:3)`)
 #'
-#' @param plot [logical] (*with default*):
-#' plot output
+#' @param plot [logical] (*with default*): enable/disable the  plot output.
 #'
 #' @param ... further arguments to pass. See details for their usage.
 #'
@@ -91,20 +84,20 @@
 #' \item{.$data}{[data.frame] original input data}
 #' \item{.$args}{[list] used arguments}
 #' \item{.$call}{[call] the function call}
-#' \item{.$mle}{ covariance matrices of the log likelhoods}
+#' \item{.$mle}{ covariance matrices of the log likelihoods}
 #' \item{.$BIC}{ BIC score}
 #' \item{.$llik}{ maximum log likelihood}
 #' \item{.$grain.probability}{ probabilities of a grain belonging to a component}
 #' \item{.$components}{[matrix] estimates of the de, de error and proportion for each component}
-#' \item{.$single.comp}{[data.frame] single componente FFM estimate}
+#' \item{.$single.comp}{[data.frame] single component FFM estimate}
 #'
 #' If a vector for `n.components` is provided (e.g.  `c(2:8)`),
 #' `mle` and `grain.probability` are lists containing matrices of the
 #' results for each iteration of the model.
 #'
-#' The output should be accessed using the function [get_RLum]
+#' The output should be accessed using the function [get_RLum].
 #'
-#' @section Function version: 0.4.1
+#' @section Function version: 0.4.2
 #'
 #' @author
 #' Christoph Burow, University of Cologne (Germany) \cr
@@ -170,7 +163,7 @@
 #' ## the following lines to make the example work.
 #' FMM<- calc_FiniteMixture(ExampleData.DeValues$CA1,
 #'                          sigmab = 0.2, n.components = c(2:4),
-#'                          pdf.weight = TRUE, dose.scale = c(0, 100))
+#'                          pdf.weight = TRUE)
 #'
 #' ## show structure of the results
 #' FMM
@@ -186,76 +179,58 @@ calc_FiniteMixture <- function(
   sigmab,
   n.components,
   grain.probability = FALSE,
-  dose.scale,
   pdf.weight = TRUE,
   pdf.sigma = "sigmab",
   pdf.colors = "gray",
-  pdf.scale,
   plot.proportions = TRUE,
   plot=TRUE,
   ...
-){
+) {
+  .set_function_name("calc_FiniteMixture")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  ##============================================================================##
-  ## CONSISTENCY CHECK OF INPUT DATA
-  ##============================================================================##
+  ## Integrity checks -------------------------------------------------------
 
-  if(missing(data)==FALSE){
-
-    if(is(data, "data.frame") == FALSE & is(data,"RLum.Results") == FALSE){
-      stop("[calc_FiniteMixture] Error: 'data' object has to be of type
-           'data.frame' or 'RLum.Results'!")
-    } else {
-      if(is(data, "RLum.Results") == TRUE){
-        data <- get_RLum(data, "data")
-      }
-    }
+  .validate_class(data, c("data.frame", "RLum.Results"))
+  if (is(data, "RLum.Results")) {
+    data <- get_RLum(data, "data")
   }
-  try(colnames(data)<- c("ED","ED_Error"),silent=TRUE)
-  if(colnames(data[1])!="ED"||colnames(data[2])!="ED_Error") {
-    cat(paste("Columns must be named 'ED' and 'ED_Error'"), fill = FALSE)
-    stop(domain=NA)
+  if (ncol(data) < 2) {
+    .throw_error("'data' object must have two columns")
   }
-  if(sigmab <0 | sigmab >1) {
-    cat(paste("sigmab needs to be given as a fraction between",
-              "0 and 1 (e.g. 0.2)"), fill = FALSE)
-    stop(domain=NA)
+  if (sigmab < 0 || sigmab > 1) {
+    .throw_error("'sigmab' must be a value between 0 and 1")
   }
   if(any(n.components<2) == TRUE) {
-    cat(paste("Atleast two components need to be fitted"), fill = FALSE)
-    stop(domain=NA)
-  }
-  if(pdf.sigma!="se" ) {
-    if(pdf.sigma!="sigmab") {
-      cat(paste("Only 'se' or 'sigmab' allowed for the pdf.sigma argument"),
-          fill = FALSE)
-      stop(domain=NA)
-    }
+    .throw_error("At least two components need to be fitted")
   }
+  pdf.sigma <- .validate_args(pdf.sigma, c("sigmab", "se"))
+  pdf.colors <- .validate_args(pdf.colors, c("gray", "colors", "none"))
 
-  ##============================================================================##
-  ## ... ARGUMENTS
+  ## set expected column names
+  colnames(data)[1:2] <- c("ED", "ED_Error")
+
+  ## ... ARGUMENTS ------------
   ##============================================================================##
 
   extraArgs <- list(...)
 
+  ## default values
+  verbose <- TRUE
+  trace <- FALSE
+  main <- "Finite Mixture Model"
+
   ## console output
   if("verbose" %in% names(extraArgs)) {
     verbose<- extraArgs$verbose
-  } else {
-    verbose<- TRUE
   }
   # trace calculations
   if("trace" %in% names(extraArgs)) {
     trace<- extraArgs$trace
-  } else {
-    trace<- FALSE
   }
   # plot title
   if("main" %in% names(extraArgs)) {
     main<- extraArgs$main
-  } else {
-    main<- "Finite Mixture Model"
   }
 
   ##============================================================================##
@@ -282,7 +257,6 @@ calc_FiniteMixture <- function(
     # grain probabilities
     vmat.n<- vector(mode = "list", length = length(n.components))
     grain.probability.n<- vector(mode = "list", length = length(n.components))
-
   }
 
   ## start actual calculation (loop) for each provided maximum components to
@@ -328,7 +302,7 @@ calc_FiniteMixture <- function(
     bic<- -2*llik + (2*k - 1)*log(n)
 
     # calculate the covariance matrix and standard errors of the estimates
-    # i.e., the dose estimtes in Gy and relative standard errors, and
+    # i.e., the dose estimates in Gy and relative standard errors, and
     # the mixing proportions and standard errors.
     aui<- matrix(0,n,k)
     bui<- matrix(0,n,k)
@@ -357,12 +331,10 @@ calc_FiniteMixture <- function(
 
     invvmat<- rbind(cbind(Au,Bu),cbind(t(Bu),Cu))
     vmat<- solve(invvmat, tol=.Machine$double.xmin)
-    rek<- sqrt(sum(vmat[1:(k-1),1:(k-1)]))
 
     # calculate DE, relative standard error, standard error
     dose<- exp(mu)
-    re<- sqrt(diag(vmat))[-c(1:(k-1))]
-
+    re <- suppressWarnings(sqrt(diag(vmat)))[-c(1:(k-1))]
     if (any(is.nan(re)))
       re[is.nan(re)] <- NA
 
@@ -374,6 +346,7 @@ calc_FiniteMixture <- function(
 
     # this calculates the proportional standard error of the proportion of grains
     # in the fitted components. However, the calculation is most likely erroneous.
+    # rek<- sqrt(sum(vmat[1:(k-1),1:(k-1)]))
     # sep<-  c(sqrt(diag(vmat))[c(1:(k-1))],rek)
 
     # rename proportion
@@ -449,18 +422,14 @@ calc_FiniteMixture <- function(
     BIC.lowest<- n.components[which.min(BIC.n)]
   }
 
+  ## OUTPUT ---------
   ##============================================================================##
-  ## OUTPUT
-  ##============================================================================##
-
-  if(verbose==TRUE) {
+  if(verbose) {
 
     ## HEADER (always printed)
     cat("\n [calc_FiniteMixture]")
 
-    ##----------------------------------------------------------------------------
     ## OUTPUT WHEN ONLY ONE VALUE FOR n.components IS PROVIDED
-
     if(length(n.components) == 1) {
 
       # covariance matrix
@@ -565,8 +534,7 @@ calc_FiniteMixture <- function(
     }#EndOf::Output for length(n.components) > 1
   }
 
-  ##============================================================================##
-  ## RETURN VALUES
+  ## RETURN VALUES --------
   ##============================================================================##
 
   # .@data$meta
@@ -610,15 +578,15 @@ calc_FiniteMixture <- function(
       single.comp=single.comp))
 
   if (anyNA(unlist(summary)) && verbose)
-    warning("\n[calc_FiniteMixture] The model produced NA values. Either the input data are inapplicable for the model",
-            " or the the model parameters need to be adjusted (e.g. 'sigmab')", call. = FALSE)
+    .throw_warning("The model produced NA values: either the input data are ",
+                   "inapplicable for the model, or the model parameters ",
+                   "need to be adjusted (e.g. 'sigmab')")
 
   ##=========##
-  ## PLOTTING
+  ## PLOTTING -----------
   if(plot && !anyNA(unlist(summary)))
     try(do.call(plot_RLum.Results, c(list(newRLumResults.calc_FiniteMixture), as.list(sys.call())[-c(1,2)])))
 
   # Return values
   invisible(newRLumResults.calc_FiniteMixture)
-
 }
