@@ -1,5 +1,6 @@
-#' Nonlinear Least Squares Fit for CW-OSL curves -beta version-
+#' @title Nonlinear Least Squares Fit for CW-OSL curves -beta version-
 #'
+#' @description
 #' The function determines the weighted least-squares estimates of the
 #' component parameters of a CW-OSL signal for a given maximum number of
 #' components and returns various component parameters. The fitting procedure
@@ -26,7 +27,7 @@
 #'
 #' **Goodness of fit**
 #'
-#' The goodness of the fit is given as pseudoR^2 value (pseudo coefficient of
+#' The goodness of the fit is given as pseudoR² value (pseudo coefficient of
 #' determination). According to Lave (1970), the value is calculated as:
 #'
 #' \deqn{pseudoR^2 = 1 - RSS/TSS}
@@ -35,7 +36,6 @@
 #' and \eqn{TSS = Total~Sum~of~Squares}
 #'
 #'
-#'
 #' **Error of fitted component parameters**
 #'
 #' The 1-sigma error for the
@@ -54,7 +54,7 @@
 #' maximum number of components that are to be used for fitting.
 #' The upper limit is 7.
 #'
-#' @param fit.failure_threshold [vector] (*with default*):
+#' @param fit.failure_threshold [integer] (*with default*):
 #' limits the failed fitting attempts.
 #'
 #' @param fit.method [character] (*with default*):
@@ -69,7 +69,7 @@
 #' calculate 1-sigma error range of components using [stats::confint]
 #'
 #' @param LED.power [numeric] (*with default*):
-#' LED power (max.) used for intensity ramping in mW/cm^2.
+#' LED power (max.) used for intensity ramping in mW/cm².
 #' **Note:** The value is used for the calculation of the absolute
 #' photoionisation cross section.
 #'
@@ -84,20 +84,19 @@
 #' @param sample_code [character] (*optional*):
 #' sample code used for the plot and the optional output table (`mtext`).
 #'
-#' @param output.path [character] (*optional*):
-#' output path for table output containing the results of the fit. The file
-#' name is set automatically. If the file already exists in the directory,
-#' the values are appended.
-#'
-#' @param output.terminal [logical] (*with default*):
-#' terminal output with fitting results.
+#' @param verbose [logical] (*with default*):
+#' enable/disable output to the terminal.
 #'
 #' @param output.terminalAdvanced [logical] (*with default*):
-#' enhanced terminal output. Requires `output.terminal = TRUE`.
-#' If `output.terminal = FALSE` no advanced output is possible.
+#' enhanced terminal output. Only valid if `verbose = TRUE`.
 #'
 #' @param plot [logical] (*with default*):
-#' returns a plot of the fitted curves.
+#' enable/disable the plot output.
+#'
+#' @param method_control [list] (*optional*): options to control the output
+#' produced. Currently only the 'export.comp.contrib.matrix' (logical) option
+#' is supported, to enable/disable export of the component contribution
+#' matrix.
 #'
 #' @param ... further arguments and graphical parameters passed to [plot].
 #'
@@ -106,13 +105,7 @@
 #'
 #' the fitted CW-OSL curves are returned as plot.
 #'
-#' **table (*optional*)**
-#'
-#' an output table (`*.csv`) with parameters of the fitted components is
-#' provided if the `output.path` is set.
-#'
-#'
-#' **RLum.Results**
+#' **RLum.Results object**
 #'
 #' Beside the plot and table output options, an [RLum.Results-class] object is
 #' returned.
@@ -128,6 +121,7 @@
 #' `component.contribution.matrix`:
 #' [matrix] containing the values for the component to sum contribution plot
 #' (`$component.contribution.matrix`).
+#' Produced only if `method_control$export.comp.contrib.matrix = TRUE`).
 #'
 #' Matrix structure:\cr
 #' Column 1 and 2: time and `rev(time)` values \cr
@@ -136,42 +130,19 @@
 #' the relative component contribution for each time interval including the row
 #' sum for this values.
 #'
-#' **object**
-#'
-#' beside the plot and table output options, an [RLum.Results-class] object
-#' is returned.
-#'
-#' `fit`:
-#' an `nls` object (`$fit`) for which generic R functions
-#' are provided, e.g. [summary], [confint], [profile]. For more
-#' details, see [nls].
-#'
-#' `output.table`:
-#' a [data.frame] containing the summarised parameters including the error\cr
-#' `component.contribution.matrix`: [matrix] containing the values
-#' for the component to sum contribution plot (`$component.contribution.matrix`).\cr
-#'
-#' Matrix structure:\cr
-#' Column 1 and 2: time and `rev(time)` values\cr
-#' Additional columns are used for the components, two for each component,
-#' containing I0 and n0. The last columns `cont.` provide information on
-#' the relative component contribution for each time interval including the row
-#' sum for this values.
-#'
-#'
 #' @note
 #'
 #' **Beta version - This function has not been properly tested yet and**
 #' **should therefore not be used for publication purposes!**
 #'
-#' The pseudo-R^2 may not be the best parameter to describe the goodness of the
-#' fit. The trade off between the `n.components` and the pseudo-R^2 value
+#' The pseudo-R² may not be the best parameter to describe the goodness of the
+#' fit. The trade off between the `n.components` and the pseudo-R² value
 #' is currently not considered.
 #'
 #' The function **does not** ensure that the fitting procedure has reached a
 #' global minimum rather than a local minimum!
 #'
-#' @section Function version: 0.5.2
+#' @section Function version: 0.5.4
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -215,39 +186,41 @@ fit_CWCurve<- function(
   LED.wavelength = 470,
   cex.global = 0.6,
   sample_code = "Default",
-  output.path,
-  output.terminal = TRUE,
+  verbose = TRUE,
   output.terminalAdvanced = TRUE,
   plot = TRUE,
+  method_control = list(),
   ...
-){
-  ##TODO
-  ##remove output.path
+) {
+  .set_function_name("fit_CWCurve")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  # INTEGRITY CHECKS --------------------------------------------------------
-
-  ##INPUT OBJECTS
-  if(is(values, "RLum.Data.Curve") == FALSE & is(values, "data.frame") == FALSE){
-    stop("[fit_CWCurve()] Input object is not of type 'RLum.Data.Curve' or 'data.frame'!", call. = FALSE)
-  }
+  ## Integrity checks -------------------------------------------------------
 
+  .validate_class(values, c("RLum.Data.Curve", "data.frame"))
+  .validate_not_empty(values)
 
   if(is(values, "RLum.Data.Curve") == TRUE){
-
-    x <- values@data[,1]
-    y <- values@data[,2]
-
-    ##needed due to inconsistencies in the R code below
-    values <- data.frame(x,y)
-
-  }else{
-
-    ##set x and y values
-    x<-values[,1]
-    y<-values[,2]
-
+    values <- as.data.frame(values@data[, 1:2, drop = FALSE])
   }
 
+  ## set x and y values
+  x <- values[, 1]
+  y <- values[, 2]
+
+  if (sum(y > 0) == 0) {
+    .throw_error("'values' contains no positive counts")
+  }
+  if (any(order(x) != seq_along(x))) {
+    .throw_error("Time values are not ordered")
+  }
+  fit.method <- .validate_args(fit.method, c("port", "LM"))
+  .validate_positive_scalar(n.components.max, int = TRUE)
+  .validate_positive_scalar(fit.failure_threshold, int = TRUE)
+  .validate_logical_scalar(verbose)
+  .validate_logical_scalar(output.terminalAdvanced)
+  .validate_logical_scalar(plot)
+  .validate_class(method_control, "list")
 
   # Deal with extra arguments -----------------------------------------------
 
@@ -266,6 +239,11 @@ fit_CWCurve<- function(
   ylab <- if("ylab" %in% names(extraArgs)) {extraArgs$ylab} else
   {paste("OSL [cts/",round(max(x)/length(x), digits = 2)," s]",sep="")}
 
+  method_control <- modifyList(x = list(export.comp.contrib.matrix = FALSE),
+                               val = method_control)
+
+  if ("output.path" %in% names(extraArgs))
+    .throw_warning("Argument 'output.path' no longer supported, ignored")
 
   ##============================================================================##
   ## FITTING
@@ -280,16 +258,6 @@ fit_CWCurve<- function(
   }
   ##////equation used for fitting///(end)
 
-  ##////equation used for fitting////(start)
-  fit.equation.simple <- function(I0.i,lambda.i){
-    equation<-parse(
-      text=paste("I0[",I0.i,"]*exp(-lambda[",lambda.i,"]*x)",
-                 collapse="+",sep=""))
-    return(equation)
-  }
-  ##////equation used for fitting///(end)
-
-
   ##set formula elements for fitting functions
   ## the upper two funtions should be removed ... but chances are needed ... TODO
   ##////equation used for fitting////(start)
@@ -297,8 +265,9 @@ fit_CWCurve<- function(
 
     I0 <- paste0("I0.",1:n.components)
     lambda <- paste0("lambda.",1:n.components)
-    as.formula(paste0("y ~ ", paste(I0," * ", lambda, "* exp(-",lambda," * x)", collapse=" + ")))
-
+    stats::as.formula(paste0("y ~ ", paste(I0," * ", lambda,
+                                           "* exp(-",lambda," * x)",
+                                           collapse = " + ")))
   }
   ##////equation used for fitting///(end)
 
@@ -307,23 +276,22 @@ fit_CWCurve<- function(
 
     I0 <- paste0("I0.",1:n.components)
     lambda <- paste0("lambda.",1:n.components)
-    as.formula(paste0("y ~ ", paste(I0," * exp(-",lambda," * x)", collapse=" + ")))
-
+    stats::as.formula(paste0("y ~ ", paste(I0," * exp(-",lambda," * x)",
+                             collapse = " + ")))
   }
   ##////equation used for fitting///(end)
 
   ##set variables
-  fit.trigger <- TRUE #triggers if the fitting should stopped
+  keep.fitting <- TRUE # set to FALSE if the fitting should be stopped early
   n.components <- 1 #number of components used for fitting - start with 1
   fit.failure_counter <- 0 #counts the failed fitting attempts
 
   ##if n.components_max is missing, then it is Inf
   if(missing(n.components.max)==TRUE){n.components.max<-Inf}
 
-
   ##
   ##++++Fitting loop++++(start)
-  while(fit.trigger==TRUE & n.components <= n.components.max){
+  while(keep.fitting && n.components <= n.components.max) {
 
     ##(0) START PARAMETER ESTIMATION
     ##rough automatic start parameter estimation
@@ -335,13 +303,9 @@ fit_CWCurve<- function(
     ##lambda
     ##ensure that no values <=0 are included remove them for start parameter
     ##estimation and fit an linear function a first guess
-    if(min(y)<=0){
-      temp.values<-data.frame(x[-which(y<=0)], log(y[-which(y<=0)]))
-    }else{
-      temp.values<-data.frame(x, log(y))
-    }
+    temp.values <- data.frame(x[y > 0], log(y[y > 0]))
 
-    temp<-lm(temp.values)
+    temp <- stats::lm(temp.values)
     lambda<-abs(temp$coefficient[2])/nrow(values)
 
     k<-2
@@ -364,7 +328,6 @@ fit_CWCurve<- function(
                                             maxiter = 500
                                           )),
                                     silent = TRUE
-
       ))#end try
 
 
@@ -377,7 +340,7 @@ fit_CWCurve<- function(
                                         algorithm="port",
                                         na.action = "na.exclude",
                                         start=c(I0,lambda),
-                                        nls.control(
+                                        stats::nls.control(
                                           tol = 1,
                                           maxiter=100,
                                           warnOnly=FALSE,
@@ -386,14 +349,8 @@ fit_CWCurve<- function(
                                         lower=rep(0,n.components * 2)# set lower boundaries for components
       ), silent=TRUE# nls
       ))#end try
-
-    }else{
-
-      stop("[fit_CWCurve()] fit.method unknown.", call. = FALSE)
-
     }
 
-
     ##(3) FIT WITH THE FULL FUNCTION
     if(inherits(fit.try,"try-error") == FALSE){
 
@@ -428,7 +385,6 @@ fit_CWCurve<- function(
 
       }else{
 
-
         ##try fit
         fit.try<-suppressWarnings(try(nls(fit.formula(n.components),
                                           trace=fit.trace,
@@ -436,7 +392,7 @@ fit_CWCurve<- function(
                                           algorithm="port",
                                           na.action = "na.exclude",
                                           start=c(I0,lambda),
-                                          nls.control(
+                                          stats::nls.control(
                                             maxiter = 500,
                                             warnOnly = FALSE,
                                             minFactor = 1/4096
@@ -447,30 +403,22 @@ fit_CWCurve<- function(
 
       }#fit.method
     }
+    n.components <- n.components + 1
 
     ##count failed attempts for fitting
-    if(inherits(fit.try,"try-error")==FALSE){
-
+    if (!inherits(fit.try, "try-error")) {
       fit <- fit.try
-      n.components <- n.components + 1
 
     }else{
-
-      n.components<-n.components+1
       fit.failure_counter <- fit.failure_counter+1
-      if(n.components==fit.failure_counter & exists("fit")==FALSE){fit<-fit.try}}
-
+      if (!exists("fit")) {
+        fit <- fit.try
+      }
+    }
 
     ##stop fitting after a given number of wrong attempts
     if(fit.failure_counter>=fit.failure_threshold){
-
-      fit.trigger <- FALSE
-      if(!exists("fit")){fit <- fit.try}
-
-    }else if(n.components == n.components.max & exists("fit") == FALSE){
-
-      fit <- fit.try
-
+      keep.fitting <- FALSE
     }
 
   }##end while
@@ -481,6 +429,7 @@ fit_CWCurve<- function(
   ##============================================================================##
 
   ##grep parameters
+  output.table <- component.contribution.matrix <- NA
   if(inherits(fit,"try-error")==FALSE){
 
     parameters <- coef(fit)
@@ -503,7 +452,6 @@ fit_CWCurve<- function(
     ## Additional Calculation
     ##============================================================================##
 
-
     ## ---------------------------------------------
     ##calculate stimulation intensity Schmidt (2008)
 
@@ -512,7 +460,7 @@ fit_CWCurve<- function(
     ny<-299792458/(LED.wavelength/10^9) #frequency of light
     E<-h*ny
 
-    ##transform LED.power in W/cm^2
+    ## transform LED.power in W/cm²
     LED.power<-LED.power/1000
 
     ##gets stimulation intensity
@@ -533,7 +481,7 @@ fit_CWCurve<- function(
     pR<-round(1-RSS/TSS,digits=4)
 
     if(pR<0){
-      warning("pseudo-R^2 < 0!")
+      .throw_warning("pseudo-R^2 < 0!")
     }
 
     ## ---------------------------------------------
@@ -542,23 +490,29 @@ fit_CWCurve<- function(
     lambda.error<-rep(NA, n.components)
     I0.error<-rep(NA, n.components)
 
-    if(fit.calcError==TRUE){
-      ##option for confidence interval
-      values.confint<-confint(fit, level=0.68)
-      I0.confint<-values.confint[1:(length(values.confint[,1])/2),]
-      lambda.confint<-values.confint[((length(values.confint[,1])/2)+1):length(values.confint[,1]),]
-
-      ##error calculation
-      I0.error<-as.vector(abs(I0.confint[,1]-I0.confint[,2]))
-      lambda.error<-as.vector(abs(lambda.confint[,1]-lambda.confint[,2]))
+    ## option for confidence interval
+    if (fit.calcError) {
+      tryCatch({
+        values.confint <- stats::confint(fit, level = 0.68)
+        half <- nrow(values.confint) / 2
+        I0.confint <- values.confint[1:half, ]
+        lambda.confint <- values.confint[half + 1:half, ]
 
+        ## error calculation
+        I0.error <- abs(I0.confint[, 1] - I0.confint[, 2])
+        lambda.error <- abs(lambda.confint[, 1] - lambda.confint[, 2])
+      }, error = function(e) {
+        ## report the error from confint()
+        .throw_message("Computation of confidence interval failed: ",
+                       e$message)
+      })
     }#endif::fit.calcError
 
     ##============================================================================##
     ## Terminal Output
     ##============================================================================##
 
-    if (output.terminal==TRUE){
+    if (verbose) {
 
       ##print rough fitting information - use the nls() control for more information
       writeLines("\n[fit_CWCurve()]")
@@ -582,7 +536,7 @@ fit_CWCurve<- function(
     ##============================================================================##
     ## Terminal Output (advanced)
     ##============================================================================##
-    if (output.terminalAdvanced==TRUE && output.terminal==TRUE){
+    if (verbose && output.terminalAdvanced) {
 
       ##sum of squares
       writeLines(paste("pseudo-R^2 = ",pR,sep=""))
@@ -635,22 +589,6 @@ fit_CWCurve<- function(
       colnames(output.table)<-c("sample_code","n.components",
                                 output.tableColNames,"pseudo-R^2")
 
-      if(missing(output.path)==FALSE){
-
-        ##write file with just the header if the file not exists
-        if(file.exists(paste(output.path,"fit_CWCurve_Output_",sample_code,".csv",sep=""))==FALSE){
-          write.table(output.table,file=paste(output.path,"fit_CWCurve_Output_",
-                                              sample_code,".csv",sep=""), sep=";"
-                      ,row.names=FALSE)
-        }else{
-          write.table(output.table,file=paste(output.path,"fit_CWCurve_Output_",
-                                              sample_code,".csv",sep=""), sep=";"
-                      ,row.names=FALSE, append=TRUE, col.names=FALSE)
-
-        }#endif::for write option
-
-      }#endif::table output
-
       ##============================================================================##
       ## COMPONENT TO SUM CONTRIBUTION PLOT
       ##============================================================================##
@@ -733,7 +671,6 @@ fit_CWCurve<- function(
 
           matrixStats::rowDiffs(cbind(rev(component.contribution.matrix[,(x+1)]),
                          component.contribution.matrix[,x]))
-
         })
 
       ##append to existing matrix
@@ -749,16 +686,7 @@ fit_CWCurve<- function(
         paste(c("cont.c"),rep(1:n.components,each=1), sep=""),
         "cont.sum")
 
-
     }#endif :: (exists("fit"))
-
-  }else{
-
-    if (output.terminal==TRUE)
-      writeLines("[fit_CWCurve()] Fitting Error >> Plot without fit produced!")
-
-    output.table<-NA
-    component.contribution.matrix <- NA
   }
 
   ##============================================================================##
@@ -767,31 +695,35 @@ fit_CWCurve<- function(
   if(plot==TRUE){
 
     ##grep par parameters
-    par.default <- par(no.readonly = TRUE)
+    par.default <- par()[c("mfrow", "cex", "mar", "omi", "oma")]
+    on.exit(par(par.default), add = TRUE)
 
     ##set colors gallery to provide more colors
     col <- get("col", pos = .LuminescenceEnv)
 
     ##set plot frame
+    par(cex = cex.global)
     if(!inherits(fit, "try-error")){
       layout(matrix(c(1,2,3),3,1,byrow=TRUE),c(1.6,1,1), c(1,0.3,0.4),TRUE)
-      par(oma=c(1,1,1,1),mar=c(0,4,3,0),cex=cex.global)
-    }else{
-      par(cex=cex.global)
+      par(oma = c(1, 1, 1, 1), mar = c(0, 4, 3, 0))
     }
-
-
-    ##==uppper plot==##
+    ##== upper plot ==##
     ##open plot area
-
-    plot(NA,NA,
+    plot_check <- try(plot(NA, NA,
          xlim=c(min(x),max(x)),
-         ylim=if(log=="xy"){c(1,max(y))}else{c(0,max(y))},
-         xlab=if(!inherits(fit, "try-error")){""}else{xlab},
-         xaxt=if(!inherits(fit, "try-error")){"n"}else{"s"},
+         ylim = c(ifelse(log == "xy", 1, 0), max(y)),
+         xlab = ifelse(inherits(fit, "try-error"), xlab, ""),
+         xaxt = ifelse(inherits(fit, "try-error"), "s", "n"),
          ylab=ylab,
          main=main,
-         log=log)
+         log = log), silent = TRUE)
+
+    if (is(plot_check, "try-error")) {
+      ## reset the graphic device if plotting failed
+      .throw_message("Figure margins too large or plot area too small, ",
+                     "nothing plotted")
+      grDevices::dev.off()
+    } else {
 
     ##plotting measured signal
     points(x,y,pch=20, col="grey")
@@ -825,7 +757,7 @@ fit_CWCurve<- function(
       ##==lower plot==##
       ##plot residuals
       par(mar=c(4.2,4,0,0))
-      plot(x,residuals(fit),
+      plot_check2 <- try(plot(x,residuals(fit),
            xlim=c(min(x),max(x)),
            xlab=xlab,
            type="l",
@@ -833,7 +765,14 @@ fit_CWCurve<- function(
            ylab="Residual [a.u.]",
            lwd=2,
            log=if(log=="x" | log=="xy"){log="x"}else{""}
-      )
+      ), silent = TRUE)
+
+      if (is(plot_check2, "try-error")) {
+        ## reset the graphic device if plotting failed
+        .throw_message("Figure margins too large or plot area too small, ",
+                       "nothing plotted")
+        grDevices::dev.off()
+      } else {
 
       ##add 0 line
       abline(h=0)
@@ -864,18 +803,23 @@ fit_CWCurve<- function(
                 col = col[i+1])
       }
       rm(stepping)
+      } # end if (plot_check2)
 
-
+    } else {
+      if (verbose)
+        .throw_message("Fitting failed, plot without fit produced")
     }#end if try-error for fit
 
-    par(par.default)
-    rm(par.default)
+    } # end if (plot_check)
   }
 
   ##============================================================================##
   ## Return Values
   ##============================================================================##
 
+  if (!method_control$export.comp.contrib.matrix) {
+    component.contribution.matrix <- NA
+  }
   newRLumResults.fit_CWCurve <- set_RLum(
     class = "RLum.Results",
     data = list(
@@ -891,5 +835,4 @@ fit_CWCurve<- function(
   rm(component.contribution.matrix)
 
   invisible(newRLumResults.fit_CWCurve)
-
 }
