@@ -1,8 +1,9 @@
-#' Attempts to install the development version of the 'Luminescence' package
+#' @title Attempts to install the development version of the 'Luminescence' package
 #'
-#' This function is a convenient method for installing the development
+#' @description This function is a convenient method for installing the development
 #' version of the R package 'Luminescence' directly from GitHub.
 #'
+#' @details
 #' This function uses [Luminescence::github_branches][Luminescence::GitHub-API] to check
 #' which development branches of the R package 'Luminescence' are currently
 #' available on GitHub. The user is then prompted to choose one of the branches
@@ -12,9 +13,9 @@
 #' to install the desired development version of the package.
 #'
 #'
-#' If `force_install=TRUE` the functions checks if 'devtools' is available
+#' If `force_install = TRUE` the functions checks if 'devtools' is available
 #' and then attempts to install the chosen development branch via
-#' [devtools::remote-reexports].
+#' [devtools::install_github].
 #'
 #' @param force_install [logical] (*optional*):
 #' If `FALSE` (the default) the function produces and prints the required
@@ -36,7 +37,7 @@
 #' @md
 #' @export
 install_DevelopmentVersion <- function(force_install = FALSE) {
-
+  # nocov start
   message("\n[install_DevelopmentVersion]\n")
 
   # check which branches are currently available
@@ -73,7 +74,7 @@ install_DevelopmentVersion <- function(force_install = FALSE) {
 
     message("Please copy and run the following code in your R command-line:\n")
     if (!requireNamespace("devtools", quietly = TRUE))
-      message("install.packages('devtools')")
+      message("install.packages('devtools')") # nocov
 
     message(branches$INSTALL[as.numeric(index)], "\n")
 
@@ -110,7 +111,6 @@ install_DevelopmentVersion <- function(force_install = FALSE) {
 
     # install the development version
     devtools::install_github(paste0("r-lum/luminescence@", branch))
-
   }
-
+  # nocov end
 }
