@@ -1,32 +1,38 @@
 #' Create LaTex tables from data.frames and RLum objects
 #'
 #' This function takes a data.frame and returns a table in LaTex code that
-#' can be copied in any tex document.
+#' can be copied into any tex document.
 #'
 #' @param x [data.frame] or `RLum` object (**required**)
-#' 
+#'
 #' @param row.names currently unused
-#' 
+#'
 #' @param col.names currently unused
-#' 
-#' @param comments [logical] (*with default*): 
+#'
+#' @param comments [logical] (*with default*):
 #' insert LaTex comments
-#' 
-#' @param pos [character] (*with default*): 
-#' `character` of length one specifying the alignment of each column, e.g., 
+#'
+#' @param pos [character] (*with default*):
+#' `character` of length one specifying the alignment of each column, e.g.,
 #' pos'clr' for a three column data frame and center, left
 #'  and right alignment
-#' 
-#' @param digits [numeric] (*with default*): 
+#'
+#' @param digits [numeric] (*with default*):
 #' number of digits (numeric fields)
-#' 
-#' @param select [character] (*optional*): 
+#'
+#' @param rm.zero [logical] (*with default*): remove columns containing
+#' only zeros, however this might not be wanted in all cases
+#'
+#' @param select [character] (*optional*):
 #' a [character] vector passed to [subset]
-#' 
-#' @param split [integer] (*optional*): 
+#'
+#' @param split [integer] (*optional*):
 #' an [integer] specifying the number of individual tables
-#' the data frame is split into. Useful for wide tables. Currently unnused.
-#' 
+#' the data frame is split into. Useful for wide tables. Currently unused.
+#'
+#' @param tabular_only [logical] (*with default*): if `TRUE` only the tabular but not the
+#' table environment is returned. This gives a lot of additional flexibility at hand
+#'
 #' @param ... options: `verbose`
 #'
 #' @section TODO:
@@ -50,8 +56,10 @@
                             comments = TRUE,
                             pos = "c",
                             digits = 3,
+                            rm.zero = TRUE,
                             select,
                             split = NULL,
+                            tabular_only = FALSE,
                             ...) {
 
   args <- list(x = x,
@@ -60,7 +68,9 @@
                comments = comments,
                pos = pos,
                digits = digits,
+               rm.zero = rm.zero,
                split = split,
+               tabular_only = tabular_only,
                ... = ...)
   if (!missing(select))
     args$select <- select
@@ -80,6 +90,7 @@
                                          comments = TRUE,
                                          pos = "c",
                                          digits = 3,
+                                         rm.zero = TRUE,
                                          select,
                                          split = NULL,
                                          ...) {
@@ -89,30 +100,27 @@
     x <- get_RLum(x)$highlights
     x <- .digits(x, digits)
 
-    ##remove the first project ID, we don't want them in the LaTeX; the table does not fit
-    ##on a page
-    x[1] <- NULL
-
     ##remove columns containing zero values ... they add no information
-    x <- x[sapply(x, function(y){
-      y <- suppressWarnings(as.numeric(y))
-      if(anyNA(y) || sum(y, na.rm = TRUE) != 0){
-        TRUE
+    if(rm.zero){
+      x <- x[sapply(x, function(y){
+        y <- suppressWarnings(as.numeric(y))
+        if(anyNA(y) || sum(y, na.rm = TRUE) != 0){
+          TRUE
 
-      }else{
-        FALSE
+        }else{
+          FALSE
 
-      }
-    })]
+        }
+      })]
+    }
 
     ##add +/- symbol and remove the columns we don't need
     fields.w.error <- (grep(names(x), pattern = "err", fixed = TRUE) - 1)
 
     for(i in fields.w.error)
-      x[ ,i] <- paste0(x[ ,i], "\\,$\\pm$\\,", x[ ,i + 1])
+      x[ ,i] <- paste0(x[ ,i], "\\,$\\pm$\\,", trimws(x[ ,i + 1]))
     x <- x[-c(fields.w.error + 1)]
 
-
     ##create latex table
     text <- .as.latex.table(x, comments = comments, pos = pos, split = split, ...)
 
@@ -121,12 +129,11 @@
 
     ##exchange columns ... or delete them at all (2nd step)
 
-        ##Sample ID
-        text[[1]][7] <-  "\t\\multicolumn{1}{p{2cm}}{\\centering \\textbf{ID}} & "
-
-        ##Mineral
-        text[[1]][8] <-  "\t\\multicolumn{1}{p{0.5cm}}{\\centering \\textbf{M.} } & "
-
+      ##Mineral ID
+      for(i in 1:length(text)){
+        text[[i]][grepl(pattern = "Mineral", x = text[[i]], fixed = TRUE)] <-
+          "\t\\multicolumn{1}{p{0.5cm}}{\\centering \\textbf{M.}} & "
+      }
 
     ##put things again together (single character)
     text <- paste(text[[1]], collapse = "\n")
@@ -143,12 +150,10 @@
     text <- gsub(pattern = "Internal \\\\ doserate", replacement = "$\\dot{D}_{int.}$", x = text, fixed = TRUE)
     text <- gsub(pattern = "Environmental \\\\ Dose \\\\ Rate", replacement = "$\\dot{D}_{env.}$", x = text, fixed = TRUE)
 
-
-    ##retrun result
+    ## return result
     return(text)
 
   }# EndOf::use_DRAC
-
 }
 
 ################################################################################
@@ -162,19 +167,19 @@
                                        digits = 3,
                                        select,
                                        split = NULL,
+                                       tabular_only = FALSE,
                                        ...) {
-  ## Integrity checks ----
-  if (!is.data.frame(x))
-    stop("x must be a data frame", call. = FALSE)
+  .set_function_name("as.latex.table.data.frame")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  ## Integrity tests --------------------------------------------------------
+  .validate_class(x, "data.frame")
   if (!is.null(col.names) && length(col.names) != ncol(x))
-    stop("length of col.names does not match the number of columns",
-         call. = FALSE)
+    .throw_error("Length of 'col.names' does not match the number of columns")
   if (!is.null(row.names) && length(row.names) != nrow(x))
-    stop("length of row.names does not match the number of rows",
-         call. = FALSE)
+    .throw_error("Length of 'row.names' does not match the number of rows")
   if (length(pos) != 1)
-    stop("length of pos does not match the number of columns",
-         call. = FALSE)
+    .throw_error("Length of 'pos' does not match the number of columns")
 
   ## Default settings ----
   options <- list(verbose = TRUE)
@@ -186,8 +191,8 @@
   if (!missing(select)) {
     is.name <- select %in% names(x)
     if (any(!is.name))
-      stop("Undefined columns selected. Please check provided column names in 'select'.",
-           call. = FALSE)
+      .throw_error("Undefined columns selected. Please check provided ",
+                   "column names in 'select'.")
     x <- subset(x, select = select)
   }
 
@@ -213,7 +218,6 @@
       colnames(x.chunk) <- names(x[i])
     }
 
-
     ## Comments ----
     tex.comment.usePackage <- ifelse(comments,
                                      "% add usepackage{adjustbox} to latex preamble \n",
@@ -238,24 +242,38 @@
                                      "\\\\ \n"))
     }
 
+    ## catch potential latex problems with underscores - after all are numbers, in can be only
+    ## on the ID
+    tex.table.rows <- gsub("_", "\\_", tex.table.rows, fixed = TRUE)
+
     ## Tex table ----
-    if (nchar(pos) != 1 && nchar(pos) != ncol(x))
-      pos <- "c"
-    if (!any(strsplit(pos, split = "")[[1]] %in% c("l", "c", "r")))
+    pos.chars <- unlist(strsplit(pos, split = ""))
+    if (!all(pos.chars %in% c("l", "c", "r")) ||
+        (length(pos.chars) != 1 && length(pos.chars) != ncol(x))) {
       pos <- "c"
+    }
     if (nchar(pos) == 1)
       pos <- paste0(rep(pos, ncol(x)), collapse = "")
 
-    tex.table.begin <- paste0("\\begin{table}[ht] \n",
-                              "  \\centering \n",
-                              "  \\begin{adjustbox}{max width=\\textwidth} \n",
-                              paste("  \\begin{tabular}{", pos, "}\n"),
-                              "     \\hline \n")
+    if(tabular_only){
+      tex.table.begin <- paste0(paste("  \\begin{tabular}{", pos, "}\n"),
+                                "     \\hline \n")
 
-    tex.table.end <-  paste0("     \\hline \n",
-                             "   \\end{tabular} \n",
-                             "   \\end{adjustbox} \n",
-                             "\\end{table}")
+      tex.table.end <-  paste0("     \\hline \n",
+                               "   \\end{tabular}")
+
+    }else{
+      tex.table.begin <- paste0("\\begin{table}[ht] \n",
+                                "  \\centering \n",
+                                "  \\begin{adjustbox}{max width=\\textwidth} \n",
+                                paste("  \\begin{tabular}{", pos, "}\n"),
+                                "     \\hline \n")
+
+      tex.table.end <-  paste0("     \\hline \n",
+                               "   \\end{tabular} \n",
+                               "   \\end{adjustbox} \n",
+                               "\\end{table}")
+    }
 
     tex.table <- paste0(tex.comment.usePackage,
                         tex.table.begin,
@@ -274,7 +292,7 @@
 }
 
 # This function takes a data.frame, checks each column and tries to
-# force the specified amount of digits if numeric or coercable to numeric
+# force the specified amount of digits if numeric or coerceable to numeric
 .digits <- function(x, digits) {
   for (i in 1:ncol(x)) {
     if (is.factor(x[ ,i]))
@@ -285,4 +303,4 @@
 
   }
   return(x)
-}
\ No newline at end of file
+}
