@@ -1,6 +1,6 @@
-##################################################################################
+##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
 ##                      METHODS FOR S3 GENERICS                                 ##
-##################################################################################
+##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++##
 
 ##CAUTION NOTE:
 ##(1) Please DO NOT access to the S4 objects by using the slots this causes inconsistent
@@ -18,8 +18,7 @@
 ##
 
 
-## -------------------- INTRODUCED WITH 0.5.0 ----------------------- ##
-
+##  INTRODUCED WITH 0.5.0 ##
 
 #' methods_RLum
 #'
@@ -54,7 +53,7 @@
 #' @param j [integer] (*optional*):
 #' column of the data matrix in the `RLum.Data.Curve` object
 #'
-#' @param value [numeric] **(required)**:
+#' @param value [numeric] (**required**):
 #' numeric value which replace the value in the `RLum.Data.Curve` object
 #'
 #' @param drop [logical] (*with default*):
@@ -66,10 +65,10 @@
 #' of the object structure. Works also on [RLum.Analysis-class] objects.
 #'
 #' @param row.names [logical] (*with default*):
-#' enables or disables row names (`as.data.frame`)
+#' enable/disable row names (`as.data.frame`).
 #'
 #' @param recursive [logical] (*with default*):
-#' enables or disables further sub-setting (`unlist`)
+#' enable/disable further sub-setting (`unlist`).
 #'
 #' @param optional [logical] (*with default*):
 #' logical. If TRUE, setting row names and converting column names
@@ -91,9 +90,8 @@
 #' @name methods_RLum
 NULL
 
-####################################################################################################
-# methods for generic: plot()
-# ##################################################################################################
+# plot()------------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @method plot list
 #' @export
@@ -104,11 +102,10 @@ plot.list <- function(x, y, ...) {
   else {
     if (missing(y))
       y <- NULL
-    plot.default(x, y, ...)
+    graphics::plot.default(x, y, ...)
   }
 }
 
-
 #' @rdname methods_RLum
 #' @method plot RLum.Results
 #' @export
@@ -137,11 +134,9 @@ plot.RLum.Data.Image <- function(x, y, ...) plot_RLum(object = x, ...)
 #' @rdname methods_RLum
 #' @method plot Risoe.BINfileData
 #' @export
-plot.Risoe.BINfileData <- function(x, y, ...) plot_Risoe.BINfileData(BINfileData = x, ...)
+plot.Risoe.BINfileData <- function(x, y, ...) plot_Risoe.BINfileData(data = x, ...)
 
-####################################################################################################
-# methods for generic: hist()
-# ##################################################################################################
+# hist() -----------------------------------------------------------------------
 
 #' @rdname methods_RLum
 #' @export
@@ -149,7 +144,7 @@ hist.RLum.Results <- function(x, ...) plot_Histogram(data = x, ...)
 
 #' @rdname methods_RLum
 #' @export
-hist.RLum.Data.Image <- function(x, ...) hist(x =get_RLum(x)@data@values, ...)
+hist.RLum.Data.Image <- function(x, ...) hist(x = get_RLum(x), ...)
 
 #' @rdname methods_RLum
 #' @export
@@ -160,10 +155,8 @@ hist.RLum.Data.Curve <- function(x, ...) hist(as(get_RLum(x),"matrix")[,2])
 hist.RLum.Analysis <- function(x, ...) lapply(1:length_RLum(x), function(z){
   hist(as(get_RLum(x, record.id = z, ...),"matrix")[,2])})
 
-####################################################################################################
-# methods for generic: summary()
-# ##################################################################################################
-# methods for generic: summary()
+# summary() --------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @method summary RLum.Results
 #' @export
@@ -177,7 +170,7 @@ summary.RLum.Analysis <- function(object, ...) lapply(object@records, function(x
 #' @rdname methods_RLum
 #' @method summary RLum.Data.Image
 #' @export
-summary.RLum.Data.Image <- function(object, ...) summary(object@data@data@values)
+summary.RLum.Data.Image <- function(object, ...) summary(object@data)
 
 # summary.RLum.Data.Spectrum <- function(object, ...)
 
@@ -186,9 +179,8 @@ summary.RLum.Data.Image <- function(object, ...) summary(object@data@data@values
 #' @export
 summary.RLum.Data.Curve <- function(object, ...) summary(object@data, ...)
 
-####################################################################################################
-# methods for generic: subset()
-# ##################################################################################################
+# subset() ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @method subset Risoe.BINfileData
 #'
@@ -198,10 +190,11 @@ summary.RLum.Data.Curve <- function(object, ...) summary(object@data, ...)
 #' @md
 #' @export
 subset.Risoe.BINfileData <- function(x, subset, records.rm = TRUE, ...) {
+  .set_function_name("subset.Risoe.BINfileData")
+  on.exit(.unset_function_name(), add = TRUE)
 
   if(length(list(...)))
-    warning(paste("Argument not supported and skipped:", names(list(...))))
-
+    .throw_warning("Argument not supported and skipped:", names(list(...)))
 
   ##select relevant rows
   sel <- tryCatch(eval(
@@ -210,7 +203,8 @@ subset.Risoe.BINfileData <- function(x, subset, records.rm = TRUE, ...) {
     enclos = parent.frame()
   ),
   error = function(e) {
-    stop("\n\nInvalid subset options. \nValid terms are: ", paste(names(x@METADATA), collapse = ", "))
+    .throw_error("\nInvalid subset options, valid terms are: ",
+                 .collapse(names(x@METADATA)))
   })
 
   ##probably everything is FALSE now?
@@ -223,14 +217,11 @@ subset.Risoe.BINfileData <- function(x, subset, records.rm = TRUE, ...) {
 
     } else{
       return(NULL)
-
     }
   }else{
     x@METADATA[["SEL"]] <- sel
     return(x)
-
   }
-
 }
 
 #' @rdname methods_RLum
@@ -239,10 +230,13 @@ subset.Risoe.BINfileData <- function(x, subset, records.rm = TRUE, ...) {
 subset.RLum.Analysis <- function(x, subset = NULL, ...) {
   do.call(get_RLum, list(object = x, drop = FALSE, subset = substitute(subset), env = parent.frame())) }
 
+# bin() ------------------------------------------------------------------------
+#' @rdname methods_RLum
+#' @export
+bin <- function(x, ...) {
+  UseMethod("bin")
+}
 
-####################################################################################################
-# methods for generic: bin()
-# ##################################################################################################
 #' @rdname methods_RLum
 #' @export
 bin.RLum.Data.Curve <- function(x, bin_size = 2, ...) bin_RLum.Data(x, bin_size = bin_size)
@@ -253,9 +247,8 @@ bin.RLum.Data.Spectrum <- function(x, bin_size.row = 1, bin_size.col = 1, ...){
   bin_RLum.Data(x, bin_size.row = bin_size.row, bin_size.col = bin_size.col)
 }
 
-####################################################################################################
-# methods for generic: length()
-# ##################################################################################################
+# length() ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 length.RLum.Results <- function(x, ...) length_RLum(x)
@@ -272,10 +265,8 @@ length.RLum.Data.Curve <- function(x, ...) length_RLum(x)
 #' @export
 length.Risoe.BINfileData <- function(x, ...) length(x@METADATA$ID)
 
-####################################################################################################
-# methods for generic: dim()
-# ##################################################################################################
-# methods for generic: dim()
+# dim() ------------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 dim.RLum.Data.Curve <- function(x) dim(as(x, "matrix"))
@@ -284,16 +275,15 @@ dim.RLum.Data.Curve <- function(x) dim(as(x, "matrix"))
 #' @export
 dim.RLum.Data.Spectrum <- function(x) dim(as(x, "matrix"))
 
-####################################################################################################
-# methods for generic: rep()
-# ##################################################################################################
+# rep() -------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 rep.RLum <- function(x, ...) replicate_RLum(x, ...)
 
-####################################################################################################
-# methods for generic: name()
-# ##################################################################################################
+
+# names() -----------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 names.RLum.Data.Curve <- function(x, ...) names_RLum(x)
@@ -318,16 +308,16 @@ names.RLum.Results <- function(x, ...) names_RLum(x)
 #' @export
 names.Risoe.BINfileData <- function(x)  as.character(x@METADATA$LTYPE)
 
-####################################################################################################
-# methods for generic: row.name()
-# ##################################################################################################
+
+# row.name() --------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 row.names.RLum.Data.Spectrum <- function(x, ...) rownames(as(x, "matrix"))
 
-####################################################################################################
-# methods for generic: as.data.frame()
-# ##################################################################################################
+
+# as.data.frame() ---------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 as.data.frame.RLum.Data.Curve <- function(x, row.names = NULL, optional = FALSE, ...) as(x, "data.frame")
@@ -342,7 +332,7 @@ as.data.frame.RLum.Data.Spectrum <- function(x,  row.names = NULL, optional = FA
 as.data.frame.Risoe.BINfileData <- function(x,  row.names = NULL, optional = FALSE, ...) {
 
   ## set matrix
-  m <- matrix(NA, ncol = max(sapply(x@DATA, length)), nrow = length(x@DATA))
+  m <- matrix(NA, ncol = max(lengths(x@DATA)), nrow = length(x@DATA))
 
   ## rename columns
   colnames(m) <- paste0("C",1:ncol(m))
@@ -352,13 +342,11 @@ as.data.frame.Risoe.BINfileData <- function(x,  row.names = NULL, optional = FAL
 
   ##convert to data.frame and bind
   cbind(x@METADATA, as.data.frame(m))
-
 }
 
 
-####################################################################################################
-# methods for generic: as.list()
-# ##################################################################################################
+# as.list() ---------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 as.list.RLum.Results <- function(x, ...) as(x, "list")
@@ -371,13 +359,17 @@ as.list.RLum.Data.Curve <- function(x, ...) as(x, "list")
 #' @export
 as.list.RLum.Data.Image <- function(x, ...) as(x, "list")
 
+#' @rdname methods_RLum
+#' @export
+as.list.RLum.Data.Spectrum <- function(x, ...) as(x, "list")
+
 #' @rdname methods_RLum
 #' @export
 as.list.RLum.Analysis <- function(x, ...) as(x, "list")
 
-####################################################################################################
-# methods for generic: as.matrix()
-# ##################################################################################################
+
+# as.matrix() -------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 as.matrix.RLum.Data.Curve <- function(x, ...) as(x, "matrix")
@@ -391,9 +383,9 @@ as.matrix.RLum.Data.Spectrum <- function(x, ...) as(x, "matrix")
 as.matrix.RLum.Data.Image <- function(x, ...) as(x, "matrix")
 # for RLum.Results ... makes no sense and may yield in unpredictable behaviour
 
-####################################################################################################
-# methods for generic: is()
-####################################################################################################
+
+# is() --------------------------------------------------------------------
+
 #For this function no S4 method was written, as this would come at the cost of performance and
 #is totally unnecessary
 
@@ -425,16 +417,15 @@ is.RLum.Analysis <- function(x, ...) is(x, "RLum.Analysis")
 #' @export
 is.RLum.Results <- function(x, ...) is(x, "RLum.Results")
 
-####################################################################################################
-# methods for generic: merge()
-####################################################################################################
+# merge() -----------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 merge.RLum <- function(x, y, ...) merge_RLum(append(list(...), values = c(x, y)))
 
-####################################################################################################
-# methods for generic: unlist()
-####################################################################################################
+
+# unlist() ----------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @method unlist RLum.Analysis
 #' @export
@@ -448,14 +439,12 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
 
   }else{
     return(temp)
-
   }
-
 }
 
-####################################################################################################
-# methods for generic: `+`
-####################################################################################################
+
+# `+` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #'
 #' @examples
@@ -471,30 +460,46 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
 #' @export
 `+.RLum.Data.Curve` <- function(x, y) merge_RLum(list(x, y), merge.method = "sum")
 
-####################################################################################################
-# methods for generic: `-`
-####################################################################################################
+#' @rdname methods_RLum
+#' @export
+`+.RLum.Data.Spectrum` <- function(x, y) merge_RLum(list(x, y), merge.method = "sum")
+
+
+# `-` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `-.RLum.Data.Curve` <- function(x, y) merge_RLum(list(x, y), merge.method = "-")
 
-####################################################################################################
-# methods for generic: `*`
-####################################################################################################
+#' @rdname methods_RLum
+#' @export
+`-.RLum.Data.Spectrum` <- function(x, y) merge_RLum(list(x, y), merge.method = "-")
+
+
+# `*` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `*.RLum.Data.Curve` <- function(x, y) merge_RLum(list(x, y), merge.method = "*")
 
-####################################################################################################
-# methods for generic: `/`
-####################################################################################################
+#' @rdname methods_RLum
+#' @export
+`*.RLum.Data.Spectrum` <- function(x, y) merge_RLum(list(x, y), merge.method = "*")
+
+
+# `/` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `/.RLum.Data.Curve` <- function(x, y) merge_RLum(list(x, y), merge.method = "/")
 
-####################################################################################################
-# methods for generic: `[`
-####################################################################################################
+#' @rdname methods_RLum
+#' @export
+`/.RLum.Data.Spectrum` <- function(x, y) merge_RLum(list(x, y), merge.method = "/")
+
+
+# `[` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `[.RLum.Data.Curve` <- function(x,y,z, drop = TRUE) {as(x, "matrix")[y,z, drop = drop]}
@@ -515,7 +520,6 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
 
   } else{
     get_RLum(x, record.id = i, drop = drop)
-
   }
 }
 
@@ -524,9 +528,8 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
 `[.RLum.Results` <- function(x, i, drop = TRUE) {get_RLum(x, data.object = i, drop = drop)}
 
 
-####################################################################################################
-# methods for generic: `[<-`
-####################################################################################################
+# `[<-` -------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `[<-.RLum.Data.Curve` <- function(x, i, j, value){
@@ -534,9 +537,9 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
   return(x)
 }
 
-####################################################################################################
-# methods for generic: `[[`
-####################################################################################################
+
+# `[[` --------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `[[.RLum.Analysis` <- function(x, i) {
@@ -553,9 +556,9 @@ unlist.RLum.Analysis <- function(x, recursive = TRUE, ...){
 #' @export
 `[[.RLum.Results` <- function(x, i) {get_RLum(x, data.object = i)}
 
-####################################################################################################
-# methods for generic: `$`
-####################################################################################################
+
+# `$` ---------------------------------------------------------------------
+
 #' @rdname methods_RLum
 #' @export
 `$.RLum.Data.Curve` <- function(x, i) {get_RLum(x, info.object = i)}
