@@ -1,26 +1,36 @@
-#'Create a Dose-Response Curve Summary Plot
+#' @title Create a Dose-Response Curve Summary Plot
 #'
-#'While analysing OSL SAR or pIRIR-data the view on the data is limited usually to one
-#'dose-response curve (DRC) at the time for one aliquot. This function overcomes this limitation
-#'by plotting all DRC from an [RLum.Results-class] object created by the function [analyse_SAR.CWOSL]
-#'in one single plot.
+#' @description
+#' While analysing OSL SAR or pIRIR-data the view on the data is usually
+#' limited to one dose-response curve (DRC) at the time for one aliquot. This
+#' function overcomes this limitation by plotting all DRCs from an
+#' [RLum.Results-class] object created by [analyse_SAR.CWOSL] in one single
+#' plot.
 #'
-#'If you want plot your DRC on an energy scale (dose in Gy), you can either use the option `source_dose_rate` provided
-#'below or your can SAR analysis with the dose points in Gy (better axis scaling).
+#' If you want plot your DRC on an energy scale (dose in Gy), you can either
+#' use option `source_dose_rate` or perform your SAR analysis with the dose
+#' points in Gy (better axis scaling).
 #'
-#'@param object [RLum.Results-class] object (**required**): input object created by the function [analyse_SAR.CWOSL]. The input object can be provided as [list].
+#' @param object [RLum.Results-class] (**required**): input object created by
+#' [analyse_SAR.CWOSL]. The input object can be provided as [list].
 #'
-#'@param source_dose_rate [numeric] (*optional*): allows to modify the axis and show values in Gy, instead seconds. Only a single numerical values is allowed.
+#' @param source_dose_rate [numeric] (*optional*): allows to modify the axis
+#' and show values in Gy, instead seconds. Only a single numerical value is
+#' allowed.
 #'
-#'@param sel_curves [numeric] (optional): id of the curves to be plotting in its occurring order. A sequence can
-#'be provided for selecting, e.g., only every 2nd curve from the input object
+#' @param sel_curves [numeric] (*optional*): id of the curves to be plotted in
+#' its occurring order. A sequence can be provided for selecting, e.g., only
+#' every 2nd curve from the input object.
 #'
-#'@param show_dose_points [logical] (with default): enable or disable plot of dose points in the graph
+#' @param show_dose_points [logical] (*with default*): enable/disable plotting
+#' of dose points in the graph.
 #'
-#'@param show_natural [logical] (with default): enable or disable the plot of the natural `Lx/Tx` values
+#' @param show_natural [logical] (*with default*): enable/disable the plot
+#' of the natural `Lx/Tx` values.
 #'
-#'@param n [integer] (with default): the number of x-values used to evaluate one curve object. Large numbers slow
-#'down the plotting process and are usually not needed
+#' @param n [integer] (*with default*): number of x-values used to evaluate
+#' one curve object. Large numbers slow down the plotting process and are
+#' usually not needed.
 #'
 #'@param ... Further arguments and graphical parameters to be passed. In particular: `main`, `xlab`, `ylab`, `xlim`, `ylim`, `lty`, `lwd`, `pch`, `col.pch`, `col.lty`, `mtext`
 #'
@@ -82,8 +92,9 @@ plot_DRCSummary <- function(
   show_natural = FALSE,
   n = 51L,
   ...
-){
-
+) {
+  .set_function_name("plot_DRCSummary")
+  on.exit(.unset_function_name(), add = TRUE)
 
 # Self-call -----------------------------------------------------------------------------------
 if(inherits(object, "list")){
@@ -91,21 +102,21 @@ if(inherits(object, "list")){
   ##catch ... arguments
   plot_settings <- list(...)
 
-  ##expand arguments
+  ## expand input arguments
   if("main" %in% names(list(...))){
-    main <- as.list(rep(list(...)[["main"]], length(object)))
+    main <- .listify(list(...)[["main"]], length(object))
 
     ##filter main from the ... argument list otherwise we will have a collusion
     plot_settings["main" %in% names(plot_settings)] <- NULL
 
   }else{
-    main <- as.list(rep("DRC", length(object)))
-
+    main <- .listify("DRC", length(object))
   }
 
-  results <- lapply(1:length(object), function(o){
+  results <- lapply(seq_along(object), function(o) {
     plot_DRCSummary(
       object = object[[o]],
+      source_dose_rate = source_dose_rate,
       sel_curves = sel_curves,
       show_dose_points = show_dose_points,
       show_natural = show_natural,
@@ -113,17 +124,14 @@ if(inherits(object, "list")){
       main = main[[o]],
       ... = plot_settings
       )
-
   })
 
   ##return merged object
   return(results)
-
 }
 
-# Check input ---------------------------------------------------------------------------------
-  if(!inherits(object, "RLum.Results"))
-    stop("[plot_DRCSummary()] The input is not of class 'RLum.Results'!",call. = FALSE)
+  ## Integrity checks -------------------------------------------------------
+  .validate_class(object, "RLum.Results")
 
 # Extract data from object --------------------------------------------------------------------
   ##get data from RLum.Results object
@@ -136,7 +144,7 @@ if(inherits(object, "list")){
       if(min(sel_curves) < 1 ||
          max(sel_curves) > length(object@data$Formula) ||
          length(sel_curves) > length(object@data$Formula)){
-        warning("[plot_DRCSummary()] 'sel_curves' out of bounds, reset to full dataset.", call. = FALSE, immediate. = TRUE)
+        .throw_warning("'sel_curves' out of bounds, reset to full dataset")
         sel_curves <- 1:length(object@data$Formula)
       }
 
@@ -144,8 +152,8 @@ if(inherits(object, "list")){
 
     ## check the whether the fitting was all the same
     if(length(unique(object@data[["data"]][["Fit"]])) != 1)
-      stop("[plot_DRCSummary()] I can only visualise dose-response curves based on the same fitting equation!",
-           call. = FALSE)
+      .throw_error("I can only visualise dose-response curves based ",
+                   "on the same fitting equation")
 
     ##get DRC
     DRC <- object@data$Formula[sel_curves]
@@ -167,10 +175,8 @@ if(inherits(object, "list")){
    })[sel_curves]
 
   }else{
-    stop(
-      "[plot_DRCSummary()] 'object' was created by none supported function, cf. manual for allowed originators!",
-      call. = FALSE)
-
+    .throw_error("'object' was not created by a supported function, ",
+                 "see the manual for allowed originators")
   }
 
 # Plotting ------------------------------------------------------------------------------------
@@ -240,7 +246,6 @@ if(inherits(object, "list")){
         col = plot_settings$col.pch[[i]],
         pch = plot_settings$pch[[i]]
       )
-
     }
 
     ##plot dose points
@@ -255,16 +260,15 @@ if(inherits(object, "list")){
         col = plot_settings$col.pch[[i]],
         pch = plot_settings$pch[[i]]
       )
-
     }
 
     ##plot lines
     x <- seq(min(plot_settings$xlim),max(plot_settings$xlim), length.out = n)
     y <- eval(DRC[[i]])
 
-    if (any(is.na(y)) || any(is.nan(y))) {
-      warning("[plot_DRCSummary()] Dose response curve ", i, " is NA/NaN and was removed before plotting.",
-              call. = FALSE)
+    if (anyNA(y) || any(is.nan(y))) {
+      .throw_warning("Dose response curve ", i, " contains NA/NaN values, ",
+                     "curve removed before plotting")
       next
     }
 
@@ -275,7 +279,6 @@ if(inherits(object, "list")){
       lwd = plot_settings$lwd,
       lty = plot_settings$lty[[i]]
     )
-
   }
 
   ## Results -------------------------------------------------------------------
