@@ -1,6 +1,6 @@
-#' Plot function for an RLum.Analysis S4 class object
+#' @title Plot function for an RLum.Analysis S4 class object
 #'
-#' The function provides a standardised plot output for curve data of an
+#' @description The function provides a standardised plot output for curve data of an
 #' RLum.Analysis S4 class object
 #'
 #' The function produces a multiple plot output. A file output is recommended
@@ -15,11 +15,10 @@
 #' `OSL` and `IRSL`.
 #'
 #' Please note: The curve transformation within this functions works roughly,
-#' i.e. every IRSL or OSL curve is transformed, without considerung whether it
+#' i.e. every IRSL or OSL curve is transformed, without considering whether it
 #' is measured with the PMT or not! However, for a fast look it might be
 #' helpful.
 #'
-#'
 #' @param object [RLum.Analysis-class] (**required**):
 #' S4 object of class `RLum.Analysis`
 #'
@@ -37,8 +36,8 @@
 #' tries to find a value.
 #'
 #' @param abline [list] (*optional*):
-#' allows to add ablines to the plot. Argument are provided
-#' in a list and will be forwared to the function [abline],
+#' allows to add ab-lines to the plot. Argument are provided
+#' in a list and will be forward to the function [abline],
 #' e.g., `list(v = c(10, 100))` adds two vertical lines add 10 and 100 to all
 #' plots. In contrast `list(v = c(10), v = c(100)` adds a vertical at 10 to
 #' the first and a vertical line at 100 to the 2nd plot.
@@ -46,21 +45,28 @@
 #' @param combine [logical] (*with default*):
 #' allows to combine all [RLum.Data.Curve-class] objects in one single plot.
 #'
-#' @param curve.transformation [character] (*optional*):
+#' @param records_max [numeric] (*optional*): limits number of records
+#' shown if `combine = TRUE`. Shown are always the first and the last curve,
+#' the other number of curves to be shown a distributed evenly, this may result
+#' in fewer curves plotted as specified. This parameter has only
+#' an effect for  n > 2.
+#'
+#' @param curve.transformation [character] (*with default*):
 #' allows transforming CW-OSL and CW-IRSL curves to pseudo-LM curves via
 #' transformation functions. Allowed values are: `CW2pLM`, `CW2pLMi`,
-#' `CW2pHMi` and `CW2pPMi`. See details.
+#' `CW2pHMi` and `CW2pPMi`, see details. If set to `None` (default), no
+#' transformation is applied.
 #'
-#' @param plot.single [logical] (*with default*):
+#' @param plot_singlePanels [logical] (*with default*):
 #' global par settings are considered, normally this should end in one plot per page
 #'
 #' @param ... further arguments and graphical parameters will be passed to
 #' the `plot` function.
 #'
 #' Supported arguments: `main`, `mtext`, `log`, `lwd`, `lty` `type`, `pch`, `col`,
-#' `norm`, `xlim`,`ylim`, `xlab`, `ylab`...
+#' `norm` (see [plot_RLum.Data.Curve]), `xlim`,`ylim`, `xlab`, `ylab`, ...
 #'
-#' and for `combine = TRUE` also: `sub`, `legend`, `legend.text`, `legend.pos`
+#' and for `combine = TRUE` also: `sub_title`, `legend`, `legend.text`, `legend.pos`
 #' (typical plus 'outside'), `legend.col`, `smooth`.
 #'
 #' All arguments can be provided as `vector` or `list` to gain in full control
@@ -73,10 +79,10 @@
 #' way you might expect them to work. This function was designed to serve as an overview
 #' plot, if you want to have more control, extract the objects and plot them individually.
 #'
-#' @section Function version: 0.3.11
+#' @section Function version: 0.3.16
 #'
 #' @author
-#' Sebastian Kreutzer, IRAMAT-CRP2A, Université Bordeaux Montaigne (France)
+#' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
 #' @seealso [plot], [plot_RLum], [plot_RLum.Data.Curve]
 #'
@@ -119,27 +125,22 @@ plot_RLum.Analysis <- function(
   ncols,
   abline = NULL,
   combine = FALSE,
-  curve.transformation,
-  plot.single = FALSE,
+  records_max = NULL,
+  curve.transformation = "None",
+  plot_singlePanels = FALSE,
   ...
-){
+) {
+  .set_function_name("plot_RLum.Analysis")
+  on.exit(.unset_function_name(), add = TRUE)
 
-  # Integrity check ----------------------------------------------------------------------------
+  ## Integrity checks -------------------------------------------------------
 
-  ##check if object is of class RLum.Analysis (lists are handled via plot_RLum())
-  if (!is(object, "RLum.Analysis")) {
-    stop("[plot_RLum.Analysis()] Input object is not of type 'RLum.Analysis'")
-
-  }
-
-  # Make selection if wanted  -------------------------------------------------------------------
+  .validate_class(object, "RLum.Analysis")
 
   if(!is.null(subset)){
-
     ##check whether the user set the drop option and remove it, as we cannot work with it
     subset <- subset[!sapply(names(subset), function(x){"drop" %in% x})]
     object <- do.call(get_RLum, c(object = object, subset, drop = FALSE))
-
   }
 
   # Deal with additional arguments.  ------------------------------------------------------------
@@ -159,7 +160,7 @@ plot_RLum.Analysis <- function(
     pch = 1,
     col = "auto",
     norm = FALSE,
-    sub = NULL,
+    sub_title = NULL,
     cex = 1,
     legend = TRUE,
     legend.text = NULL,
@@ -170,109 +171,94 @@ plot_RLum.Analysis <- function(
 
   plot.settings <- modifyList(x = plot.settings, val = list(...), keep.null = TRUE)
 
+  ## deprecated argument
+  if ("plot.single" %in% names(list(...))) {
+    plot_singlePanels <- list(...)$plot.single
+    .throw_warning("'plot.single' is deprecated, use 'plot_singlePanels' ",
+                   "instead")
+  }
+
   ##try to find optimal parameters, this is however, a little bit stupid, but
   ##better than without any presetting
-
-  if(combine){
+  .validate_logical_scalar(combine)
+  if (combine && length(object@records) <= 1) {
+    combine <- FALSE
+    .throw_warning("'combine' can't be used with fewer than two curves, ",
+                   "reset to FALSE")
+  }
+  if (combine) {
+    sapply(object@records, function(x) {
+      if (!inherits(x, "RLum.Data.Curve")) {
+        .throw_error("'combine' is valid only for 'RLum.Data.Curve' objects")
+      }
+    })
     n.plots <- length(unique(as.character(structure_RLum(object)$recordType)))
-
-  }else{
-    n.plots <- length_RLum(object)
-
   }
+  else
+    n.plots <- max(length_RLum(object), 1)
 
+  .validate_positive_scalar(nrows)
+  .validate_positive_scalar(ncols)
 
-  if (missing(ncols) | missing(nrows)) {
-    if (missing(ncols) & !missing(nrows)) {
-      if (n.plots  == 1) {
+  ## set appropriate values for nrows and ncols if not both specified
+  if (missing(nrows) || missing(ncols)) {
+    if (n.plots == 1) {
+      if (missing(nrows))
+        nrows <- 1
+      if (missing(ncols))
         ncols <- 1
 
-      } else{
+    } else { # n.plots > 1
+      if (missing(ncols)) {
         ncols <- 2
-
       }
-
+      if (missing(nrows)) {
+        if (n.plots <= 4) {
+          nrows <- ceiling(n.plots / 2)
+        } else {
+          nrows <- 3
+        }
+      }
     }
-    else if (!missing(ncols) & missing(nrows)) {
-      if (n.plots  == 1) {
-        nrows <- 1
-
-      }
-      else if (n.plots  > 1 & n.plots <= 4) {
-        nrows <- 2
-
-      } else{
-        nrows <- 3
-
-      }
-
-
-    } else{
-      if (n.plots  == 1) {
-        nrows <- 1
-        ncols <- 1
-
-      }
-      else if (n.plots  > 1 & n.plots  <= 2) {
-        nrows <- 1
-        ncols <- 2
-
-      } else if (n.plots  > 2 & n.plots <= 4) {
-        nrows <- 2
-        ncols <- 2
-
-      }
-      else{
-        nrows <- 3
-        ncols <- 2
-
-      }
-
-    }
-
   }
 
+  curve.transformation <- .validate_args(curve.transformation,
+                                         c("CW2pLM", "CW2pLMi",
+                                           "CW2pHMi", "CW2pPMi", "None"))
+  ## complete the function name
+  if (curve.transformation != "None") {
+    curve.transformation <- paste0("convert_", curve.transformation)
+  }
 
   # Plotting ------------------------------------------------------------------
-
   ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ##(1) NORMAL (combine == FALSE)
   ##+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-  if(!combine || length(object@records) == 1){
 
-    ##show warning message
-    if(combine & length(object@records) == 1){
-      warning("Nothing to combine, object contains a single curve.")
-
-    }
+  if (!combine) {
 
     ##grep RLum.Data.Curve or RLum.Data.Spectrum objects
-    temp <- lapply(1:length(object@records), function(x){
-
-      if(is(object@records[[x]], "RLum.Data.Curve") ||
-         is(object@records[[x]], "RLum.Data.Spectrum")){
-
-        object@records[[x]]
-
+    temp <- lapply(object@records, function(x) {
+      if (inherits(x, "RLum.Data.Curve") ||
+          inherits(x, "RLum.Data.Spectrum")) {
+        x
       }})
 
     ##calculate number of pages for mtext
-    if (length(temp) %% (nrows * ncols) > 0) {
+    if (length(temp) == 0 || length(temp) %% (nrows * ncols) > 0) {
       n.pages <- round(length(temp) / (nrows * ncols), digits = 0) + 1
 
     } else{
       n.pages <- length(temp) / (nrows * ncols)
-
     }
 
     ##set par
     par.default <- par("mfrow")
-    if(!plot.single){on.exit(par(mfrow = par.default))}
-    if(!plot.single) {
+    if (!plot_singlePanels) {
       par(mfrow = c(nrows, ncols))
+      on.exit(par(mfrow = par.default), add = TRUE)
     }
 
-
     ##expand plot settings list
     plot.settings <- lapply(setNames(1:length(plot.settings), names(plot.settings)),
                             function(x) {
@@ -281,20 +267,16 @@ plot_RLum.Analysis <- function(
 
                                   if(is(plot.settings[[x]], "list")){
                                     rep_len(plot.settings[[x]], length.out = length(temp))
-
                                   }else{
                                    rep_len(list(plot.settings[[x]]), length.out = length(temp))
-
                                   }
 
                                 }else{
                                   rep_len(plot.settings[[x]], length.out = length(temp))
-
                                 }
 
                               } else{
                                 plot.settings[[x]]
-
                               }
                             })
 
@@ -303,71 +285,56 @@ plot_RLum.Analysis <- function(
       abline.names <- rep_len(names(abline), length.out = length(temp))
       abline <- rep_len(abline, length.out = length(temp))
       names(abline) <- abline.names
-
     }
 
     ##apply curve transformation
-    for(i in 1:length(temp)){
+    for (i in seq_along(temp)) {
 
-      if(is(temp[[i]], "RLum.Data.Curve") == TRUE){
+      if (inherits(temp[[i]], "RLum.Data.Curve")) {
 
         ##set curve transformation if wanted
-        if((grepl("IRSL", temp[[i]]@recordType) | grepl("OSL", temp[[i]]@recordType)) &
-           !missing(curve.transformation)){
-
-          if(curve.transformation=="CW2pLM"){
-            temp[[i]] <- CW2pLM(temp[[i]])
-
-          }else if(curve.transformation=="CW2pLMi"){
-            temp[[i]] <- CW2pLMi(temp[[i]])
-
-          }else if(curve.transformation=="CW2pHMi"){
-            temp[[i]]<- CW2pHMi(temp[[i]])
-
-          }else if(curve.transformation=="CW2pPMi"){
-            temp[[i]] <- CW2pPMi(temp[[i]])
-
-          }else{
-            warning("Function for 'curve.transformation' is unknown. No transformation is performed.")
-
-          }
+        if (grepl("IRSL|OSL", temp[[i]]@recordType) &&
+            curve.transformation != "None") {
 
+          ## get the actual function from the parameter value and apply it
+          temp[[i]] <- get(curve.transformation)(temp[[i]])
         }
 
-
         ##check plot settings and adjust
         ##xlim
         if (!is.null(plot.settings$xlim)) {
           xlim.set <- plot.settings$xlim[[i]]
           if (plot.settings$xlim[[i]][1] < min(temp[[i]]@data[,1])) {
-            warning(paste0("[plot_RLum.Analysis()] min('xlim') < x-value range for curve #",i,"; reset to minimum."), call. = FALSE)
+            .throw_warning("min('xlim') < x-value range for curve #", i,
+                           ", reset to minimum")
             xlim.set[1] <- min(temp[[i]]@data[,1])
           }
           if (plot.settings$xlim[[i]][2] > max(temp[[i]]@data[,1])) {
-            warning(paste0("[plot_RLum.Analysis()] max('xlim') > x-value range for curve #",i,"; reset to maximum."), call. = FALSE)
+            .throw_warning("max('xlim') > x-value range for curve #", i,
+                           ", reset to maximum")
             xlim.set[2] <- max(temp[[i]]@data[,1])
           }
 
         }else{
           xlim.set <- plot.settings$xlim[[i]]
-
         }
 
         ##ylim
         if (!is.null(plot.settings$ylim)) {
           ylim.set <- plot.settings$ylim[[i]]
           if (plot.settings$ylim[[i]][1] < min(temp[[i]]@data[,2])) {
-            warning(paste0("[plot_RLum.Analysis()] min('ylim') < y-value range for curve #",i,"; reset to minimum."), call. = FALSE)
+            .throw_warning("min('ylim') < y-value range for curve #", i,
+                           ", reset to minimum")
             ylim.set[1] <- min(temp[[i]]@data[,2])
           }
           if (plot.settings$ylim[[i]][2] > max(temp[[i]]@data[,2])) {
-            warning(paste0("[plot_RLum.Analysis()] max('ylim') > y-value range for curve #",i,"; reset to maximum."), call. = FALSE)
+            .throw_warning("max('ylim') > y-value range for curve #", i,
+                           ", reset to maximum")
             ylim.set[2] <- max(temp[[i]]@data[,2])
           }
 
         }else{
           ylim.set <- plot.settings$ylim[[i]]
-
         }
 
         ##col
@@ -375,15 +342,12 @@ plot_RLum.Analysis <- function(
           col <- plot.settings$col[i]
 
         } else{
+          col <- "black"
           if (grepl("IRSL", temp[[i]]@recordType)) {
             col <- "red"
-          } else
-            if (grepl("OSL", temp[[i]]@recordType)) {
-              col <- "blue"
-            } else
-            {
-              col <- "black"
-            }
+          } else if (grepl("OSL", temp[[i]]@recordType)) {
+            col <- "blue"
+          }
         }
 
         ##main
@@ -397,7 +361,6 @@ plot_RLum.Analysis <- function(
         ##PLOT
         ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
         ##plot RLum.Data.Curve curve
-
           ##we have to do this via this way, otherwise we run into a duplicated arguments
           ##problem
           ##check and remove duplicated arguments
@@ -418,8 +381,10 @@ plot_RLum.Analysis <- function(
               lty = plot.settings$lty[[i]],
               xlim = xlim.set,
               ylim = ylim.set,
+              norm = plot.settings$norm,
               pch = plot.settings$pch[[i]],
               cex = plot.settings$cex[[i]],
+              legend.col = plot.settings$legend.col[[i]],
               smooth = plot.settings$smooth[[i]]
             ),
             list(...)
@@ -427,33 +392,25 @@ plot_RLum.Analysis <- function(
 
           arguments[duplicated(names(arguments))] <- NULL
 
-        ##call the fucntion plot_RLum.Data.Curve
+        ##call the function plot_RLum.Data.Curve
         do.call(what = "plot_RLum.Data.Curve", args = arguments)
         rm(arguments)
 
-
         ##add abline
         if(!is.null(abline[[i]])){
           do.call(what = "abline", args = abline[i])
-
         }
 
+      } else if(inherits(temp[[i]], "RLum.Data.Spectrum")) {
+        ## remove already provided arguments
+        args <- list(...)[!names(list(...)) %in% c("object", "mtext", "par.local", "main")]
 
-      } else if(is(temp[[i]], "RLum.Data.Spectrum")) {
-
-        plot_RLum.Data.Spectrum(temp[[i]],
-                                mtext =  if(!is.null(plot.settings$mtext[[i]])){
-                                  plot.settings$mtext[[i]]
-                                }else{
-                                  paste("#", i, sep = "")
-                                },
-                                par.local = FALSE,
-                                main = if(!is.null(plot.settings$main)){
-                                  plot.settings$main
-                                }else{
-                                  temp[[i]]@recordType
-                                })
-
+        do.call(what = "plot_RLum.Data.Spectrum", args = c(list(
+            object = temp[[i]],
+            mtext =  if(!is.null(plot.settings$mtext[[i]])) plot.settings$mtext[[i]] else paste("#", i, sep = ""),
+            par.local = FALSE,
+            main = if(!is.null(plot.settings$main)) plot.settings$main else temp[[i]]@recordType
+        ), args))
       }
 
     }#end for loop
@@ -464,42 +421,23 @@ plot_RLum.Analysis <- function(
     ##(2) NORMAL (combine == TRUE)
     ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     ##(1) check RLum objects in the set
-    object.list <- get_RLum(object)
-
-    sapply(1:length(object.list), function(x){
-      if(is(object.list[[x]])[1] != "RLum.Data.Curve"){
-        stop("[plot_RLum.Analysis()] Using 'combine' is limited to 'RLum.Data.Curve' objects.",
-             call. = FALSE)
-
-      }
-
-    })
 
     ##account for different curve types, combine similar
     temp.object.structure  <- structure_RLum(object)
     temp.recordType <- as.character(unique(temp.object.structure$recordType))
 
-
     ##change graphic settings
-    if(!plot.single){
+    if (!plot_singlePanels) {
       par.default <- par()[c("cex", "mfrow")]
-
       if(!missing(ncols) & !missing(nrows)){
         par(mfrow = c(nrows, ncols))
-
       }
-
-
-      ##this 2nd par request is needed as seeting mfrow resets the par settings ... this might
-      ##not be wanted
-      par(cex = plot.settings$cex[1])
-
-    }else{
-      par.default <- par()[c("cex")]
-      par(cex = plot.settings$cex)
-
+    } else {
+      par.default <- par()["cex"]
     }
-
+    ## this 2nd par request is needed as setting mfrow resets the par
+    ## settings ... this might not be wanted
+    par(cex = plot.settings$cex)
 
     ##expand plot settings list
     ##expand list
@@ -510,13 +448,10 @@ plot_RLum.Analysis <- function(
 
         }else{
           rep_len(list(plot.settings[[x]]), length.out = length(temp.recordType))
-
         }
 
-
       } else{
         plot.settings[[x]]
-
       }
     })
 
@@ -525,7 +460,6 @@ plot_RLum.Analysis <- function(
       abline.names <- rep_len(names(abline), length.out = length(temp.recordType))
       abline <- rep_len(abline, length.out = length(temp.recordType))
       names(abline) <- abline.names
-
     }
 
 
@@ -543,40 +477,31 @@ plot_RLum.Analysis <- function(
       object.list <-
         get_RLum(object, recordType = temp.recordType[k], recursive = FALSE)
 
-      ##prevent problems for non set argument
-      if (missing(curve.transformation)) {
-        curve.transformation <- "None"
+      ## limit number of records shown ... show always first and last;
+      ## distribute the rest
+      if(!is.null(records_max) && records_max[1] > 2){
+        records_show <- ceiling(seq(1,length(object.list), length.out = records_max))
+        object.list[(1:length(object.list))[-records_show]] <- NULL
       }
 
       ##transform values to data.frame and norm values
       temp.data.list <- lapply(1:length(object.list), function(x) {
-        ##set curve transformation if wanted
 
-        if (grepl("IRSL", object.list[[x]]@recordType) |
-            grepl("OSL", object.list[[x]]@recordType)) {
-          if (curve.transformation == "CW2pLM") {
-            object.list[[x]] <- CW2pLM(object.list[[x]])
-
-          }else if (curve.transformation == "CW2pLMi") {
-            object.list[[x]] <- CW2pLMi(object.list[[x]])
-
-          }else if (curve.transformation == "CW2pHMi") {
-            object.list[[x]] <- CW2pHMi(object.list[[x]])
-
-          }else if (curve.transformation == "CW2pPMi") {
-            object.list[[x]] <- CW2pPMi(object.list[[x]])
-
-          }
+        ## set curve transformation if wanted
+        if (grepl("IRSL|OSL", object.list[[x]]@recordType) &&
+            curve.transformation != "None") {
 
+          ## get the actual function from the parameter value and apply it
+          object.list[[x]] <- get(curve.transformation)(object.list[[x]])
         }
 
-
         temp.data <- as(object.list[[x]], "data.frame")
 
-        ##normalise curves if argument has been set
-        if (plot.settings$norm[[k]]) {
-          temp.data[,2] <- temp.data[,2] / max(temp.data[,2])
-
+        ## curve normalisation
+        if (plot.settings$norm[[k]][1] == TRUE ||
+            plot.settings$norm[[k]][1] %in% c("max", "last", "huot")) {
+          temp.data[[2]] <- .normalise_curve(temp.data[[2]],
+                                             plot.settings$norm[[k]])
         }
 
         return(temp.data)
@@ -595,15 +520,10 @@ plot_RLum.Analysis <- function(
       xlab <- if(!is.null(plot.settings$xlab[[k]])){
         plot.settings$xlab[[k]]
       }else{
-        switch(temp.recordType[[k]],
-               "TL" = "Temperature [\u00B0C]",
-               "IRSL" = "Time [s]",
-               "OSL" = "Time [s]",
-               "RF" = "Time [s]",
-               "RBR" = "Time [s]",
-               "LM-OSL" = "Time [s]"
-        )
-
+        if (temp.recordType[[k]] == "TL")
+          "Temperature [\u00B0C]"
+        else
+          "Time [s]"
       }
 
       ##ylab
@@ -629,8 +549,8 @@ plot_RLum.Analysis <- function(
         range(unlist(lapply(X = temp.data.list, FUN = function(x){
           range(x[,2])
         })))
-
       }
+
       if (grepl("y", plot.settings$log[[k]], ignore.case = TRUE))
         ylim[which(ylim == 0)] <- 1
 
@@ -640,31 +560,23 @@ plot_RLum.Analysis <- function(
 
       }else{
         col <- get("col", pos = .LuminescenceEnv)
-
       }
 
       ##if length of provided colours is < the number of objects, just one colour is supported
       if (length(col) < length(object.list)) {
         col <- rep_len(col, length(object.list))
-
       }
 
       ##lty
+      lty <- plot.settings$lty[[k]]
       if (length(plot.settings$lty[[k]]) < length(object.list)) {
         lty <- rep(plot.settings$lty[[k]], times = length(object.list))
-
-      }else{
-        lty <- plot.settings$lty[[k]]
-
       }
 
       ##pch
+      pch <- plot.settings$pch[[k]]
       if (length(plot.settings$pch[[k]]) < length(object.list)) {
         pch <- rep(plot.settings$pch[[k]], times = length(object.list))
-
-      }else{
-        pch <- plot.settings$pch[[k]]
-
       }
 
       ##legend.text
@@ -672,18 +584,16 @@ plot_RLum.Analysis <- function(
         plot.settings$legend.text[[k]]
 
       }else{
-        paste("Curve", 1:length(object.list))
+        if(!is.null(records_max) && records_max[1] > 2) {
+          paste("Curve", records_show)
 
+        } else {
+          paste("Curve", 1:length(object.list))
+        }
       }
 
       ##legend.col
-      legend.col <- if(!is.null(plot.settings$legend.col[[k]])){
-        plot.settings$legend.col[[k]]
-
-      }else{
-        NULL
-
-      }
+      legend.col <- plot.settings$legend.col[[k]]
 
       ##legend.pos
       legend.pos <- if(!is.null(plot.settings$legend.pos[[k]])){
@@ -691,7 +601,6 @@ plot_RLum.Analysis <- function(
 
       }else{
         "topright"
-
       }
 
       if (legend.pos == "outside") {
@@ -708,22 +617,22 @@ plot_RLum.Analysis <- function(
         xlab = xlab,
         ylab = ylab,
         log = plot.settings$log[[k]],
-        sub = plot.settings$sub[[k]]
+        sub = plot.settings$sub_title[[k]]
       )
 
       ##plot single curve values
       ## ...?Why using matplot is a bad idea: The channel resolution might be different
       for (n in 1:length(temp.data.list)) {
 
-
         ##smooth
         ##Why here again ... because the call differs from the one before, where the argument
         ##is passed to plot_RLum.Data.Curve()
         if(plot.settings$smooth[[k]]){
 
           k_factor <- ceiling(length(temp.data.list[[n]][, 2])/100)
-          temp.data.list[[n]][, 2] <- zoo::rollmean(temp.data.list[[n]][, 2],
-                                            k = k_factor, fill = NA)
+          temp.data.list[[n]][, 2] <- .smoothing(temp.data.list[[n]][, 2],
+                                                 k = k_factor, fill = NA,
+                                                 align = "center")
         }
 
         ##remove 0 values if plotted on a log-scale
@@ -742,7 +651,6 @@ plot_RLum.Analysis <- function(
             lty = lty[n],
             lwd = plot.settings$lwd[[k]]
           )
-
         }
 
         ##add points if requested
@@ -751,17 +659,13 @@ plot_RLum.Analysis <- function(
             temp.data.list[[n]],
             col = col[n],
             pch = pch[n],
-
           )
-
         }
-
       }
 
       ##add abline
       if(!is.null(abline[[k]])){
         do.call(what = "abline", args = abline[k])
-
       }
 
       ##mtext
@@ -773,16 +677,14 @@ plot_RLum.Analysis <- function(
         par(xpd = TRUE)
 
         # determine legend position on log(y) scale
+        ypos <- par()$usr[4]
         if (grepl("y", plot.settings$log[[k]], ignore.case = TRUE))
           ypos <- 10^par()$usr[4]
-        else
-          ypos <- par()$usr[4]
 
         # determine position on log(x) scale
+        xpos <- par()$usr[2]
         if (grepl("x", plot.settings$log[[k]], ignore.case = TRUE))
           xpos <- 10^par()$usr[2]
-        else
-          xpos <- par()$usr[2]
       }
 
       ##legend
@@ -802,21 +704,16 @@ plot_RLum.Analysis <- function(
           cex = 0.8 * plot.settings$cex[[k]]
         )
 
-        # revert the overplotting
+        # revert the over plotting
         if (legend.pos == "outside")
           par(xpd = FALSE)
       }
-
     }
 
     ##reset graphic settings
     if (exists("par.default.outside")) {
       par(par.default.outside)
-      rm(par.default.outside)
     }
     par(par.default)
-    rm(par.default)
-
   }
-
 }
