@@ -5,11 +5,16 @@
 #'
 #'@param file [character] (**required**): file name
 #'
+#' @param verbose [logical] (*with default*): enable/disable output to the
+#' terminal.
+#'
+#'@param ... not in use, for compatibility reasons only
+#'
 #'@return [RLum.Data.Image-class] object
 #'
 #'@author Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
 #'
-#'@section Function version: 0.1.1
+#'@section Function version: 0.1.2
 #'
 #'@seealso [tiff::readTIFF], [RLum.Data.Image-class]
 #'
@@ -26,20 +31,31 @@
 #'@md
 #'@export
 read_TIFF2R <- function(
-  file
-){
-# Integrity ---------------------------------------------------------------
-  ## most of the users don't need this import, no need to bother them
-  ## with required libraries
-  if (!requireNamespace("tiff", quietly = TRUE))
-    stop("Importing TIFF files requires the package tiff.\n",
-         "To install this package run 'install.packages('tiff')' in your R console.",
-         call. = FALSE)
+  file,
+  verbose = TRUE,
+  ...
+) {
+  .set_function_name("read_TIFF2R")
+  on.exit(.unset_function_name(), add = TRUE)
+
+  ## Integrity checks -------------------------------------------------------
+
+  .validate_class(file, "character")
+  .validate_length(file, 1)
+  .require_suggested_package("tiff", "Importing TIFF files")
 
   if(!file.exists(file))
-    stop("[read_TIFF2R()] File does not exist or is not readable!", call. = FALSE)
+    .throw_error("File does not exist or is not readable")
+
+  ## Import -----------------------------------------------------------------
+
+  if (verbose) {
+    cat("\n[read_TIFF2R()] Importing ...")
+    cat("\n path: ", dirname(file))
+    cat("\n file: ", .shorten_filename(basename(file)))
+    cat("\n")
+  }
 
-# Import ------------------------------------------------------------------
   ## import
   temp <- tiff::readTIFF(file, all = TRUE, as.is = TRUE)
 
@@ -48,6 +64,4 @@ read_TIFF2R <- function(
 
 # Return ------------------------------------------------------------------
   set_RLum(class = "RLum.Data.Image", data = temp@data)
-
 }
-
