@@ -1,14 +1,16 @@
-#' Verify single grain data sets and check for invalid grains, i.e.
+#' @title Verify single grain data sets and check for invalid grains, i.e.
 #' zero-light level grains
 #'
-#' This function tries to identify automatically zero-light level curves (grains)
+#' @description This function tries to identify automatically zero-light level curves (grains)
 #' from single grain data measurements.
 #'
+#' @details
+#'
 #' **How does the method work?**
 #'
 #' The function compares the expected values (\eqn{E(X)}) and the variance
 #' (\eqn{Var(X)}) of the count values for each curve. Assuming that the
-#' background roughly follows a Poisson distribution the absolute difference
+#' background roughly follows a Poisson distribution, the absolute difference
 #' of both values should be zero or at least around zero as
 #'
 #' \deqn{E(x) = Var(x) = \lambda}
@@ -18,7 +20,7 @@
 #' \deqn{abs(E(x) - Var(x)) >= \Theta}
 #'
 #' With \eqn{\Theta} an arbitrary, user defined, threshold. Values above the
-#' threshold indicating curves comprising a signal.
+#' threshold indicate curves comprising a signal.
 #'
 #' Note: the absolute difference of \eqn{E(X)} and \eqn{Var(x)} instead of the
 #' ratio was chosen as both terms can become 0 which would result in 0 or `Inf`,
@@ -31,25 +33,31 @@
 #' numeric threshold value for the allowed difference between the `mean` and
 #' the `var` of the count values (see details)
 #'
+#' @param use_fft [logical] (*with default*): applies an additional approach based on [stats::fft].
+#' The threshold is fixed and cannot be changed.
+#'
 #' @param cleanup [logical] (*with default*):
-#' if set to `TRUE` curves identified as zero light level curves are
+#' if set to `TRUE`, curves/aliquots identified as zero light level curves/aliquots are
 #' automatically removed. Output is an object as same type as the input, i.e.
 #' either [Risoe.BINfileData-class] or [RLum.Analysis-class]
 #'
 #' @param cleanup_level [character] (*with default*):
-#' selects the level for the cleanup of the input data sets.
+#' selects the level for the clean-up of the input data sets.
 #' Two options are allowed: `"curve"` or `"aliquot"`:
 #'
-#' - If  `"curve"` is selected every single curve marked as `invalid` is removed.
+#' - If  `"curve"` is selected, every single curve marked as `invalid` is removed.
 #' - If `"aliquot"` is selected, curves of one aliquot (grain or disc) can be
 #' marked as invalid, but will not be removed. An aliquot will be only removed
 #' if all curves of this aliquot are marked as invalid.
 #'
 #' @param verbose [logical] (*with default*):
-#' enables or disables the terminal feedback
+#' enable/disables output to the terminal.
 #'
 #' @param plot [logical] (*with default*):
-#' enables or disables the graphical feedback
+#' enable/disable the plot output.
+#'
+#' @param ... further parameters to control the plot output; if selected.
+#' Supported arguments `main`, `ylim`
 #'
 #' @return
 #' The function returns
@@ -76,9 +84,9 @@
 #' **Output variation**
 #'
 #' For `cleanup = TRUE` the same object as the input is returned, but cleaned up
-#' (invalid curves were removed). This means: Either an [Risoe.BINfileData-class]
+#' (invalid curves were removed). This means: Either a [Risoe.BINfileData-class]
 #' or an [RLum.Analysis-class] object is returned in such cases.
-#' An [Risoe.BINfileData-class] object can be exported to a BIN-file by
+#' A [Risoe.BINfileData-class] object can be exported to a BINX-file by
 #' using the function [write_R2BIN].
 #'
 #' @note
@@ -90,9 +98,10 @@
 #'
 #' The function checking for invalid curves works rather robust and it is likely
 #' that Reg0 curves within a SAR cycle are removed as well. Therefore it is
-#' strongly recommended to use the argument `cleanup = TRUE` carefully.
+#' strongly recommended to use the argument `cleanup = TRUE` carefully if
+#' the cleanup works only on curves.
 #'
-#' @section Function version: 0.2.2
+#' @section Function version: 0.2.5
 #'
 #'
 #' @author
@@ -136,14 +145,17 @@
 #' @md
 #' @export
 verify_SingleGrainData <- function(
-  object,
-  threshold = 10,
-  cleanup = FALSE,
-  cleanup_level = 'aliquot',
-  verbose = TRUE,
-  plot = FALSE
-){
-
+    object,
+    threshold = 10,
+    use_fft = FALSE,
+    cleanup = FALSE,
+    cleanup_level = 'aliquot',
+    verbose = TRUE,
+    plot = FALSE,
+    ...
+) {
+  .set_function_name("verify_SingleGrainData")
+  on.exit(.unset_function_name(), add = TRUE)
 
   ##three types of input are allowed:
   ##(1) RisoeBINfileData
@@ -152,130 +164,148 @@ verify_SingleGrainData <- function(
 
   # Self Call -----------------------------------------------------------------------------------
   if(is(object, "list")){
+    if (length(object) == 0)
+      return(set_RLum(class = if (cleanup) "RLum.Analysis" else "RLum.Results"))
 
     results <- .warningCatcher(lapply(1:length(object), function(x) {
       verify_SingleGrainData(
         object = object[[x]],
         threshold = threshold,
+        use_fft = use_fft[1],
         cleanup = cleanup,
         cleanup_level = cleanup_level,
         verbose = verbose,
-        plot = plot
+        plot = plot,
+        main = paste0("Record #",x)
       )
     }))
 
-      ##account for cleanup
-      if(cleanup){
-        return(results)
+    ##account for cleanup
+    if(cleanup[1]){
+      results <- .rm_NULL_elements(.rm_nonRLum(results))
+      if(length(results) == 0)
+        return(NULL)
+      else
+      return(results)
 
-      }else{
-        return(merge_RLum(results))
+    }else{
+      return(merge_RLum(results))
+    }
+  }
 
-      }
+  ## ------------------------------------------------------------------------
+  ## input validation
+  .validate_class(object, c("Risoe.BINfileData", "RLum.Analysis"))
+  cleanup_level <- .validate_args(cleanup_level, c("aliquot", "curve"))
 
+  ## implement Fourier Transform for Frequency Analysis
+  ## inspired by ChatGPT (OpenAI, 2024)
+  .calc_FFT_selection <- function(l, tmp_threshold = threshold/2) {
+    vapply(l, function(x){
+      x <- x[x>0]
+      tmp_power_spectrum <- Mod(stats::fft(x)^2)
+      tmp_mean_power <- mean(tmp_power_spectrum[-1])
+      tmp_dominant_power <- max(tmp_power_spectrum[2:(length(tmp_power_spectrum)/2)])
+      tmp_sel <- tmp_dominant_power > tmp_threshold * tmp_mean_power
+      tmp_sel
+    }, logical(1))
   }
 
   ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   ##RisoeBINfileData
-  if(is(object, "Risoe.BINfileData")){
+  if(inherits(object, "Risoe.BINfileData")){
+    ##run test on DATA slot
+    ##MEAN + SD
+    temp.results_matrix <- t(vapply(X = object@DATA, FUN = function(x){
+      c(mean(x), stats::var(x))
+    }, numeric(2)))
 
-      ##run test on DATA slot
-        ##MEAN + SD
-        temp.results_matrix <- lapply(X = object@DATA, FUN = function(x){
-            c(mean(x), var(x))
 
-        })
+    ##DIFF
+    temp.results_matrix_RATIO <- temp.results_matrix[,2]/temp.results_matrix[,1]
 
-        temp.results_matrix <- do.call(rbind,  temp.results_matrix)
+    ##SEL
+    temp.results_matrix_VALID <-
+      temp.results_matrix_RATIO > threshold &
+      if(use_fft[1]) .calc_FFT_selection(object@DATA) else TRUE
 
-        ##DIFF
-        temp.results_matrix_RATIO <- temp.results_matrix[,2]/temp.results_matrix[,1]
+    ##combine everything to in a data.frame
+    selection <- data.frame(
+      POSITION = object@METADATA$POSITION,
+      GRAIN = object@METADATA$GRAIN,
+      MEAN = temp.results_matrix[, 1],
+      VAR = temp.results_matrix[, 2],
+      RATIO = temp.results_matrix_RATIO,
+      THRESHOLD = rep_len(threshold, length(object@DATA)),
+      VALID = temp.results_matrix_VALID
+    )
 
-        ##SEL
-        temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold
+    ##get unique pairs for POSITION and GRAIN for VALID == TRUE
+    unique_pairs <- unique(
+      selection[selection[["VALID"]], c("POSITION", "GRAIN")])
 
-      ##combine everything to in a data.frame
-        selection <- data.frame(
-          POSITION = object@METADATA$POSITION,
-          GRAIN = object@METADATA$GRAIN,
-          MEAN = temp.results_matrix[, 1],
-          VAR = temp.results_matrix[, 2],
-          RATIO = temp.results_matrix_RATIO,
-          THRESHOLD = rep_len(threshold, length(object@DATA)),
-          VALID = temp.results_matrix_VALID
+    if(cleanup_level == "aliquot"){
+      selection_id <- sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
+        which(
+          .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
+            .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
         )
+      })))
 
-        ##get unique pairs for POSITION and GRAIN for VALID == TRUE
-        unique_pairs <- unique(
-          selection[selection[["VALID"]], c("POSITION", "GRAIN")])
+    }else{
+      ##reduce data to TRUE selection
+      selection_id <- which(selection[["VALID"]])
+    }
 
-        if(cleanup_level == "aliquot"){
-          selection_id <- sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
-            which(
-              .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
-                .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
-            )
+    ##select output on the chosen input
+    if(cleanup){
+      ##selected wanted elements
+      object@DATA <- object@DATA[selection_id]
 
-
-          })))
-
-
-        }else{
-
-         ##reduce data to TRUE selection
-         selection_id <- which(selection[["VALID"]])
-
-        }
-
-
-      ##select output on the chosen input
-      if(cleanup){
-        ##selected wanted elements
-        object@DATA <- object@DATA[selection_id]
+      if(length(object@DATA) > 0) {
         object@METADATA <- object@METADATA[selection_id,]
         object@METADATA$ID <- 1:length(object@DATA)
 
-
         ##print message
-        selection_id <- paste(selection_id, collapse = ", ")
+        selection_id <- .collapse(selection_id, quote = FALSE)
         if(verbose){
           cat(paste0("\n[verify_SingleGrainData()] Risoe.BINfileData object reduced to records: \n", selection_id))
-          cat("\n\n[verify_SingleGrainData()] Risoe.BINfileData object record index reset.")
+          cat("\n\n[verify_SingleGrainData()] Risoe.BINfileData object record index reset.\n")
 
         }
-
-         ##return
-        return_object <- object
-
-      }else{
-        return_object <- set_RLum(
-          class = "RLum.Results",
-          data = list(
-            unique_pairs =  unique_pairs,
-            selection_id = selection_id,
-            selection_full = selection),
-          info = list(call = sys.call())
-        )
-
+      } else {
+        object <- NULL
       }
 
+      ##return
+      return_object <- object
 
-  ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-  ##RLum.Analysis and list with RLum.Analysis objects
-  ## ... and yes it make sense not to mix that up with the code above
+    }else{
+      return_object <- set_RLum(
+        class = "RLum.Results",
+        data = list(
+          unique_pairs =  unique_pairs,
+          selection_id = selection_id,
+          selection_full = selection),
+        info = list(call = sys.call())
+      )
+    }
+
+
+    ##++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+    ##RLum.Analysis and list with RLum.Analysis objects
+    ## ... and yes it make sense not to mix that up with the code above
   }else if(is(object,"RLum.Analysis")){
     ##first extract all count values from all curves
-    object_list <- lapply(get_RLum(object), function(x){
-        ##yes, would work differently, but it is faster
-        x@data[,2]
-
+    object_list <- lapply(object@records, function(x){
+      ##yes, would work differently, but it is faster
+      x@data[,2]
     })
 
     ##MEAN + SD
     temp.results_matrix <- lapply(X = object_list, FUN = function(x){
-      c(mean(x), var(x))
-
-     })
+      c(mean(x), stats::var(x))
+    })
 
     temp.results_matrix <- do.call(rbind,  temp.results_matrix)
 
@@ -283,107 +313,97 @@ verify_SingleGrainData <- function(
     temp.results_matrix_RATIO <- temp.results_matrix[,2]/temp.results_matrix[,1]
 
     ##SEL
-    temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold
+    temp.results_matrix_VALID <- temp.results_matrix_RATIO > threshold &
+      if(use_fft[1]) .calc_FFT_selection(object_list) else TRUE
 
     ##get structure for the RLum.Analysis object
     temp_structure <- structure_RLum(object, fullExtent = TRUE)
 
-      ##now we have two cases, depending on where measurement is coming from
-      if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
+    ##now we have two cases, depending on where measurement is coming from
+    if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
+      ##combine everything to in a data.frame
+      selection <- data.frame(
+        POSITION = temp_structure$info.POSITION,
+        GRAIN = temp_structure$info.GRAIN,
+        MEAN = temp.results_matrix[, 1],
+        VAR = temp.results_matrix[, 2],
+        RATIO = temp.results_matrix_RATIO,
+        THRESHOLD = rep_len(threshold, length(object_list)),
+        VALID = temp.results_matrix_VALID
+      )
 
-        ##combine everything to in a data.frame
-        selection <- data.frame(
-          POSITION = temp_structure$info.POSITION,
-          GRAIN = temp_structure$info.GRAIN,
-          MEAN = temp.results_matrix[, 1],
-          VAR = temp.results_matrix[, 2],
-          RATIO = temp.results_matrix_RATIO,
-          THRESHOLD = rep_len(threshold, length(object_list)),
-          VALID = temp.results_matrix_VALID
-        )
+      ##get unique pairs for POSITION and GRAIN for VALID == TRUE
+      unique_pairs <- unique(
+        selection[selection[["VALID"]], c("POSITION", "GRAIN")])
 
-        ##get unique pairs for POSITION and GRAIN for VALID == TRUE
-        unique_pairs <- unique(
-          selection[selection[["VALID"]], c("POSITION", "GRAIN")])
+    } else if (object@originator == "read_XSYG2R") {
+      ##combine everything to in a data.frame
+      selection <- data.frame(
+        POSITION = if(any(grepl(pattern = "position", names(temp_structure)))){
+          temp_structure$info.position}else{
+            NA
+          },
+        GRAIN = NA,
+        MEAN = temp.results_matrix[, 1],
+        VAR = temp.results_matrix[, 2],
+        RATIO = temp.results_matrix_RATIO,
+        THRESHOLD = rep_len(threshold, length(object_list)),
+        VALID = temp.results_matrix_VALID
+      )
 
-      } else if (object@originator == "read_XSYG2R") {
-        ##combine everything to in a data.frame
-        selection <- data.frame(
-          POSITION = if(any(grepl(pattern = "position", names(temp_structure)))){
-            temp_structure$info.position}else{
-              NA
-            },
-          GRAIN = NA,
-          MEAN = temp.results_matrix[, 1],
-          VAR = temp.results_matrix[, 2],
-          RATIO = temp.results_matrix_RATIO,
-          THRESHOLD = rep_len(threshold, length(object_list)),
-          VALID = temp.results_matrix_VALID
-        )
+      ##get unique pairs for POSITION for VALID == TRUE
+      unique_pairs <- unique(
+        selection[["POSITION"]][selection[["VALID"]]])
 
-        ##get unique pairs for POSITION for VALID == TRUE
-        unique_pairs <- unique(
-          selection[["POSITION"]][selection[["VALID"]]])
-
-      } else{
-
-        stop("[verify_SingleGrainData()] I don't know what to do object 'originator' not supported!",
-             call. = FALSE)
-      }
+    } else{
+      .throw_error("Object originator '", object@originator, "' not supported")
+    }
 
 
-      ##set up cleanup
-      if(cleanup_level == "aliquot") {
-        if (object@originator == "read_XSYG2R") {
+    ##set up cleanup
+    if(cleanup_level == "aliquot") {
+      if (object@originator == "read_XSYG2R") {
 
-          if(!is.na(unique_pairs)){
-            selection_id <-
-              sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
-                which(.subset2(selection, 1) == .subset2(unique_pairs, 1)[x])
-
-
-            })))
-
-          }else{
-           selection_id <- NA
-
-          }
-
-
-        } else if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
+        if(!is.na(unique_pairs)){
           selection_id <-
             sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
-              which(
-                .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
-                  .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
-              )
-
-
+              which(.subset2(selection, 1) == .subset2(unique_pairs, 1)[x])
             })))
 
+        }else{
+          selection_id <- NA
         }
 
-        ##make sure that we do not break subsequent code
-        if(length(selection_id) == 0) selection_id <- NA
-
-
-      } else{
-        ##reduce data to TRUE selection
-        selection_id <- which(selection[["VALID"]])
 
+      } else if (object@originator == "Risoe.BINfileData2RLum.Analysis") {
+        selection_id <-
+          sort(unlist(lapply(1:nrow(unique_pairs), function(x) {
+            which(
+              .subset2(selection, 1) == .subset2(unique_pairs, 1)[x] &
+                .subset2(selection, 2) == .subset2(unique_pairs, 2)[x]
+            )
+          })))
       }
 
+      ##make sure that we do not break subsequent code
+      if(length(selection_id) == 0) selection_id <- NA
+
+    } else{
+      ##reduce data to TRUE selection
+      selection_id <- which(selection[["VALID"]])
+    }
 
     ##return value
     ##select output on the chosen input
-    if(cleanup && !any(is.na(selection_id))){
-
+    if (cleanup[1] && !anyNA(selection_id)) {
       ##print message
-      if(verbose){
-        selection_id_text <- paste(selection_id, collapse = ", ")
-        cat(paste0("\n[verify_SingleGrainData()] RLum.Analysis object reduced to records: ",
-                   selection_id_text))
+      if(verbose && cleanup_level == "curve"){
+        selection_id_text <- .collapse(selection_id, quote = FALSE)
+        if(selection_id_text == "")
+          selection_id_text <- "<none>"
 
+        cat(paste0("[verify_SingleGrainData()] RLum.Analysis object reduced to records: ",
+                   selection_id_text), "\n")
       }
 
       ##selected wanted elements
@@ -400,7 +420,6 @@ verify_SingleGrainData <- function(
         )
 
       } else{
-
         object <- set_RLum(
           class = "RLum.Analysis",
           records = get_RLum(object, record.id = selection_id, drop = FALSE),
@@ -409,18 +428,15 @@ verify_SingleGrainData <- function(
             selection_id = selection_id,
             selection_full = selection)
         )
+      }
 
-     }
 
       ##return
       return_object <- object
 
     }else{
-      if(any(is.na(selection_id))){
-        warning("[verify_SingleGrainData()] selection_id is NA, nothing removed, everything selected!",
-                call. = FALSE)
-
-      }
+      if (anyNA(selection_id))
+        .throw_warning("'selection_id' is NA, everything tagged for removal")
 
       return_object <- set_RLum(
         class = "RLum.Results",
@@ -431,27 +447,32 @@ verify_SingleGrainData <- function(
         info = list(call = sys.call())
       )
 
+      ## cleanup means cleanup
+      if(cleanup[1])
+        return_object <- NULL
     }
-
-
-  }else{
-    stop(paste0("[verify_SingleGrainData()] Input type '", is(object)[1], "' is not allowed for this function!"), call. = FALSE)
-
   }
 
   # Plot ----------------------------------------------------------------------------------------
   if(plot){
+    ##set plot settings
+    plot_settings <-
+      modifyList(x = list(
+        main = "Record selection",
+        ylim = range(c(selection[["RATIO"]], threshold * 1.1))
+      ),
+      val = list(...))
 
     ##plot area
     plot(
       NA,
       NA,
       xlim = c(1,nrow(selection)),
-      ylim = range(selection[["RATIO"]]),
+      ylim = plot_settings$ylim,
       log = "y",
       xlab = "Record index",
       ylab = "Calculated ratio [a.u.]",
-      main = "Record selection"
+      main = plot_settings$main
     )
 
     ##plot points above the threshold
@@ -469,12 +490,11 @@ verify_SingleGrainData <- function(
         " | valid: ", length(which(selection[["VALID"]])),
         " | invalid: ", length(which(!selection[["VALID"]])), ")"),
       cex = 0.9 * par()$cex)
-
   }
 
   # Return --------------------------------------------------------------------------------------
+  if(is.null(return_object))
+    .throw_warning("Verification and cleanup removed all records. NULL returned!")
+
   return(return_object)
-
-
 }
-
