@@ -21,7 +21,7 @@
 #' - `[MAC/LINUX]`: `write_R2BIN("/User/test/Desktop/test.bin")`
 #'
 #' @param version [character] (*optional*):
-#' version number for the output file. If no value is provided the highest
+#' version number for the output file. If no value is provided, the highest
 #' version number from the [Risoe.BINfileData-class] is taken automatically.
 #'
 #' **Note:**
@@ -32,12 +32,15 @@
 #' value to 48. The old position number is appended as comment (e.g., 'OP: 70).
 #' This option accounts for potential compatibility problems with the Analyst software.
 #' It further limits the maximum number of points per curve to 9,999. If a curve contains more
-#' data the curve data got binned using the smallest possible bin width.
+#' data the curve data get binned using the smallest possible bin width.
+#'
+#' @param verbose [logical] (*with default*):
+#' enable/disable output to the terminal.
 #'
 #' @param txtProgressBar [logical] (*with default*):
-#' enables or disables [txtProgressBar].
+#' enable/disable the progress bar. Ignored if `verbose = FALSE`.
 #'
-#' @return Write a binary file.
+#' @return Write a binary file and returns the name and path of the file as [character].
 #'
 #' @note
 #' The function just roughly checks the data structures. The validity of
@@ -49,13 +52,13 @@
 #'
 #' **Warning**
 #'
-#' Although the coding was done carefully it seems that the BIN/BINX-files
+#' Although the coding was done carefully, it seems that the BIN/BINX-files
 #' produced by Risø DA 15/20 TL/OSL readers slightly differ on the byte level.
 #' No obvious differences are observed in the METADATA, however, the
-#' BIN/BINX-file may not fully compatible, at least not similar to the once
+#' BIN/BINX-file may not fully compatible, at least not similar to the ones
 #' directly produced by the Risø readers!
 #'
-#' @section Function version: 0.5.2
+#' @section Function version: 0.5.4
 #'
 #' @author
 #' Sebastian Kreutzer, Institute of Geography, Heidelberg University (Germany)
@@ -73,7 +76,7 @@
 #' @keywords IO
 #'
 #' @examples
-#' ##load exampled dataset
+#' ##load example dataset
 #' file <- system.file("extdata/BINfile_V8.binx", package = "Luminescence")
 #' temp <- read_BIN2R(file)
 #'
@@ -91,36 +94,34 @@ write_R2BIN <- function(
   file,
   version,
   compatibility.mode = FALSE,
+  verbose = TRUE,
   txtProgressBar = TRUE
-){
+) {
+  .set_function_name("write_R2BIN")
+  on.exit(.unset_function_name(), add = TRUE)
 
   # Config ------------------------------------------------------------------
-
   ##set supported BIN format version
   VERSION.supported <- as.raw(c(3, 4, 5, 6, 7, 8))
 
-  # Check integrity ---------------------------------------------------------
+  ## Integrity tests --------------------------------------------------------
 
-  ##check if input object is of type 'Risoe.BINfileData'
-  if(is(object, "Risoe.BINfileData") == FALSE){
-    stop("[write_R2BIN()] Input object is not of type Risoe.BINfileData!", call. = FALSE)
+  .validate_class(object, "Risoe.BINfileData")
+  .validate_class(file, "character")
+  .validate_logical_scalar(verbose)
+  .validate_logical_scalar(txtProgressBar)
 
-  }
-
-  ##check if it fulfils the latest definition ...
+  ## check if it fulfills the latest definition ...
   if(ncol(object@METADATA) != ncol(set_Risoe.BINfileData()@METADATA)){
-    stop("[write_R2BIN()] The number of columns in your slot 'METADATA' does not fit to the latest definition. What you are probably trying to do is to export a Risoe.BINfileData object you generated by your own or you imported with an old package version some time ago. Please re-import the BIN-file using the function read_BIN2R().", call. = FALSE)
-
-  }
-
-  ##check if input file is of type 'character'
-  if(is(file, "character") == FALSE){
-    stop("[write_R2BIN()] argument 'file' has to be of type character!", call. = FALSE)
-
+    .throw_error("Your Risoe.BINfileData object is not compatible with the ",
+                 "latest specification of this S4-class object. You are ",
+                 "probably trying to export a Risoe.BINfileData from your ",
+                 "workspace you produced manually or with an old version. ",
+                 "Please re-import the BIN-file using function read_BIN2R().")
   }
 
   # Check Risoe.BINfileData Struture ----------------------------------------
-  ##check wether the BIN-file DATA slot contains more than 9999 records; needs to be run all the time
+  ##check whether the BIN-file DATA slot contains more than 9999 records; needs to be run all the time
   temp_check <- vapply(object@DATA, function(x){
     if(length(x) > 9999){
       TRUE
@@ -132,9 +133,8 @@ write_R2BIN <- function(
 
   ##force compatibility
   if(compatibility.mode && any(temp_check)){
-
-    ##drop warning
-    warning("[write_R2BIN()] Compatibility mode selected: Some data sets are longer than 9,999 points and will be binned!", call. = FALSE)
+    .throw_warning("Compatibility mode selected: some data sets have ",
+                   "more than 9,999 points and will be binned")
 
     ##BIN data to reduce amount of data if the BIN-file is too long
     object@DATA <- lapply(object@DATA, function(x){
@@ -142,10 +142,9 @@ write_R2BIN <- function(
         ##we want to have a minimum binning (smallest number possible)
         bin_width <- ceiling(length(x)/9999)
 
-        ##it should be symatric, thus, remove values
+        ##it should be symmetric, thus, remove values
         if((length(x)/bin_width)%%2 != 0){
           x <- x[-length(x)]
-
         }
 
         ##create matrix and return
@@ -153,16 +152,14 @@ write_R2BIN <- function(
 
       }else{
         x
-
       }
-
     })
 
     ##reset temp_check
     temp_check <- FALSE
 
     ##get new number of points
-    temp_NPOINTS <- sapply(object@DATA, length)
+    temp_NPOINTS <- lengths(object@DATA)
 
     ##correct LENGTH
     object@METADATA[["LENGTH"]] <- object@METADATA[["LENGTH"]] - (4 * object@METADATA[["NPOINTS"]]) + (temp_NPOINTS * 4)
@@ -175,27 +172,30 @@ write_R2BIN <- function(
 
     ##write comment
     object@METADATA[["COMMENT"]] <- paste(object@METADATA[["COMMENT"]], " - binned")
-
   }
 
   if(any(temp_check))
-    stop(paste("[write_R2BIN()]", length(which(temp_check)), " out of ",length(temp_check), "records contain more than 9,999 data points. This violates the BIN/BINX-file definition!"), call. = FALSE)
+    .throw_error(length(which(temp_check)), " out of ", length(temp_check),
+                 " records contain more than 9,999 data points. ",
+                 "This violates the BIN/BINX-file definition")
 
   ##remove
   rm(temp_check)
 
+  ## UTF-8 conversion
+  object@METADATA[["SAMPLE"]] <- base::iconv(object@METADATA[["SAMPLE"]], "latin1", "ASCII", sub="_")
+  object@METADATA[["COMMENT"]] <- base::iconv(object@METADATA[["COMMENT"]], "latin1", "ASCII", sub="_")
+  object@METADATA[["FNAME"]] <- base::iconv(object@METADATA[["FNAME"]], "latin1", "ASCII", sub="_")
+  object@METADATA[["USER"]] <- base::iconv(object@METADATA[["USER"]], "latin1", "ASCII", sub="_")
+  object@METADATA[["SEQUENCE"]] <- base::iconv(object@METADATA[["SEQUENCE"]], "latin1", "ASCII", sub="_")
 
   ##VERSION
-
   ##If missing version argument set to the highest value
   if(missing(version)){
-
     version <- as.raw(max(as.numeric(object@METADATA[,"VERSION"])))
     version.original <- version
 
-
   }else{
-
     version.original <- as.raw(max(as.numeric(object@METADATA[,"VERSION"])))
     version <- as.raw(version)
     object@METADATA[["VERSION"]] <- version
@@ -221,15 +221,7 @@ write_R2BIN <- function(
         if(x == 1) 0 else header.stepping + 4 * object@METADATA[x-1,"NPOINTS"]
 
       }, numeric(1))
-
     }
-
-  }
-
-  ##check whether this file can be exported without problems due to the latest specifications
-  if(ncol(object@METADATA) != 80){
-    stop("[write_R2BIN()] Your Risoe.BINfileData object seems not to be compatible with the latest specification of this S4-class object. You are probably trying to export a Risoe.BINfileData from your workspace you produced manually or with an old version.", call. = FALSE)
-
   }
 
   ##Check if the BINfile object contains of unsupported versions
@@ -237,11 +229,10 @@ write_R2BIN <- function(
        version %in% VERSION.supported == FALSE){
 
     ##show error message
-    error.text <- paste("[write_R2BIN()] Writing BIN-files in format version (",
-                        object@METADATA[1,"VERSION"],") is currently not supported!
-                        Supported version numbers are: ",
-                        paste(VERSION.supported,collapse=", "),".",sep="")
-    stop(error.text)
+    .throw_error("Writing BIN-files in format version (",
+                 object@METADATA[1, "VERSION"], ") is currently not supported, ",
+                 "supported version numbers are: ",
+                 .collapse(VERSION.supported))
   }
 
   ##CHECK file name for version == 06 it has to be *.binx and correct for it
@@ -253,7 +244,6 @@ write_R2BIN <- function(
     if(temp.file.name[length(temp.file.name)]=="bin"){
       temp.file.name[length(temp.file.name)] <- "binx"
       file <- paste(temp.file.name, collapse=".")
-
     }
   }
 
@@ -261,25 +251,22 @@ write_R2BIN <- function(
   ##SEQUENCE
   if (suppressWarnings(max(nchar(as.character(object@METADATA[,"SEQUENCE"]), type =
                                  "bytes"), na.rm = TRUE)) > 8) {
-    stop("[write_R2BIN()] Value in 'SEQUENCE' exceed storage limit!")
-
+    .throw_error("Value in 'SEQUENCE' exceeds storage limit")
   }
 
   ##USER
   if (suppressWarnings(max(nchar(as.character(object@METADATA[,"USER"]), type =
                                  "bytes"), na.rm = TRUE)) > 8) {
-    stop("[write_R2BIN()] 'USER' exceed storage limit!")
-
+    .throw_error("'USER' exceeds storage limit")
   }
 
   ##SAMPLE
   if (suppressWarnings(max(nchar(as.character(object@METADATA[,"SAMPLE"]), type =
                                  "bytes"), na.rm = TRUE)) > 20) {
-    stop("[write_R2BIN()] 'SAMPLE' exceed storage limit!")
-
+    .throw_error("'SAMPLE' exceeds storage limit")
   }
 
-  ##enables compatibility to the Analyst as the the max value for POSITION becomes 48
+  ## enables compatibility to the Analyst as the max value for POSITION becomes 48
   if(compatibility.mode){
     ##just do if position values > 48
     if(max(object@METADATA[,"POSITION"])>48){
@@ -302,17 +289,13 @@ write_R2BIN <- function(
 
         object@METADATA[object@METADATA[,"POSITION"] == temp.POSITION48.unique[i],"POSITION"] <-
           temp.POSITION48.new[i]
-
       }
-
     }
-
   }
 
   ##COMMENT
   if(max(nchar(as.character(object@METADATA[,"COMMENT"]), type="bytes"))>80){
-    stop("[write_R2BIN()] 'COMMENT' exceeds storage limit!", call. = FALSE)
-
+    .throw_error("'COMMENT' exceeds storage limit")
   }
 
   # Translation Matrices -----------------------------------------------------
@@ -345,7 +328,6 @@ write_R2BIN <- function(
 
     object@METADATA[,"LTYPE"]<- sapply(1:length(object@METADATA[,"LTYPE"]),function(x){
       as.integer(LTYPE.TranslationMatrix[object@METADATA[x,"LTYPE"]==LTYPE.TranslationMatrix[,2],1])
-
     })
   }
 
@@ -367,7 +349,6 @@ write_R2BIN <- function(
 
       as.integer(LIGHTSOURCE.TranslationMatrix[
         object@METADATA[x,"LIGHTSOURCE"]==LIGHTSOURCE.TranslationMatrix[,2],1])
-
     })}
 
   ##TIME
@@ -377,7 +358,6 @@ write_R2BIN <- function(
 
     }else{
       as.character(gsub(":","",object@METADATA[["TIME"]][x]))
-
     }
 
   }, character(1))
@@ -388,13 +368,23 @@ write_R2BIN <- function(
   object@METADATA[,"TAG"] <- ifelse(object@METADATA[,"SEL"] == TRUE, 1, 0)
 
   # SET FILE AND VALUES -----------------------------------------------------
-  con<-file(file, "wb")
+  con <- file(file, "wb")
+  on.exit(close(con), add = TRUE)
 
   ##get records
   n.records <- length(object@METADATA[,"ID"])
 
-  ##output
-  message(paste0("[write_R2BIN()]\n\t >> ",file))
+  ## don't show the progress bar if not verbose
+  if (!verbose)
+    txtProgressBar <- FALSE
+
+  if (verbose) {
+    cat("\n[write_R2BIN()] Exporting ...")
+    cat("\n path: ", dirname(file))
+    cat("\n file: ", .shorten_filename(basename(file)))
+    cat("\n n_rec:", n.records)
+    cat("\n")
+  }
 
   ##set progressbar
   if(txtProgressBar)
@@ -478,7 +468,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##DATE
@@ -597,7 +586,6 @@ write_R2BIN <- function(
 
         SAMPLE_SIZE <- as.integer(2)
         object@METADATA[ID,"SAMPLE"] <- "  "
-
       }
 
       writeBin(SAMPLE_SIZE,
@@ -645,7 +633,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##LIGHTSOURCE, SET, TAG
@@ -687,7 +674,6 @@ write_R2BIN <- function(
                    con,
                    size = 1,
                    endian="little")
-
         }
 
         ##ONTIME, OFFTIME
@@ -724,7 +710,6 @@ write_R2BIN <- function(
                    con,
                    size = 1,
                    endian="little")
-
         }
 
       } else {
@@ -740,7 +725,6 @@ write_R2BIN <- function(
                    con,
                    size = 1,
                    endian="little")
-
         }
 
         ##CURVENO
@@ -796,7 +780,6 @@ write_R2BIN <- function(
                    con,
                    size = 1,
                    endian="little")
-
         }
       }
       ##DPOINTS
@@ -809,7 +792,6 @@ write_R2BIN <- function(
       ID <- ID + 1
       ##update progress bar
       if(txtProgressBar) setTxtProgressBar(pb, ID)
-
     }
   }
   ## ====================================================
@@ -864,7 +846,6 @@ write_R2BIN <- function(
 
         SAMPLE_SIZE <- as.integer(2)
         object@METADATA[ID,"SAMPLE"] <- "  "
-
       }
 
       writeBin(SAMPLE_SIZE,
@@ -894,7 +875,6 @@ write_R2BIN <- function(
 
         COMMENT_SIZE <- as.integer(2)
         object@METADATA[ID,"COMMENT"] <- "  "
-
       }
 
       writeBin(COMMENT_SIZE,
@@ -913,7 +893,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##Instrument and sequence characteristics
@@ -927,7 +906,7 @@ write_R2BIN <- function(
       FNAME_SIZE <- as.integer(nchar(as.character(object@METADATA[ID,"FNAME"]), type="bytes"))
 
         ##correct for case that this is of 0 length
-        if(length(FNAME_SIZE) == 0){FNAME_SIZE <- as.integer(0)}
+        if(is.na(FNAME_SIZE) || length(FNAME_SIZE) == 0){FNAME_SIZE <- as.integer(0)}
 
       writeBin(FNAME_SIZE,
                con,
@@ -949,7 +928,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##USER
@@ -972,7 +950,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##TIME
@@ -994,7 +971,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##DATE
@@ -1063,7 +1039,6 @@ write_R2BIN <- function(
                  con,
                  size = 1,
                  endian="little")
-
       }
 
       ##Measurement characteristics
@@ -1232,7 +1207,6 @@ write_R2BIN <- function(
         }
 
       }else{
-
         ##DETECTOR_ID
         writeBin(as.integer(object@METADATA[ID,"DETECTOR_ID"]),
                  con,
@@ -1318,17 +1292,17 @@ write_R2BIN <- function(
 
       ##update progress bar
       if(txtProgressBar)  setTxtProgressBar(pb, ID)
-
     }
   }
 
-  # ##close con
-  close(con)
   #
   # ##close
   if(txtProgressBar) close(pb)
 
   ##output
-  message(paste0("\t >> ",ID-1,"records have been written successfully!\n\n"))
+  if (verbose)
+    message("\t >> ", ID - 1, " records written successfully\n")
 
+  ## return path
+  invisible(file)
 }
