@@ -10,12 +10,21 @@
 //    a vertical sliding of the curve
 // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
+#include <sstream>
 #include <RcppArmadillo.h>
 #include <RcppArmadilloExtensions/sample.h>
 
 // [[Rcpp::depends(RcppArmadillo)]]
 using namespace Rcpp;
 
+// print a 2-element vector horizontally
+std::string fmt_vec(const arma::vec& vec) {
+  std::stringstream ss;
+  ss << " [" << std::setw(11) << vec[0]
+     << ", " << std::setw(11) << vec[1] << "]";
+  return ss.str();
+}
+
 // [[Rcpp::export("src_analyse_IRSARRF_SRS")]]
 RcppExport SEXP analyse_IRSARRF_SRS(arma::vec values_regenerated_limited,
                                     arma::vec values_natural_limited,
@@ -29,101 +38,85 @@ RcppExport SEXP analyse_IRSARRF_SRS(arma::vec values_regenerated_limited,
     stop("[:::src_analyse_IRSAR_SRS()] 'vslide_range' exceeded maximum size (1e+07)!");
   }
 
-
-  //pre-define variables
-  arma::vec residuals(values_natural_limited.size());
-  arma::vec results(values_regenerated_limited.size() - values_natural_limited.size());
-  arma::vec results_vector_min_MC(n_MC);
-
+  const size_t nat_size = values_natural_limited.size();
+  const size_t res_size = values_regenerated_limited.size() - nat_size;
+  const size_t two_size = 2;
 
   //variables for the algorithm
-  int v_length;
-  int v_index;
-  arma::vec v_leftright(2); //the virtual vector
-  arma::vec t_leftright(2); //the test points
-  arma::vec c_leftright(2); //the calculation
-
-  //(1) calculate sum of the squared residuals
-  // this will be used to find the best fit of the curves (which is the minimum)
-
-  //initialise values
-  v_length = vslide_range.size();
-  v_index = 0;
+  int v_length = vslide_range.size();
+  int v_index = 0;
+  arma::vec results(res_size);
+  arma::vec::fixed<two_size> v_leftright; // the virtual vector
+  arma::vec::fixed<two_size> t_leftright; // the test points
+  arma::vec::fixed<two_size> c_leftright; // the calculation
 
+  // initialise values: at the beginning, the virtual vector includes all
+  // points in vslide_range
   v_leftright[0] = 0;
   v_leftright[1] = vslide_range.size() - 1;
 
-  if(v_length == 1){
-    t_leftright[0] = 0;
-    t_leftright[1] = 0;
-
-  }else{
+  if (v_length > 1) {
+    // select the test region to be the central third of the virtual vector
     t_leftright[0] = v_length/3;
     t_leftright[1] = 2 * v_length/3;
-
   }
 
   //***TRACE****
   if(trace == true){
     Rcout << "\n\n [:::src_analyse_IRSAR_SRS()]";
-    Rcout << "\n\n--- Inititalisation --- \n ";
-    Rcout << "\n >> v_leftright: " << v_leftright;
-    Rcout << "\n >> t_leftright: " << t_leftright;
+    Rcout << "\n\n --- Initialisation ---\n";
+    Rcout << "\n >> v_leftright: " << fmt_vec(v_leftright);
+    Rcout << "\n >> t_leftright: " << fmt_vec(t_leftright);
     Rcout << "\n\n --- Optimisation --- \n ";
-    Rcout << "\n ---------------------------------------------------------------------------------------------------------";
-    Rcout << "\n v_length \t\t v_leftright \t\t  c_leftright  \t\t\t\t absolute offset";
-    Rcout << "\n ---------------------------------------------------------------------------------------------------------";
-
+    Rcout << "\n --------------------------------------------------------------------------------------------------------------------------";
+    Rcout << "\n  v_length\t\t\tv_leftright\t\t\tc_leftright\t\t\tt_leftright\tabs.offset";
+    Rcout << "\n --------------------------------------------------------------------------------------------------------------------------";
   }
 
+  //(1) calculate sum of the squared residuals
+  // this will be used to find the best fit of the curves (which is the minimum)
+
   //start loop
   do {
 
-    for (int t=0;t<static_cast<int>(t_leftright.size()); t++){
+    for (size_t t = 0; t < two_size; ++t) {
 
       //HORIZONTAL SLIDING CORE -------------------------------------------------------------(start)
+
+      results.zeros();
+      auto curr_vslide_range = vslide_range[t_leftright[t]];
+
       //slide the curves against each other
-      for (int i=0; i<static_cast<int>(results.size()); ++i){
-
-        //calculate squared residuals along one curve
-        for (int j=0; j<static_cast<int>(values_natural_limited.size()); ++j){
-          residuals[j] = pow((values_regenerated_limited[j+i] - (values_natural_limited[j] + vslide_range[t_leftright[t]])),2);
+      for (unsigned int i = 0u; i < res_size; ++i) {
+        // calculate the sum of squared residuals along one curve
 
+        for (unsigned int j = 0u; j < nat_size; ++j) {
+          double residual = values_regenerated_limited[j + i] -
+            (values_natural_limited[j] + curr_vslide_range);
+          results[i] += residual * residual;
         }
-
-        //sum results and fill the results vector
-        results[i] = sum(residuals);
-
       }
 
       //HORIZONTAL SLIDING CORE ---------------------------------------------------------------(end)
       c_leftright[t] = min(results);
-
-
     }
-    //compare results and re-initialise variables
 
-    if(c_leftright[0] < c_leftright[1]){
-      v_index = v_leftright[0]; //set index to left test index
-
-      //update vector window (the left remains the same)
-      v_leftright[1] = t_leftright[1];
-
-      //update window length
-      v_length = v_leftright[1] - v_leftright[0];
-
-    }else if (c_leftright[0] > c_leftright[1]){
-      v_index = v_leftright[1]; //set index to right test index
-
-      //update vector window (the right remains the same this time)
-      v_leftright[0] = t_leftright[0];
-
-      //update window length
-      v_length = v_leftright[1] - v_leftright[0];
-
-    }else{
+    // compare results and update variables
+    auto diff = c_leftright[0] - c_leftright[1];
+    if (diff == 0) {
       v_length = 1;
+    } else {
+      // find on which side the minimum is, the maximum is on the other side
+      auto idx_min = diff < 0 ? 0 : 1, idx_max = 1 - idx_min;
 
+      // set index to where the minimum is
+      v_index = v_leftright[idx_min];
+
+      // update vector window
+      v_leftright[idx_max] = t_leftright[idx_max];
+
+      //update window length
+      v_length = v_leftright[1] - v_leftright[0];
     }
 
     //update test point index
@@ -132,19 +125,22 @@ RcppExport SEXP analyse_IRSARRF_SRS(arma::vec values_regenerated_limited,
 
     //***TRACE****
     if(trace == true){
-      Rcout << "\n " << v_length << " \t\t\t " << v_leftright << " \t\t " << c_leftright << " \t\t\t " << vslide_range[v_index];
-
+      Rcout <<
+        "\n" << std::setw(10) << v_length <<
+        "\t" << fmt_vec(v_leftright) <<
+        "\t" << fmt_vec(c_leftright) <<
+        "\t" << fmt_vec(t_leftright) <<
+        "\t" << std::setw(10) << vslide_range[v_index];
     }
 
   } while (v_length > 1);
 
   //***TRACE****
   if(trace == true){
-    Rcout << "\n ---------------------------------------------------------------------------------------------------------";
+    Rcout << "\n ------------------------------------------------------------------------------------------";
     Rcout << "\n >> SRS minimum: \t\t " << c_leftright[0];
     Rcout << "\n >> Vertical offset index: \t " << v_index + 1;
     Rcout << "\n >> Vertical offset absolute: \t " << vslide_range[v_index] << "\n\n";
-
   }
 
   //(2) error calculation
@@ -155,9 +151,9 @@ RcppExport SEXP analyse_IRSARRF_SRS(arma::vec values_regenerated_limited,
   //function sample() in R, but faster)
   //http://gallery.rcpp.org/articles/using-the-Rcpp-based-sample-implementation
 
-
   //this follows the way described in Frouin et al., 2017 ... still ...
-  for (int i=0; i<static_cast<int>(results_vector_min_MC.size()); ++i){
+  arma::vec results_vector_min_MC(n_MC);
+  for (int i = 0; i < n_MC; ++i) {
     results_vector_min_MC[i] = min(
       RcppArmadillo::sample(
         results,
