@@ -7,27 +7,38 @@
 
 test_that("Check github_commits()", {
   testthat::skip_on_cran()
-  local_edition(3)
 
   response <- tryCatch(github_commits(), error = function(e) return(e))
 
   if (inherits(response, "error")){
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
   } else {
     expect_s3_class(response, "data.frame")
   }
 
-  rm(response)
+  tryCatch(expect_error(github_commits(branch = "error"),
+                        "Branch 'error' does not exist"),
+           error = function(e) {
+             expect_output(print(e), "returned status code")
+           })
 })
 
 test_that("Check github_branches()", {
   testthat::skip_on_cran()
-  local_edition(3)
+
+  expect_error(github_branches(user = NA),
+               "'user' should be of class 'character'")
+  expect_error(github_branches(user = letters),
+               "'user' should have length 1")
+  expect_error(github_branches(repo = NA),
+               "'repo' should be of class 'character'")
+  expect_error(github_branches(repo = letters),
+               "'repo' should have length 1")
 
   response <- tryCatch(github_branches(), error = function(e) return(e))
 
   if (inherits(response, "error")) {
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
  }else {
     expect_s3_class(response, "data.frame")
  }
@@ -37,12 +48,18 @@ test_that("Check github_branches()", {
 
 test_that("Check github_issues()", {
   testthat::skip_on_cran()
-  local_edition(3)
 
+  expect_error(github_issues(user = NA),
+               "'user' should be of class 'character'")
+  expect_error(github_issues(repo = NA),
+               "'repo' should be of class 'character'")
+
+  SW({
   response <- tryCatch(github_issues(), error = function(e) return(e))
+  })
 
   if (inherits(response, "error")){
-    expect_output(print(response), regexp = "status code 403")
+    expect_output(print(response), regexp = "returned status code")
   }else{
     expect_type(response, "list")
   }
