@@ -1,29 +1,58 @@
-test_that("test pure success of the plotting without warning or error", {
+## load data
+data(ExampleData.XSYG, envir = environment())
+
+test_that("input validation", {
+  testthat::skip_on_cran()
+
+  expect_error(plot_RLum.Data.Spectrum("error"),
+               "'object' should be of class 'RLum.Data.Spectrum' or 'matrix'")
+  expect_error(plot_RLum.Data.Spectrum(set_RLum("RLum.Data.Spectrum")),
+               "'object' contains no data")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, plot.type = "error"),
+               "'plot.type' should be one of 'contour', 'persp', 'single'")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, norm = "error"),
+               "'norm' should be one of 'min', 'max' or NULL")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, bg.spectrum = "error"),
+               "'bg.spectrum' should be of class 'RLum.Data.Spectrum' or 'matrix'")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, bin.rows = 1.7),
+               "'bin.rows' should be a positive integer scalar")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, bin.cols = 0),
+               "'bin.cols' should be a positive integer scalar")
+
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, xlim = c(0, 100)),
+      "No data left after applying 'xlim' and 'ylim'")
+  expect_error(plot_RLum.Data.Spectrum(TL.Spectrum, ylim = c(5, 10)),
+      "No data left after applying 'xlim' and 'ylim'")
+
+  expect_warning(plot_RLum.Data.Spectrum(TL.Spectrum, bg.channels = -2),
+                 "'bg.channels' out of range")
+})
+
+test_that("check functionality", {
   testthat::skip_on_cran()
-  local_edition(3)
 
     ##RLum.Data.Spectrum -------
-    data(ExampleData.XSYG, envir = environment())
     m <- TL.Spectrum@data
     bg.spectrum <- set_RLum(class = "RLum.Data.Spectrum", data = TL.Spectrum@data[,15:16, drop = FALSE])
 
-    ##crash the function with wrong input
-    expect_error(plot_RLum.Data.Spectrum(object = "test"),
-                 regexp = "Input object neither of class 'RLum.Data.Spectrum' nor 'matrix'.")
-
     ##try a matrix as input
-    expect_message(plot_RLum.Data.Spectrum(object = m),
-                   regexp = "Input has been converted to a RLum.Data.Spectrum object using set_RLum()")
+    expect_message(plot_RLum.Data.Spectrum(object = m, xaxis.energy = TRUE),
+                   "Input has been converted to a 'RLum.Data.Spectrum' object")
 
     ##remove rownames and column names
     rownames(m) <- NULL
     colnames(m) <- NULL
     expect_message(plot_RLum.Data.Spectrum(object = m),
-        regexp = "Input has been converted to a RLum.Data.Spectrum object using set_RLum()")
+        regexp = "Input has been converted to a 'RLum.Data.Spectrum' object")
 
+    ## test duplicated column names
+    t <- TL.Spectrum
+    colnames(t@data) <- rep(50, ncol(t@data))
+    expect_warning(plot_RLum.Data.Spectrum(t),
+                   "Duplicated column names found")
 
     ##standard plot with some settings
-    expect_silent(plot(
+    expect_silent(plot_RLum.Data.Spectrum(
       TL.Spectrum,
       plot.type = "contour",
       main = "Test",
@@ -73,7 +102,7 @@ test_that("test pure success of the plotting without warning or error", {
       bg.spectrum = bg.spectrum,
       bin.rows = 10,
       bin.cols = 1
-    ), "\\[plot_RLum.Data.Spectrum\\(\\)\\] 6 channel\\(s\\) removed due to row \\(wavelength\\) binning.")
+    ), "6 channels removed due to row \\(wavelength\\) binning")
 
     ## check output and limit counts
     expect_type(suppressWarnings(plot_RLum.Data.Spectrum(
@@ -86,6 +115,18 @@ test_that("test pure success of the plotting without warning or error", {
       bin.cols = 1
     )), "double")
 
+    expect_warning(plot_RLum.Data.Spectrum(
+      TL.Spectrum,
+      plot.type = "persp",
+      xlim = c(310, 750),
+      limit_counts = 2,
+      lphi = 20,
+      ltheta = -15,
+      ticktype = "simple",
+      bin.rows = 1,
+      bin.cols = 1
+    ), "Lowest count value is larger than the set count threshold")
+
     ## check our axes
     expect_type(suppressWarnings(plot_RLum.Data.Spectrum(
     TL.Spectrum,
@@ -170,6 +211,18 @@ test_that("test pure success of the plotting without warning or error", {
         bin.cols = 1,
         contour = FALSE)))
 
+  expect_silent(suppressWarnings(
+      plot_RLum.Data.Spectrum(
+        TL.Spectrum,
+        plot.type = "transect",
+        xlim = c(310, 750),
+        ylim = c(0, 350),
+#        bin.rows = 50,
+        ## bin.cols = 1,
+        zlim = c(0, 1e6),
+        ylab = "Counts [1 / summed channels]",
+        contour = TRUE)))
+
     ## plot: single ------------
     expect_silent(suppressWarnings(
       plot_RLum.Data.Spectrum(
@@ -191,6 +244,7 @@ test_that("test pure success of the plotting without warning or error", {
         ylim = c(0, 300),
         bin.rows = 10,
         bin.cols = 6,
+        col = 1,
         contour = FALSE)))
 
     ### plot_image: colour changes -------
@@ -243,21 +297,44 @@ test_that("test pure success of the plotting without warning or error", {
         bin.rows = 10,
         bin.cols = 1,
         type = "contour",
+        norm = "max",
         showscale = TRUE
       )
     ))
 
+  ## more coverage
+  plot_RLum.Data.Spectrum(TL.Spectrum, plot.type = "multiple.lines",
+                          phi = 15, theta = -30, r = 10, log = "xyz",
+                          shade = 0.4, expand = 0.5, border = 1,
+                          axes = FALSE, norm = "min", col = 2, zlim = c(0, 2))
 
-    ##create more error
-    expect_error(plot(
-      TL.Spectrum,
-      plot.type = "contour",
-      xlim = c(310, 750),
-      ylim = c(0, 300),
-      bin.cols = 0
-    ))
-
-
-
+  expect_message(expect_null(
+      plot_RLum.Data.Spectrum(TL.Spectrum,
+                              plot.type = "multiple.lines",
+                              xlim = c(1.4, 4), ylim = c(0, 300),
+                              bg.spectrum = bg.spectrum@data * 2,
+                              xaxis.energy = TRUE)),
+      "Error: After background subtraction all counts are negative")
 })
 
+test_that("regression tests", {
+  testthat::skip_on_cran()
+
+  ## issue 415
+  expect_silent(plot_RLum.Data.Spectrum(
+      TL.Spectrum,
+      ylim = c(0, 100),
+      bin.cols = 3))
+  expect_warning(plot_RLum.Data.Spectrum(
+      TL.Spectrum,
+      ylim = c(0, 100),
+      bin.cols = 8),
+      "Single column matrix: plot.type has been automatically reset to")
+  expect_silent(plot_RLum.Data.Spectrum(
+      TL.Spectrum,
+      bin.rows = 600))
+  expect_message(expect_null(plot_RLum.Data.Spectrum(
+      TL.Spectrum,
+      bin.rows = 2000)),
+      "Insufficient data for plotting, NULL returned")
+})
