@@ -1,6 +1,5 @@
 test_that("Check subset_SingleGrain", {
   testthat::skip_on_cran()
-  local_edition(3)
 
   ## get example ready
   data(ExampleData.BINfileData, envir = environment())
@@ -9,9 +8,9 @@ test_that("Check subset_SingleGrain", {
   selection <- data.frame(POSITION = c(1,5,7), GRAIN = c(0,0,0))
 
   ## crash function
-  expect_error(
-    object = subset_SingleGrainData("error"),
-    regexp = "\\[subset\\_SingleGrainData\\(\\)\\] Only Risoe.BINfileData-class objects are allowed as input!")
+  expect_error(subset_SingleGrainData("error"),
+               "[subset_SingleGrainData()] 'object' should be of class 'Risoe.BINfileData'",
+               fixed = TRUE)
 
   ## standard run
   expect_s4_class(subset_SingleGrainData(object = CWOSL.SAR.Data, selection = selection), "Risoe.BINfileData")
